@@ -18,6 +18,9 @@
  */
 
 #ifdef HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "config.h"
 #endif
 
@@ -83,13 +86,8 @@ GST_PAD_TEMPLATE_FACTORY (sink_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "a52dec_sink",
-    "audio/a52",
+    "audio/x-ac3",
      NULL
-  ), 
-  GST_CAPS_NEW (
-    "ac3dec_sink", 
-    "audio/ac3", 
-    NULL
   )
 );
 
@@ -99,9 +97,7 @@ GST_PAD_TEMPLATE_FACTORY (src_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "a52dec_src",
-      "audio/raw",
-       "format", 	GST_PROPS_STRING ("int"),
-       "law", 		GST_PROPS_INT (0),
+      "audio/x-raw-int",
        "endianness", 	GST_PROPS_INT (G_BYTE_ORDER),
        "signed", 	GST_PROPS_BOOLEAN (TRUE),
        "width",	 	GST_PROPS_INT (16),
@@ -355,7 +351,7 @@ gst_a52dec_push (GstPad * srcpad, int flags, sample_t * _samples, gint64 timesta
   GST_BUFFER_TIMESTAMP (buf) = timestamp;
   float_to_int (samples, (int16_t *) GST_BUFFER_DATA (buf), flags);
 
-  gst_pad_push (srcpad, buf);
+  gst_pad_push (srcpad, GST_DATA (buf));
 
   return 0;
 }
@@ -365,13 +361,11 @@ gst_a52dec_push (GstPad * srcpad, int flags, sample_t * _samples, gint64 timesta
 static void
 gst_a52dec_reneg (GstPad * pad, int channels, int rate)
 {
-  GST_INFO (GST_CAT_PLUGIN_INFO, "a52dec: reneg channels:%d rate:%d\n", channels, rate);
+  GST_INFO ( "a52dec: reneg channels:%d rate:%d\n", channels, rate);
 
   if (gst_pad_try_set_caps (pad, 
 		  GST_CAPS_NEW ("a52dec_src_caps",
-			        "audio/raw",
-				  "format", 	GST_PROPS_STRING ("int"),
-				  "law", 	GST_PROPS_INT (0),
+			        "audio/x-raw-int",
 				  "endianness",	GST_PROPS_INT (G_BYTE_ORDER),
 				  "signed", 	GST_PROPS_BOOLEAN (TRUE),
 				  "width", 	GST_PROPS_INT (16),
@@ -503,7 +497,7 @@ gst_a52dec_loop (GstElement *element)
   }
 
   if (need_reneg == TRUE) {
-    GST_DEBUG (0, "a52dec reneg: sample_rate:%d stream_chans:%d using_chans:%d\n",
+    GST_DEBUG ("a52dec reneg: sample_rate:%d stream_chans:%d using_chans:%d\n",
         a52dec->sample_rate, a52dec->stream_channels, a52dec->using_channels);
     gst_a52dec_reneg (a52dec->srcpad,
         gst_a52dec_channels (a52dec->using_channels), a52dec->sample_rate);
@@ -625,10 +619,6 @@ plugin_init (GModule * module, GstPlugin * plugin)
 {
   GstElementFactory *factory;
 
-  /* this filter needs the bytestream package */
-  if (!gst_library_load ("gstbytestream"))
-    return FALSE;
-
   /* create an elementfactory for the a52dec element */
   factory = gst_element_factory_new ("a52dec", GST_TYPE_A52DEC, &gst_a52dec_details);
   g_return_val_if_fail (factory != NULL, FALSE);
