@@ -17,6 +17,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 
 /* First, include the header file for the plugin, to bring in the
@@ -34,7 +37,7 @@
  */
 static GstElementDetails dvdec_details = {
   "DV (smpte314) decoder plugin",
-  "Codec/Video/Decoder/DV",
+  "Codec/Video/Decoder",
   "LGPL",
   "Uses libdv to decode DV video (libdv.sourceforge.net)",
   VERSION,
@@ -74,11 +77,8 @@ GST_PAD_TEMPLATE_FACTORY (sink_temp,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "dv_dec_sink",
-    "video/dv",
-    "format",   GST_PROPS_LIST (
-	          GST_PROPS_STRING ("PAL"),
-	          GST_PROPS_STRING ("NTSC")
-	       )
+    "video/x-dv",
+    "systemstream", GST_PROPS_BOOLEAN (TRUE)
   )
 )
 
@@ -89,36 +89,55 @@ GST_PAD_TEMPLATE_FACTORY (video_src_temp,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "dv_dec_src",
-    "video/raw",
+    "video/x-raw-yuv",
     "format",   	GST_PROPS_FOURCC (GST_MAKE_FOURCC ('Y','U','Y','2')),
     "width",    	GST_PROPS_INT (720),
-    "height",  		GST_PROPS_INT_RANGE (NTSC_HEIGHT, PAL_HEIGHT)
+    "height",  		GST_PROPS_LIST (
+			  GST_PROPS_INT (NTSC_HEIGHT),
+			  GST_PROPS_INT (PAL_HEIGHT)
+			),
+    "framerate",	GST_PROPS_LIST (
+			  GST_PROPS_FLOAT (25.),
+			  GST_PROPS_FLOAT (30/1.001)
+			)
   ),
   GST_CAPS_NEW (
     "dv_dec_src",
-    "video/raw",
-    "format", 		GST_PROPS_FOURCC(GST_MAKE_FOURCC('R','G','B',' ')),
+    "video/x-raw-rgb",
     "bpp", 		GST_PROPS_INT(32),
     "depth", 		GST_PROPS_INT(32),
-    "endianness", 	GST_PROPS_INT (G_LITTLE_ENDIAN),
-    "red_mask", 	GST_PROPS_INT(0x00ff0000),
+    "endianness", 	GST_PROPS_INT (G_BIG_ENDIAN),
+    "red_mask", 	GST_PROPS_INT(0x000000ff),
     "green_mask", 	GST_PROPS_INT(0x0000ff00),
-    "blue_mask", 	GST_PROPS_INT(0x000000ff),
+    "blue_mask", 	GST_PROPS_INT(0x00ff0000),
     "width",     	GST_PROPS_INT (720),
-    "height",    	GST_PROPS_INT_RANGE (NTSC_HEIGHT, PAL_HEIGHT)
+    "height",  		GST_PROPS_LIST (
+			  GST_PROPS_INT (NTSC_HEIGHT),
+			  GST_PROPS_INT (PAL_HEIGHT)
+			),
+    "framerate",	GST_PROPS_LIST (
+			  GST_PROPS_FLOAT (25.),
+			  GST_PROPS_FLOAT (30/1.001)
+			)
   ),
   GST_CAPS_NEW (
     "dv_dec_src",
-    "video/raw",
-    "format", 		GST_PROPS_FOURCC(GST_MAKE_FOURCC('R','G','B',' ')),
+    "video/x-raw-rgb",
     "bpp", 		GST_PROPS_INT(24),
     "depth", 		GST_PROPS_INT(24),
-    "endianness", 	GST_PROPS_INT (G_LITTLE_ENDIAN),
+    "endianness", 	GST_PROPS_INT (G_BIG_ENDIAN),
     "red_mask", 	GST_PROPS_INT(0x0000ff),
     "green_mask", 	GST_PROPS_INT(0x00ff00),
     "blue_mask", 	GST_PROPS_INT(0xff0000),
     "width",     	GST_PROPS_INT (720),
-    "height",    	GST_PROPS_INT_RANGE (NTSC_HEIGHT, PAL_HEIGHT)
+    "height",  		GST_PROPS_LIST (
+			  GST_PROPS_INT (NTSC_HEIGHT),
+			  GST_PROPS_INT (PAL_HEIGHT)
+			),
+    "framerate",	GST_PROPS_LIST (
+			  GST_PROPS_FLOAT (25.),
+			  GST_PROPS_FLOAT (30/1.001)
+			)
   )
 )
 
@@ -128,9 +147,7 @@ GST_PAD_TEMPLATE_FACTORY ( audio_src_temp,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "arts_sample",
-    "audio/raw",
-    "format",   	GST_PROPS_STRING ("int"),
-    "law",      	GST_PROPS_INT (0),
+    "audio/x-raw-int",
     "depth",   		GST_PROPS_INT (16),
     "width",    	GST_PROPS_INT (16),
     "signed",   	GST_PROPS_BOOLEAN (TRUE),
@@ -142,30 +159,39 @@ GST_PAD_TEMPLATE_FACTORY ( audio_src_temp,
 
 /* typefind stuff */
 static GstCaps*
-dv_type_find (GstBuffer *buf, gpointer private)
+dv_type_find (GstByteStream *bs, gpointer private)
 {
-  guint32 head = GUINT32_FROM_BE(*((guint32 *)GST_BUFFER_DATA(buf)));
+  GstBuffer *buf = NULL;
   GstCaps *new = NULL;
 
-  /* check for DIF  and DV flag */
-  if ((head & 0xffffff00) == 0x1f070000 && !(GST_BUFFER_DATA(buf)[4] & 0x01)) {
-    gchar *format;
+  if (gst_bytestream_peek (bs, &buf, 5) == 5) {
+    guint32 head = GUINT32_FROM_BE (*((guint32 *) GST_BUFFER_DATA (buf)));
 
-    if ((head & 0x000000ff) & 0x80)
-      format = "PAL";
-    else
-      format = "NTSC";
+    /* check for DIF  and DV flag */
+    if ((head & 0xffffff00) == 0x1f070000 &&
+        !(GST_BUFFER_DATA (buf)[4] & 0x01)) {
+      gchar *format;
+
+      if ((head & 0x000000ff) & 0x80)
+        format = "PAL";
+      else
+        format = "NTSC";
     
-    new = GST_CAPS_NEW ("dv_type_find",
-                        "video/dv",
-                          "format",   GST_PROPS_STRING (format)
-		       );
+      new = GST_CAPS_NEW ("dv_type_find",
+                          "video/x-dv",
+                          "systemstream", GST_PROPS_BOOLEAN (TRUE));
+    }
   }
+
+  if (buf != NULL) {
+    gst_buffer_unref (buf);
+  }
+
   return new;
 }
 
 static GstTypeDefinition dv_definition = {
-  "dv_video/dv", "video/dv", ".dv", dv_type_find 
+  "dv_video/dv", "video/x-dv", ".dv", dv_type_find 
 };
 
 #define GST_TYPE_DVDEC_QUALITY (gst_dvdec_quality_get_type())
@@ -347,7 +373,7 @@ gst_dvdec_get_formats (GstPad *pad)
 {
   static const GstFormat src_formats[] = {
     GST_FORMAT_BYTES,
-    GST_FORMAT_UNITS,
+    GST_FORMAT_DEFAULT,
     GST_FORMAT_TIME,
     0
   };
@@ -379,8 +405,6 @@ gst_dvdec_src_convert (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
         default:
           res = FALSE;
@@ -388,15 +412,13 @@ gst_dvdec_src_convert (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
 	  if (pad == dvdec->videosrcpad)
 	    scale = 720 * dvdec->height * dvdec->bpp;
 	  else if (pad == dvdec->audiosrcpad)
 	    scale = dvdec->decoder->audio->num_channels * 2;
 	  /* fallthrough */
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  if (pad == dvdec->videosrcpad)
 	    *dest_value = src_value * dvdec->framerate * scale / (GST_SECOND*100);
 	  else if (pad == dvdec->audiosrcpad)
@@ -427,19 +449,21 @@ gst_dvdec_sink_convert (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
-          *dest_value = src_value * (GST_SECOND * 100) / (dvdec->length * dvdec->framerate); 
+	{
+          guint64 frame;
+	  /* get frame number */
+          frame = src_value / dvdec->length;
+
+          *dest_value = (frame * GST_SECOND * 100) / dvdec->framerate;
           break;
+	}
         default:
           res = FALSE;
       }
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
 	{
           guint64 frame;
@@ -482,8 +506,6 @@ gst_dvdec_src_query (GstPad *pad, GstQueryType type,
   switch (type) {
     case GST_QUERY_TOTAL:
       switch (*format) {
-        case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
         default:
 	{
           guint64 len;
@@ -507,8 +529,6 @@ gst_dvdec_src_query (GstPad *pad, GstQueryType type,
       break;
     case GST_QUERY_POSITION:
       switch (*format) {
-        case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
         default:
 	  res = gst_pad_convert (pad, GST_FORMAT_TIME, dvdec->next_ts, format, value);
           break;
@@ -634,6 +654,7 @@ gst_dvdec_handle_src_event (GstPad *pad, GstEvent *event)
         res = FALSE;
 	break;
       }
+      dvdec->next_ts = position;
       /* then try to figure out the byteoffset for this time */
       format = GST_FORMAT_BYTES;
       if (!gst_pad_convert (dvdec->sinkpad, GST_FORMAT_TIME, position,
@@ -647,7 +668,8 @@ gst_dvdec_handle_src_event (GstPad *pad, GstEvent *event)
       if (!gst_bytestream_seek (dvdec->bs, position, GST_SEEK_METHOD_SET)) {
 	res = FALSE;
       }
-      dvdec->end_position = -1;
+      if (GST_EVENT_TYPE (event) != GST_EVENT_SEEK_SEGMENT)
+        dvdec->end_position = -1;
       break;
     }
     default:
@@ -667,17 +689,17 @@ gst_dvdec_push (GstDVDec *dvdec, GstBuffer *outbuf, GstPad *pad, GstClockTime ts
     GstEvent *discont;
 
     discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, ts, NULL);
-    gst_pad_push (pad, GST_BUFFER (discont));
+    gst_pad_push (pad, GST_DATA (discont));
   }
 
-  gst_pad_push (pad, outbuf);
+  gst_pad_push (pad, GST_DATA (outbuf));
 
   if ((dvdec->end_position != -1) &&
       (dvdec->next_ts >= dvdec->end_position)) {
     if (dvdec->loop) 
-      gst_pad_push (pad, GST_BUFFER(gst_event_new (GST_EVENT_SEGMENT_DONE)));
+      gst_pad_push (pad, GST_DATA(gst_event_new (GST_EVENT_SEGMENT_DONE)));
     else
-      gst_pad_push (pad, GST_BUFFER(gst_event_new (GST_EVENT_EOS)));
+      gst_pad_push (pad, GST_DATA(gst_event_new (GST_EVENT_EOS)));
   }
 }
 
@@ -691,6 +713,7 @@ gst_dvdec_loop (GstElement *element)
   guint32 length, got_bytes;
   GstFormat format;
   guint64 ts;
+  gfloat fps;
 
   dvdec = GST_DVDEC (element);
 
@@ -705,6 +728,7 @@ gst_dvdec_loop (GstElement *element)
   dvdec->PAL = dv_system_50_fields (dvdec->decoder);
 
   dvdec->framerate = (dvdec->PAL ? 2500 : 2997);
+  fps = (dvdec->PAL ? 25. : 30/1.001);
   dvdec->height = height = (dvdec->PAL ? PAL_HEIGHT : NTSC_HEIGHT);
   length = (dvdec->PAL ? PAL_BUFFER : NTSC_BUFFER);
 
@@ -721,9 +745,7 @@ gst_dvdec_loop (GstElement *element)
   }
 
   /* if we did not negotiate yet, do it now */
-  if (!GST_PAD_CAPS (dvdec->videosrcpad) && 
-       GST_PAD_IS_USABLE (dvdec->videosrcpad)) 
-  {
+  if (!GST_PAD_CAPS (dvdec->videosrcpad)) {
     GstCaps *allowed;
     GstCaps *trylist;
     
@@ -731,12 +753,18 @@ gst_dvdec_loop (GstElement *element)
     allowed = gst_pad_get_allowed_caps (dvdec->videosrcpad);
 
     /* try to fix our height */
-    trylist = gst_caps_intersect (allowed,
-	    GST_CAPS_NEW (
+    trylist = gst_caps_intersect (allowed, gst_caps_append (
+            GST_CAPS_NEW (
 	      "dvdec_negotiate",
-	      "video/raw",
-	        "height",  	GST_PROPS_INT (height)
-	    ));
+	      "video/x-raw-yuv",
+	        "height",  	GST_PROPS_INT (height),
+                "framerate",	GST_PROPS_FLOAT (fps)
+	    ), GST_CAPS_NEW (
+	      "dvdec_negotiate",
+	      "video/x-raw-rgb",
+	        "height",  	GST_PROPS_INT (height),
+                "framerate",	GST_PROPS_FLOAT (fps)
+            )));
     
     /* prepare for looping */
     trylist = gst_caps_normalize (trylist);
@@ -795,10 +823,8 @@ gst_dvdec_loop (GstElement *element)
       gst_pad_try_set_caps (dvdec->audiosrcpad,
 	GST_CAPS_NEW (
 	  "dvdec_audio_caps",
-	  "audio/raw",
-    	    "format",   	GST_PROPS_STRING ("int"),
+	  "audio/x-raw-int",
     	    "rate",   		GST_PROPS_INT (dvdec->decoder->audio->frequency),
-	    "law",      	GST_PROPS_INT (0),
 	    "depth",   		GST_PROPS_INT (16),
 	    "width",    	GST_PROPS_INT (16),
 	    "signed",   	GST_PROPS_BOOLEAN (TRUE),
@@ -980,9 +1006,6 @@ plugin_init (GModule *module, GstPlugin *plugin)
   GstElementFactory *factory;
   GstTypeFactory *type;
 
-  if (!gst_library_load ("gstbytestream"))
-    return FALSE;
-
   /* We need to create an ElementFactory for each element we provide.
    * This consists of the name of the element, the GType identifier,
    * and a pointer to the details structure at the top of the file.
@@ -1015,4 +1038,3 @@ GstPluginDesc plugin_desc = {
   "dvdec",
   plugin_init
 };
-
