@@ -17,6 +17,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -120,7 +123,7 @@ static void 		dvdnavsrc_init		(DVDNavSrc *dvdnavsrc);
 static void 		dvdnavsrc_set_property		(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void 		dvdnavsrc_get_property		(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
-static GstBuffer *	dvdnavsrc_get		(GstPad *pad);
+static GstData *	dvdnavsrc_get		(GstPad *pad);
 /*static GstBuffer *	dvdnavsrc_get_region	(GstPad *pad,gulong offset,gulong size); */
 static gboolean 	dvdnavsrc_event 		(GstPad *pad, GstEvent *event);
 static const GstEventMask*
@@ -445,7 +448,7 @@ dvdnavsrc_tca_seek(DVDNavSrc *src, int title, int chapter, int angle)
   /**
    * Make sure the chapter number is valid for this title.
    */
-  if (dvdnav_get_number_of_programs (src->dvdnav, &programs) != DVDNAV_STATUS_OK) {
+  if (dvdnav_get_number_of_titles (src->dvdnav, &programs) != DVDNAV_STATUS_OK) {
     fprintf( stderr, "dvdnav_get_number_of_programs error: %s\n", dvdnav_err_to_string(src->dvdnav));
     return FALSE;
   }
@@ -775,7 +778,7 @@ dvdnavsrc_print_event (DVDNavSrc *src, guint8 *data, int event, int len)
       break;
     case DVDNAV_CELL_CHANGE:
       {
-        dvdnav_cell_change_event_t *event = (dvdnav_cell_change_event_t *)data;
+        /*dvdnav_cell_change_event_t *event = (dvdnav_cell_change_event_t *)data;*/
         /*fprintf (stderr, "  old_cell: %p\n", event->old_cell);*/
         /*fprintf (stderr, "  new_cell: %p\n", event->new_cell);*/
       }
@@ -828,7 +831,7 @@ dvdnavsrc_print_event (DVDNavSrc *src, guint8 *data, int event, int len)
   }
 }
 
-static GstBuffer *
+static GstData *
 dvdnavsrc_get (GstPad *pad) 
 {
   DVDNavSrc *src;
@@ -847,15 +850,15 @@ dvdnavsrc_get (GstPad *pad)
     GstEvent *event;
 
     src->did_seek = FALSE;
-    GST_DEBUG (GST_CAT_EVENT, "dvdnavsrc sending discont");
+    GST_DEBUG ("dvdnavsrc sending discont");
     event = gst_event_new_discontinuous (FALSE, 0);
     src->need_flush = FALSE;
-    return GST_BUFFER (event);
+    return GST_DATA (event);
   }
   if (src->need_flush) {
     src->need_flush = FALSE;
-    GST_DEBUG (GST_CAT_EVENT, "dvdnavsrc sending flush");
-    return GST_BUFFER (gst_event_new_flush());
+    GST_DEBUG ("dvdnavsrc sending flush");
+    return GST_DATA (gst_event_new_flush());
   }
 
   /* loop processing blocks until data is pushed */
@@ -896,7 +899,7 @@ dvdnavsrc_get (GstPad *pad)
         }
         break;
       case DVDNAV_STOP:
-        GST_DEBUG (GST_CAT_EVENT, "dvdnavsrc sending eos");
+        GST_DEBUG ("dvdnavsrc sending eos");
         gst_element_set_eos (GST_ELEMENT (src));
         dvdnavsrc_close(src);
         buf = GST_BUFFER (gst_event_new (GST_EVENT_EOS));
@@ -908,19 +911,23 @@ dvdnavsrc_get (GstPad *pad)
       case DVDNAV_NAV_PACKET:
         if (0) dvdnavsrc_update_buttoninfo (src);
         break;
+      case DVDNAV_WAIT:
+	/* FIXME: supposed to make sure all the data has made 
+	 * it to the sinks before skipping the wait
+	 */
+        dvdnav_wait_skip(src->dvdnav);
       case DVDNAV_VTS_CHANGE:
       case DVDNAV_SPU_STREAM_CHANGE:
       case DVDNAV_AUDIO_STREAM_CHANGE:
       case DVDNAV_HIGHLIGHT:
       case DVDNAV_SPU_CLUT_CHANGE:
       case DVDNAV_HOP_CHANNEL:
-      case DVDNAV_WAIT:
       default:
         dvdnavsrc_print_event (src, data, event, len);
         break;
     }
   }
-  return buf;
+  return GST_DATA(buf);
 }
 
 /* open the file, necessary to go to RUNNING state */
@@ -1148,7 +1155,7 @@ dvdnavsrc_event (GstPad *pad, GstEvent *event)
                 new_part = part + offset;
                 break;
               case GST_SEEK_METHOD_END:
-                if (dvdnav_get_number_of_programs(src->dvdnav, &parts) !=
+                if (dvdnav_get_number_of_titles(src->dvdnav, &parts) !=
                     DVDNAV_STATUS_OK) {
                   goto error;
                 }
@@ -1216,7 +1223,7 @@ dvdnavsrc_get_formats (GstPad *pad)
     /*
     GST_FORMAT_TIME,
     GST_FORMAT_BYTES,
-    GST_FORMAT_UNITS,
+    GST_FORMAT_DEFAULT,
     */
     0,	/* filled later */
     0,	/* filled later */
@@ -1257,7 +1264,7 @@ dvdnavsrc_convert (GstPad *pad,
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
           src_value <<= 2;	/* 4 bytes per sample */
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  *dest_value = src_value * 44100 / GST_SECOND;
 	  break;
 	default:
@@ -1278,7 +1285,7 @@ dvdnavsrc_convert (GstPad *pad,
       break;
     case GST_FORMAT_BYTES:
       src_value >>= 2;
-    case GST_FORMAT_UNITS:
+    case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
           *dest_value = src_value * 4;
@@ -1325,7 +1332,7 @@ dvdnavsrc_convert (GstPad *pad,
 	  break;
         case GST_FORMAT_BYTES:
           sector <<= 2;
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
           *dest_value = (CD_FRAMESIZE_RAW >> 2) * sector;
 	  break;
 	default:
@@ -1391,7 +1398,7 @@ dvdnavsrc_query (GstPad *pad, GstQueryType type,
         }
         *value = titles;
       } else if (*format == chapter_format) {
-        if (dvdnav_get_number_of_programs(src->dvdnav, &parts) != DVDNAV_STATUS_OK) {
+        if (dvdnav_get_number_of_titles(src->dvdnav, &parts) != DVDNAV_STATUS_OK) {
           res = FALSE;
         }
         *value = parts;
