@@ -26,7 +26,7 @@
 extern GstElementDetails flacenc_details;
 extern GstElementDetails flacdec_details;
 
-static GstCaps* 	flac_type_find 	(GstBuffer *buf, gpointer private);
+static GstCaps* 	flac_type_find 	(GstByteStream *bs, gpointer private);
 
 GstPadTemplate *gst_flacdec_src_template, *gst_flacdec_sink_template; 
 GstPadTemplate *gst_flacenc_src_template, *gst_flacenc_sink_template;
@@ -38,7 +38,10 @@ flac_caps_factory (void)
    gst_caps_new (
   	"flac_flac",
   	"application/x-flac",
-  	NULL);
+	/*gst_props_new (
+ 	    "rate",     	GST_PROPS_INT_RANGE (11025, 48000),
+    	    "channels", 	GST_PROPS_INT_RANGE (1, 2),
+	    NULL)*/ NULL);
 }
 
 static GstCaps*
@@ -47,10 +50,8 @@ raw_caps_factory (void)
   return
    gst_caps_new (
   	"flac_raw",
-  	"audio/raw",
+  	"audio/x-raw-int",
 	gst_props_new (
-  	  "format",   		GST_PROPS_STRING ("int"),
-    	    "law",   		GST_PROPS_INT (0),
     	    "endianness", 	GST_PROPS_INT (G_BYTE_ORDER),
     	    "signed", 		GST_PROPS_BOOLEAN (TRUE),
     	    "width", 		GST_PROPS_INT (16),
@@ -61,21 +62,34 @@ raw_caps_factory (void)
 }
 
 static GstTypeDefinition flacdefinition = {
-  "flac_application/x-flac",
-  "application/x-flac",
+  "flac_audio/x-flac",
+  "audio/x-flac",
   ".flac",
   flac_type_find,
 };
 
+
 static GstCaps* 
-flac_type_find (GstBuffer *buf, gpointer private) 
+flac_type_find (GstByteStream *bs, gpointer private) 
 {
-  guint32 head = GUINT32_FROM_BE (*((guint32 *)GST_BUFFER_DATA (buf)));
+  GstBuffer *buf = NULL;
+  GstCaps *new = NULL;
 
-  if (head  != 0x664C6143)
-    return NULL;
+  if (gst_bytestream_peek (bs, &buf, 4) == 4) {
+    guint32 head = GUINT32_FROM_BE (*((guint32 *) GST_BUFFER_DATA (buf)));
 
-  return gst_caps_new ("flac_type_find", "application/x-flac", NULL);
+    if (head  == 0x664C6143) {
+      new = GST_CAPS_NEW ("flac_type_find",
+			  "application/x-flac",
+			    NULL);
+    }
+  }
+
+  if (buf != NULL) {
+    gst_buffer_unref (buf);
+  }
+
+  return new;
 }
 
 
@@ -86,10 +100,6 @@ plugin_init (GModule *module, GstPlugin *plugin)
   GstTypeFactory *type;
   GstCaps *raw_caps, *flac_caps;
 
-  /* this filter needs the bytestream package */
-  if (!gst_library_load ("gstbytestream"))
-    return FALSE;
-
   gst_plugin_set_longname (plugin, "The FLAC Lossless compressor Codec");
 
   /* create an elementfactory for the flacenc element */
