@@ -24,7 +24,7 @@
 
 #include <config.h>
 #include <gst/gst.h>
-#include <gst/bytestream/bytestream.h>
+#include <gst/gstbytestream.h>
 
 #include <FLAC/all.h>
 
@@ -46,7 +46,7 @@ struct _FlacDec {
   GstElement 	 element;
 
   GstPad 	*sinkpad,*srcpad;
-  GstCaps *metadata;
+	GstCaps *metadata;
   GstByteStream *bs;
 
   FLAC__SeekableStreamDecoder *decoder;
