@@ -26,6 +26,10 @@
 #define BROKEN_SIG 1
 /*#undef BROKEN_SIG */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/time.h>
@@ -95,6 +99,7 @@ struct _GstGnomeVFSSrc {
 	/* icecast/audiocast metadata extraction handling */
 	gboolean iradio_mode;
 	gboolean http_callbacks_pushed;
+	gboolean seekable;
 
 	gint icy_metaint;
 	GnomeVFSFileSize icy_count;
@@ -161,6 +166,7 @@ enum {
 	ARG_IRADIO_GENRE,
 	ARG_IRADIO_URL,
 	ARG_IRADIO_TITLE,
+	ARG_SEEKABLE,
 };
 
 GType gst_gnomevfssrc_get_type(void);
@@ -174,7 +180,7 @@ static void 		gst_gnomevfssrc_set_property	(GObject *object, guint prop_id,
 static void 		gst_gnomevfssrc_get_property	(GObject *object, guint prop_id,
 							 GValue *value, GParamSpec *pspec);
 
-static GstBuffer*	gst_gnomevfssrc_get		(GstPad *pad);
+static GstData*	gst_gnomevfssrc_get		(GstPad *pad);
 
 static GstElementStateReturn 
 			gst_gnomevfssrc_change_state	(GstElement *element);
@@ -272,6 +278,13 @@ static void gst_gnomevfssrc_class_init(GstGnomeVFSSrcClass *klass)
 							      "Name of currently playing song",
 							      NULL,
 							      G_PARAM_READABLE));
+	g_object_class_install_property (gobject_class,
+					 ARG_SEEKABLE,
+					 g_param_spec_boolean ("seekable",
+							       "seekable",
+							       "TRUE is stream is seekable",
+							       FALSE,
+							       G_PARAM_READABLE));
 
 	gstelement_class->set_property = gst_gnomevfssrc_set_property;
 	gstelement_class->get_property = gst_gnomevfssrc_get_property;
@@ -304,6 +317,8 @@ static void gst_gnomevfssrc_init(GstGnomeVFSSrc *gnomevfssrc)
 
 	gnomevfssrc->icy_metaint = 0;
 
+	gnomevfssrc->seekable = FALSE;
+
 	gnomevfssrc->iradio_mode = FALSE;
 	gnomevfssrc->http_callbacks_pushed = FALSE;
 	gnomevfssrc->icy_count = 0;
@@ -364,7 +379,7 @@ static void gst_gnomevfssrc_set_property(GObject *object, guint prop_id, const G
 
 		g_free(src->filename);
 
-		/* clear the filename if we get a NULL (is that possible?) */
+		/* clear the filename if we get a NULL */
 		if (g_value_get_string (value) == NULL) {
 			gst_element_set_state(GST_ELEMENT(object), GST_STATE_NULL);
 			src->filename = NULL;
@@ -437,12 +452,48 @@ static void gst_gnomevfssrc_get_property(GObject *object, guint prop_id, GValue
 		g_value_set_string (value, src->iradio_title);
 		g_mutex_unlock(src->audiocast_udpdata_mutex);
 		break;
+	case ARG_SEEKABLE:
+		g_value_set_boolean (value, src->seekable);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
+static char *
+unicodify (const char *str, int len, ...)
+{
+	char *ret = NULL, *cset;
+	va_list args;
+	gsize bytes_read, bytes_written;
+
+	if (g_utf8_validate (str, len, NULL))
+		return g_strndup (str, len >= 0 ? len : strlen (str));
+
+	va_start (args, len);
+	while ((cset = va_arg (args, char *)) != NULL)
+	{
+		if (!strcmp (cset, "locale"))
+			ret = g_locale_to_utf8 (str, len, &bytes_read,
+						&bytes_written, NULL);
+		else
+			ret = g_convert (str, len, "UTF-8", cset,
+					 &bytes_read, &bytes_written, NULL);
+		if (ret)
+			break;
+	}
+	va_end (args);
+
+	return ret;
+}
+
+static char *
+gst_gnomevfssrc_unicodify (const char *str)
+{
+	return unicodify (str, -1, "locale", "ISO-8859-1", NULL);
+}
+
 /*
  * icecast/audiocast metadata extraction support code
  */
@@ -453,17 +504,21 @@ static int audiocast_init(GstGnomeVFSSrc *src)
 	GError *error = NULL;
 	if (!src->iradio_mode)
 		return TRUE;
-	GST_DEBUG (0,"audiocast: registering listener");
+	GST_DEBUG ("audiocast: registering listener");
 	if (audiocast_register_listener(&src->audiocast_port, &src->audiocast_fd) < 0)
 	{
+		char *escaped;
+
+		escaped = gnome_vfs_unescape_string_for_display (src->filename);
 		gst_element_error(GST_ELEMENT(src),
 				  "opening vfs file \"%s\" (%s)",
-				  src->filename, 
+				  escaped,
 				  "unable to register UDP port");
+		g_free (escaped);
 		close(src->audiocast_fd);
 		return FALSE;
 	}
-	GST_DEBUG (0,"audiocast: creating pipe");
+	GST_DEBUG ("audiocast: creating pipe");
 	src->audiocast_notify_queue = NULL;
 	if (pipe(pipefds) < 0)
 	{
@@ -472,13 +527,16 @@ static int audiocast_init(GstGnomeVFSSrc *src)
 	}
 	src->audiocast_thread_die_infd = pipefds[0];
 	src->audiocast_thread_die_outfd = pipefds[1];
-	GST_DEBUG (0,"audiocast: creating audiocast thread");
+	GST_DEBUG ("audiocast: creating audiocast thread");
 	src->audiocast_thread = g_thread_create((GThreadFunc) audiocast_thread_run, src, TRUE, &error);
 	if (error != NULL) {
+		char *escaped;
+		escaped = gnome_vfs_unescape_string_for_display (src->filename);
 		gst_element_error(GST_ELEMENT(src),
 				  "opening vfs file \"%s\" (unable to create thread: %s)",
-				  src->filename, 
+				  escaped,
 				  error->message);
+		g_free (escaped);
 		close(src->audiocast_fd);
 		return FALSE;
 	}
@@ -491,7 +549,7 @@ static int audiocast_register_listener(gint *port, gint *fd)
 	int sock;
 	socklen_t sinlen = sizeof (struct sockaddr_in);
 
-	GST_DEBUG (0,"audiocast: estabilishing UDP listener");
+	GST_DEBUG ("audiocast: estabilishing UDP listener");
 
 	if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
 		goto lose;
@@ -507,7 +565,7 @@ static int audiocast_register_listener(gint *port, gint *fd)
 	if (getsockname(sock, (struct sockaddr *)&sin, &sinlen) < 0)
 		goto lose_and_close;
 
-	GST_DEBUG (0,"audiocast: listening on local %s:%d", inet_ntoa(sin.sin_addr), g_ntohs(sin.sin_port));
+	GST_DEBUG ("audiocast: listening on local %s:%d", inet_ntoa(sin.sin_addr), g_ntohs(sin.sin_port));
 
 	*port = g_ntohs(sin.sin_port);
 	*fd = sock;
@@ -537,7 +595,6 @@ static void audiocast_do_notifications(GstGnomeVFSSrc *src)
 static gpointer audiocast_thread_run(GstGnomeVFSSrc *src)
 {
 	char buf[1025], **lines;
-	char *valptr;
 	gsize len;
 	fd_set fdset, readset;
 	struct sockaddr_in from;
@@ -550,7 +607,7 @@ static gpointer audiocast_thread_run(GstGnomeVFSSrc *src)
 
 	while (1)
 	{
-		GST_DEBUG (0,"audiocast thread: dropping into select");
+		GST_DEBUG ("audiocast thread: dropping into select");
 		readset = fdset;
 		if (select(FD_SETSIZE, &readset, NULL, NULL, NULL) < 0) {
 			perror("select");
@@ -558,19 +615,20 @@ static gpointer audiocast_thread_run(GstGnomeVFSSrc *src)
 		}
 		if (FD_ISSET(src->audiocast_thread_die_infd, &readset)) {
 			char buf[1];
-			GST_DEBUG (0,"audiocast thread: got die character");
+			GST_DEBUG ("audiocast thread: got die character");
 			read(src->audiocast_thread_die_infd, buf, 1);
 			close(src->audiocast_thread_die_infd);
 			close(src->audiocast_fd);
 			return NULL;
 		}
-		GST_DEBUG (0,"audiocast thread: reading data");
+		GST_DEBUG ("audiocast thread: reading data");
 		len = recvfrom(src->audiocast_fd, buf, sizeof(buf) - 1, 0, (struct sockaddr *) &from, &fromlen);
 		if (len < 0 && errno == EAGAIN)
 			continue;
 		else if (len >= 0)
 		{
 			int i;
+			char *valptr, *value;
 			buf[len] = '\0';
 			lines = g_strsplit(buf, "\n", 0);
 			if (!lines)
@@ -591,35 +649,45 @@ static gpointer audiocast_thread_run(GstGnomeVFSSrc *src)
 				g_strstrip(valptr);
 				if (!strlen(valptr))
 					continue;
+
+				value = gst_gnomevfssrc_unicodify (valptr);
+				if (!value)
+				{
+					g_print ("Unable to convert \"%s\" to UTF-8!\n", valptr);
+					continue;
+				}
 		
 				if (!strncmp(lines[i], "x-audiocast-streamtitle", 23)) {
 					g_mutex_lock(src->audiocast_udpdata_mutex);
-					if (src->iradio_title)
-						g_free(src->iradio_title);
-					src->iradio_title = g_strdup(valptr);
+					g_free(src->iradio_title);
+					src->iradio_title = value;
 					g_mutex_unlock(src->audiocast_udpdata_mutex);
+
 					g_mutex_lock(src->audiocast_queue_mutex);
 					src->audiocast_notify_queue = g_list_append(src->audiocast_notify_queue, "iradio-title");
-					GST_DEBUG(0,"audiocast title: %s\n", src->iradio_title);
+					GST_DEBUG ("audiocast title: %s\n", src->iradio_title);
 					g_mutex_unlock(src->audiocast_queue_mutex);
 				} else if (!strncmp(lines[i], "x-audiocast-streamurl", 21)) {
 					g_mutex_lock(src->audiocast_udpdata_mutex);
-					if (src->iradio_url)
-						g_free(src->iradio_url);
-					src->iradio_url = g_strdup(valptr);
+					g_free(src->iradio_url);
+					src->iradio_url = value;
 					g_mutex_unlock(src->audiocast_udpdata_mutex);
+
 					g_mutex_lock(src->audiocast_queue_mutex);
 					src->audiocast_notify_queue = g_list_append(src->audiocast_notify_queue, "iradio-url");
-					GST_DEBUG(0,"audiocast url: %s\n", src->iradio_title);
+					GST_DEBUG ("audiocast url: %s\n", src->iradio_title);
 					g_mutex_unlock(src->audiocast_queue_mutex);
 				} else if (!strncmp(lines[i], "x-audiocast-udpseqnr", 20)) {
 					gchar outbuf[120];
-					sprintf(outbuf, "x-audiocast-ack: %ld \r\n", atol(valptr));
+
+					sprintf(outbuf, "x-audiocast-ack: %ld \r\n", atol(value));
+					g_free (value);
+
 					if (sendto(src->audiocast_fd, outbuf, strlen(outbuf), 0, (struct sockaddr *) &from, fromlen) <= 0) {
-						fprintf(stderr, "Error sending response to server: %s\n", strerror(errno));
+						g_print("Error sending response to server: %s\n", strerror(errno));
 						continue;
 					}
-					GST_DEBUG(0,"sent audiocast ack: %s\n", outbuf);
+					GST_DEBUG ("sent audiocast ack: %s\n", outbuf);
 				}
 			}
 			g_strfreev(lines);
@@ -639,10 +707,10 @@ static void audiocast_thread_kill(GstGnomeVFSSrc *src)
 	  metadata, then we don't need the
 	  audiocast metadata too.
 	*/
-	GST_DEBUG (0,"audiocast: writing die character");
+	GST_DEBUG ("audiocast: writing die character");
 	write(src->audiocast_thread_die_outfd, "q", 1);
 	close(src->audiocast_thread_die_outfd);
-	GST_DEBUG (0,"audiocast: joining thread");
+	GST_DEBUG ("audiocast: joining thread");
 	g_thread_join(src->audiocast_thread);
 	src->audiocast_thread = NULL;
 }
@@ -660,7 +728,7 @@ gst_gnomevfssrc_send_additional_headers_callback (gconstpointer in,
 
 	if (!src->iradio_mode)
 		return;
-	GST_DEBUG(0,"sending headers\n");
+	GST_DEBUG ("sending headers\n");
 
         out_args->headers = g_list_append (out_args->headers,
                                            g_strdup ("icy-metadata:1\r\n"));
@@ -702,7 +770,7 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
 		{ 
                         sscanf (data + 12, "%d", &icy_metaint);
 			src->icy_metaint = icy_metaint;
-			GST_DEBUG (0,"got icy-metaint %d, killing audiocast thread",
+			GST_DEBUG ("got icy-metaint %d, killing audiocast thread",
 				   src->icy_metaint);
 			audiocast_thread_kill(src);
 			continue;
@@ -715,22 +783,22 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
 		else
 			continue;
 
-		GST_DEBUG (0,"key: %s", key);
+		GST_DEBUG ("key: %s", key);
                 if (!strncmp (key, "name", 4)) {
+			g_free (src->iradio_name);
+			src->iradio_name = gst_gnomevfssrc_unicodify (value);
 			if (src->iradio_name)
-				g_free (src->iradio_name);
-			src->iradio_name = g_strdup (value);
-			g_object_notify (G_OBJECT (src), "iradio-name");
+				g_object_notify (G_OBJECT (src), "iradio-name");
                 } else if (!strncmp (key, "genre", 5)) {
+			g_free (src->iradio_genre);
+			src->iradio_genre = gst_gnomevfssrc_unicodify (value);
 			if (src->iradio_genre)
-				g_free (src->iradio_genre);
-			src->iradio_genre = g_strdup (value);
-			g_object_notify (G_OBJECT (src), "iradio-genre");
+				g_object_notify (G_OBJECT (src), "iradio-genre");
                 } else if (!strncmp (key, "url", 3)) {
+			g_free (src->iradio_url);
+			src->iradio_url = gst_gnomevfssrc_unicodify (value);
 			if (src->iradio_url)
-				g_free (src->iradio_url);
-			src->iradio_url = g_strdup (value);
-			g_object_notify (G_OBJECT (src), "iradio-url");
+				g_object_notify (G_OBJECT (src), "iradio-url");
 		}
         }
 }
@@ -741,7 +809,7 @@ gst_gnomevfssrc_push_callbacks (GstGnomeVFSSrc *gnomevfssrc)
 	if (gnomevfssrc->http_callbacks_pushed)
 		return;
 
-	GST_DEBUG (0,"pushing callbacks");
+	GST_DEBUG ("pushing callbacks");
 	gnome_vfs_module_callback_push (GNOME_VFS_MODULE_CALLBACK_HTTP_SEND_ADDITIONAL_HEADERS,
 					gst_gnomevfssrc_send_additional_headers_callback,
 					gnomevfssrc,
@@ -760,7 +828,7 @@ gst_gnomevfssrc_pop_callbacks (GstGnomeVFSSrc *gnomevfssrc)
 	if (!gnomevfssrc->http_callbacks_pushed)
 		return;
 
-	GST_DEBUG (0,"popping callbacks");
+	GST_DEBUG ("popping callbacks");
 	gnome_vfs_module_callback_pop (GNOME_VFS_MODULE_CALLBACK_HTTP_SEND_ADDITIONAL_HEADERS);
 	gnome_vfs_module_callback_pop (GNOME_VFS_MODULE_CALLBACK_HTTP_RECEIVED_HEADERS);
 }
@@ -777,7 +845,7 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc *src)
 	gchar **tags;
 	int i;
 
-	GST_DEBUG (0,"reading icecast metadata");
+	GST_DEBUG ("reading icecast metadata");
 
         while (length == 0) {
                 res = gnome_vfs_read (src->handle, &foobyte, 1, &length);
@@ -813,19 +881,30 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc *src)
 	{
 		if (!g_ascii_strncasecmp(tags[i], "StreamTitle=", 12))
 		{
+			g_free (src->iradio_title);
+			src->iradio_title = gst_gnomevfssrc_unicodify (tags[i]+13);
 			if (src->iradio_title)
-				g_free (src->iradio_title);
-			src->iradio_title = g_strdup(tags[i]+13);
-			GST_DEBUG(0, "sending notification on icecast title");
-			g_object_notify (G_OBJECT (src), "iradio-title");
+			{
+				GST_DEBUG ("sending notification on icecast title");
+				g_object_notify (G_OBJECT (src), "iradio-title");
+			}
+			else
+				g_print ("Unable to convert icecast title \"%s\" to UTF-8!\n",
+					 tags[i]+13);
+			
 		}
 		if (!g_ascii_strncasecmp(tags[i], "StreamUrl=", 10))
 		{
+			g_free (src->iradio_url);
+			src->iradio_url = gst_gnomevfssrc_unicodify (tags[i]+11);
 			if (src->iradio_url)
-				g_free (src->iradio_url);
-			src->iradio_url = g_strdup(tags[i]+11);
-			GST_DEBUG(0, "sending notification on icecast url");
-			g_object_notify (G_OBJECT (src), "iradio-url");
+			{
+				GST_DEBUG ("sending notification on icecast url");
+				g_object_notify (G_OBJECT (src), "iradio-url");
+			}
+			else
+				g_print ("Unable to convert icecast url \"%s\" to UTF-8!\n",
+					 tags[i]+11);
 		}
 	}
 
@@ -840,7 +919,7 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc *src)
  *
  * Push a new buffer from the gnomevfssrc at the current offset.
  */
-static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
+static GstData *gst_gnomevfssrc_get(GstPad *pad)
 {
 	GstGnomeVFSSrc *src;
 	GnomeVFSResult result = 0;
@@ -857,7 +936,7 @@ static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
 	if ((src->curoffset >= src->size) && (src->size != 0))
 	{
 		gst_element_set_eos (GST_ELEMENT (src));
-		return GST_BUFFER (gst_event_new (GST_EVENT_EOS));
+		return GST_DATA (gst_event_new (GST_EVENT_EOS));
 	}
 
 	/* create the buffer */
@@ -869,14 +948,15 @@ static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
 
 	if (src->iradio_mode && src->icy_metaint > 0) {
 		GST_BUFFER_DATA (buf) = g_malloc0 (src->icy_metaint);
-		g_return_val_if_fail (GST_BUFFER_DATA (buf) != NULL, NULL);
 		data = GST_BUFFER_DATA (buf);
+		g_return_val_if_fail (GST_BUFFER_DATA (buf) != NULL, NULL);
+
 		GST_BUFFER_SIZE (buf) = 0;
 		/* FIXME GROSS HACK: We try to read in at least 8000
 		 * bytes of data so that mp3 typefinding will work. */
 		do
 		{
-			GST_DEBUG (0,"doing read: icy_count: %Lu", src->icy_count);
+			GST_DEBUG ("doing read: icy_count: %" G_GINT64_FORMAT, src->icy_count);
 			result = gnome_vfs_read (src->handle, data,
 						 src->icy_metaint - src->icy_count,
 						 &readbytes);
@@ -886,14 +966,14 @@ static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
 				gst_buffer_unref (buf);
 				gst_element_set_eos (GST_ELEMENT (src));
 				src->in_first_get = FALSE;
-				return GST_BUFFER (gst_event_new (GST_EVENT_EOS));
+				return GST_DATA (gst_event_new (GST_EVENT_EOS));
 			}
 			
 			src->icy_count += readbytes;
 			GST_BUFFER_OFFSET (buf) = src->curoffset;
 			GST_BUFFER_SIZE (buf) += readbytes;
+		      	data += readbytes;
 			src->curoffset += readbytes;
-			data += readbytes;
 			
 			if (src->icy_count == src->icy_metaint) {
 				gst_gnomevfssrc_get_icy_metadata (src);
@@ -913,20 +993,20 @@ static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
 			GstEvent *event;
 
 			gst_buffer_unref (buf);
-			GST_DEBUG (0,"new seek %lld", src->curoffset);
+			GST_DEBUG ("new seek %" G_GINT64_FORMAT, src->curoffset);
 			src->new_seek = FALSE;
 
-			GST_DEBUG (GST_CAT_EVENT, "gnomevfssrc sending discont");
+			GST_DEBUG ("gnomevfssrc sending discont");
 			event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, src->curoffset, NULL);
 			src->need_flush = FALSE;
 
-			return GST_BUFFER (event);
+			return GST_DATA (event);
 		}
 
 		result = gnome_vfs_read(src->handle, GST_BUFFER_DATA(buf),
 				   src->bytes_per_read, &readbytes);
 
-		GST_DEBUG(0, "read: %s, readbytes: %Lu",
+		GST_DEBUG ("read: %s, readbytes: %" G_GINT64_FORMAT,
 				gnome_vfs_result_to_string(result), readbytes);
 		/* deal with EOS */
 		if (readbytes == 0)
@@ -935,7 +1015,7 @@ static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
 
 			gst_element_set_eos (GST_ELEMENT (src));
 
-			return GST_BUFFER (gst_event_new (GST_EVENT_EOS));
+			return GST_DATA (gst_event_new (GST_EVENT_EOS));
 		}
 		
 		GST_BUFFER_OFFSET(buf) = src->curoffset;
@@ -946,7 +1026,7 @@ static GstBuffer *gst_gnomevfssrc_get(GstPad *pad)
 	GST_BUFFER_TIMESTAMP (buf) = -1;
 
 	/* we're done, return the buffer */
-	return buf;
+	return GST_DATA (buf);
 }
 
 /* open the file, do stuff necessary to go to READY state */
@@ -975,12 +1055,17 @@ static gboolean gst_gnomevfssrc_open_file(GstGnomeVFSSrc *src)
 				    GNOME_VFS_OPEN_READ);
 	if (result != GNOME_VFS_OK)
 	{
+		char *escaped;
+		
 		gst_gnomevfssrc_pop_callbacks (src);
 		audiocast_thread_kill(src);
+
+		escaped = gnome_vfs_unescape_string_for_display (src->filename);
 		gst_element_error(GST_ELEMENT(src),
 				  "opening vfs file \"%s\" (%s)",
-				  src->filename, 
+				  escaped,
 				  gnome_vfs_result_to_string(result));
+		g_free (escaped);
 		return FALSE;
 	}
 	
@@ -993,18 +1078,25 @@ static gboolean gst_gnomevfssrc_open_file(GstGnomeVFSSrc *src)
 			src->size = info->size;
 	}
 	else
-		GST_DEBUG (0, "getting info failed: %s", gnome_vfs_result_to_string (result));
+		GST_DEBUG ("getting info failed: %s", gnome_vfs_result_to_string (result));
 
 	gnome_vfs_file_info_unref(info);
 	
-	GST_DEBUG(0, "size %lld", src->size);
+	GST_DEBUG ("size %" G_GINT64_FORMAT, src->size);
 	
 	audiocast_do_notifications(src);
 	
-	GST_DEBUG(0, "open result: %s", gnome_vfs_result_to_string (result));
+	GST_DEBUG ("open result: %s", gnome_vfs_result_to_string (result));
 
 	src->in_first_get = TRUE;
 
+	if(gnome_vfs_seek(src->handle, GNOME_VFS_SEEK_CURRENT, 0)
+	    == GNOME_VFS_OK){
+		src->seekable = TRUE;
+	}else{
+		src->seekable = FALSE;
+	}
+
 	GST_FLAG_SET(src, GST_GNOMEVFSSRC_OPEN);
 
 	return TRUE;
@@ -1133,7 +1225,7 @@ gst_gnomevfssrc_srcpad_event (GstPad *pad, GstEvent *event)
 		
 		result = gnome_vfs_seek(src->handle,
 					GNOME_VFS_SEEK_START, desired_offset);
-		GST_DEBUG(0, "new_seek: %s",
+		GST_DEBUG ("new_seek: %s",
 			  gnome_vfs_result_to_string(result));
 		
 		if (result != GNOME_VFS_OK) {
