@@ -94,8 +94,10 @@ struct _GstLame {
 
   lame_global_flags *lgf;
 
+  GstCaps *metadata;
+
   /* time tracker */
-  guint64 last_ts;
+  guint64 last_ts, last_offs, last_duration;
 };
 
 struct _GstLameClass {
