@@ -17,7 +17,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gst/gst.h>
+#include <gst/video/video.h>
 #include <lcs/lcs.h>
 
 #define GST_TYPE_COLORSPACE \
@@ -48,6 +52,7 @@ struct _GstColorspace {
   
   GstColorSpaceConverterType type;
   gint 		width, height;
+  gfloat	fps;
   gboolean 	disabled;
 
   GstCaps	*sinkcaps;
@@ -83,42 +88,6 @@ enum {
   ARG_DEST,
 };
 
-GST_PAD_TEMPLATE_FACTORY (colorspace_src_template_factory,
-  "src",
-  GST_PAD_SRC,
-  GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "colorspace_src",
-    "video/raw",
-      "format",		GST_PROPS_LIST (
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("YV12")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("YUY2")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("RGB "))
-	                ),
-      "width", 		GST_PROPS_INT_RANGE (0, G_MAXINT),
-      "height",		GST_PROPS_INT_RANGE (0, G_MAXINT) 
-  )
-)
-
-GST_PAD_TEMPLATE_FACTORY (colorspace_sink_template_factory,
-  "sink",
-  GST_PAD_SINK,
-  GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "colorspace_sink",
-    "video/raw",
-      "format",		GST_PROPS_LIST (
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("YV12")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("YUY2")),
-	                  GST_PROPS_FOURCC (GST_STR_FOURCC ("RGB "))
-	                ),
-      "width", 		GST_PROPS_INT_RANGE (0, G_MAXINT),
-      "height",		GST_PROPS_INT_RANGE (0, G_MAXINT) 
-  )
-)
-
 static GType 		gst_colorspace_get_type 		(void);
 
 static void		gst_colorspace_class_init		(GstColorspaceClass *klass);
@@ -135,10 +104,11 @@ static GstPadLinkReturn
 			gst_colorspace_srcconnect 		(GstPad *pad, GstCaps *caps);
 static GstPadLinkReturn
 			gst_colorspace_srcconnect_func 		(GstPad *pad, GstCaps *caps, gboolean newcaps);
-static void		gst_colorspace_chain			(GstPad *pad, GstBuffer *buf);
+static void		gst_colorspace_chain			(GstPad *pad, GstData *_data);
 static GstElementStateReturn
 			gst_colorspace_change_state 		(GstElement *element);
 
+static GstPadTemplate *srctempl, *sinktempl;
 static GstElementClass *parent_class = NULL;
 /*static guint gst_colorspace_signals[LAST_SIGNAL] = { 0 }; */
 
@@ -181,7 +151,7 @@ colorspace_find_lcs_format (GstCaps *caps)
     format = lcs_format_new_simple_rgb_packed (red_bits, green_bits, blue_bits, 0,
 		    red_shift, green_shift, blue_shift, 0, depth, G_BYTE_ORDER);
 
-    GST_DEBUG (GST_CAT_NEGOTIATION, "%lu %lu %lu %lu %lu %lu %u %s\n", red_bits, green_bits, blue_bits, 
+    GST_DEBUG ("%lu %lu %lu %lu %lu %lu %u %s\n", red_bits, green_bits, blue_bits, 
 		    red_shift, green_shift, blue_shift, depth, lcs_format_get_layout (format));
   }
   else {
@@ -204,17 +174,24 @@ colorspace_setup_converter (GstColorspace *space, GstCaps *from_caps, GstCaps *t
   g_return_val_if_fail (to_caps != NULL, FALSE);
   g_return_val_if_fail (from_caps != NULL, FALSE);
 
-  gst_caps_get_fourcc_int (from_caps, "format", &from_space);
-  gst_caps_get_fourcc_int (to_caps, "format", &to_space);
+  if (gst_caps_has_property (from_caps, "format"))
+    gst_caps_get_fourcc_int (from_caps, "format", &from_space);
+  else
+    from_space = GST_MAKE_FOURCC ('R','G','B',' ');
+
+  if (gst_caps_has_property (to_caps, "format"))
+    gst_caps_get_fourcc_int (to_caps, "format", &to_space);
+  else
+    to_space = GST_MAKE_FOURCC ('R','G','B',' ');
 
   from_format 	= colorspace_find_lcs_format (from_caps);
   to_format 	= colorspace_find_lcs_format (to_caps);
 	  
-  GST_DEBUG (GST_CAT_NEGOTIATION, "trying from %4.4s to %4.4s\n", (gchar*)&from_space, (gchar*)&to_space);
+  GST_DEBUG ("trying from %4.4s to %4.4s\n", (gchar*)&from_space, (gchar*)&to_space);
   space->converter = lcs_get_converter (from_format, to_format, LCS_FLAG_FAST);
 
   if (space->converter) {
-    GST_DEBUG (GST_CAT_NEGOTIATION, "converting from %4.4s to %4.4s\n", (gchar*)&from_space, (gchar*)&to_space);
+    GST_DEBUG ("converting from %4.4s to %4.4s\n", (gchar*)&from_space, (gchar*)&to_space);
     space->type = GST_COLORSPACE_LCS;
     return TRUE;
   }
@@ -257,8 +234,9 @@ gst_colorspace_sinkconnect (GstPad *pad, GstCaps *caps)
 
   gst_caps_get_int (caps, "width", &space->width);
   gst_caps_get_int (caps, "height", &space->height);
+  gst_caps_get_float (caps, "framerate", &space->fps);
 
-  GST_INFO (GST_CAT_PROPERTIES, "size: %dx%d", space->width, space->height);
+  GST_INFO ( "size: %dx%d", space->width, space->height);
 
   space->sinkcaps = caps;
 
@@ -310,12 +288,20 @@ gst_colorspace_srcconnect_func (GstPad *pad, GstCaps *caps, gboolean newcaps)
   }
   /* then see what the peer has that matches the size */
   peercaps = gst_caps_intersect (caps,
+		  gst_caps_append (
 		  GST_CAPS_NEW (
 		   "colorspace_filter",
-		   "video/raw",
-		     "width",   GST_PROPS_INT (space->width),
-		     "height",  GST_PROPS_INT (space->height)
-		  ));
+		   "video/x-raw-yuv",
+		     "width",     GST_PROPS_INT (space->width),
+		     "height",    GST_PROPS_INT (space->height),
+		     "framerate", GST_PROPS_FLOAT (space->fps)
+		  ), GST_CAPS_NEW (
+		   "colorspace_filter",
+		   "video/x-raw-rgb",
+		     "width",     GST_PROPS_INT (space->width),
+		     "height",    GST_PROPS_INT (space->height),
+		     "framerate", GST_PROPS_FLOAT (space->fps)
+		  )));
 
   /* we are looping over the caps, so we have to get rid of the lists */
   peercaps = gst_caps_normalize (peercaps);
@@ -381,16 +367,14 @@ gst_colorspace_class_init (GstColorspaceClass *klass)
 static void
 gst_colorspace_init (GstColorspace *space)
 {
-  space->sinkpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (colorspace_sink_template_factory), "sink");
+  space->sinkpad = gst_pad_new_from_template (sinktempl, "sink");
   gst_pad_set_link_function (space->sinkpad, gst_colorspace_sinkconnect);
   gst_pad_set_getcaps_function (space->sinkpad, gst_colorspace_getcaps);
   gst_pad_set_bufferpool_function (space->sinkpad, colorspace_get_bufferpool);
   gst_pad_set_chain_function(space->sinkpad,gst_colorspace_chain);
   gst_element_add_pad(GST_ELEMENT(space),space->sinkpad);
 
-  space->srcpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (colorspace_src_template_factory), "src");
+  space->srcpad = gst_pad_new_from_template (srctempl, "src");
   gst_element_add_pad(GST_ELEMENT(space),space->srcpad);
   gst_pad_set_link_function (space->srcpad, gst_colorspace_srcconnect);
 
@@ -399,8 +383,9 @@ gst_colorspace_init (GstColorspace *space)
 }
 
 static void
-gst_colorspace_chain (GstPad *pad,GstBuffer *buf)
+gst_colorspace_chain (GstPad *pad,GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstColorspace *space;
   GstBuffer *outbuf = NULL;
 
@@ -449,7 +434,7 @@ gst_colorspace_chain (GstPad *pad,GstBuffer *buf)
 
     gst_buffer_unref (buf);
   }
-  gst_pad_push (space->srcpad, outbuf);
+  gst_pad_push (space->srcpad, GST_DATA (outbuf));
 }
 
 static GstElementStateReturn
@@ -508,17 +493,44 @@ static gboolean
 plugin_init (GModule *module, GstPlugin *plugin)
 {
   GstElementFactory *factory;
+  GstCaps *caps;
 
   lcs_init (NULL, NULL);
 
   factory = gst_element_factory_new ("colorspacelcs", GST_TYPE_COLORSPACE,
                                     &colorspace_details);
   g_return_val_if_fail (factory != NULL, FALSE);
-  
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (colorspace_src_template_factory));
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (colorspace_sink_template_factory));
+
+  /* create caps for templates */
+  caps = gst_caps_new ("csp_templ_yuv",
+                       "video/x-raw-yuv",
+	               GST_VIDEO_YUV_PAD_TEMPLATE_PROPS (
+                         GST_PROPS_LIST (
+                           GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
+	                   GST_PROPS_FOURCC (GST_STR_FOURCC ("YV12")),
+	                   GST_PROPS_FOURCC (GST_STR_FOURCC ("YUY2")))));
+  caps = gst_caps_append (caps,
+         gst_caps_new ("csp_templ_rgb24_32",
+                       "video/x-raw-rgb",
+                       GST_VIDEO_RGB_PAD_TEMPLATE_PROPS_24_32));
+  caps = gst_caps_append (caps,
+         gst_caps_new ("csp_templ_rgb15_16",
+                       "video/x-raw-rgb",
+                       GST_VIDEO_RGB_PAD_TEMPLATE_PROPS_15_16));
+
+  /* build templates */
+  srctempl  = gst_pad_template_new ("src",
+				    GST_PAD_SRC,
+				    GST_PAD_ALWAYS,
+				    caps, NULL);
+  gst_caps_ref (caps);
+  sinktempl = gst_pad_template_new ("sink",
+				    GST_PAD_SINK,
+				    GST_PAD_ALWAYS,
+				    caps, NULL);
+
+  gst_element_factory_add_pad_template (factory, srctempl);
+  gst_element_factory_add_pad_template (factory, sinktempl);
 
   gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
 
@@ -531,11 +543,3 @@ GstPluginDesc plugin_desc = {
   "colorspacelcs",
   plugin_init
 };
-
-
-
-
-
-
-
-
