@@ -17,8 +17,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gst/gst.h>
 
+GST_DEBUG_CATEGORY_EXTERN (GST_CAT_SEEK);
+
 #include <string.h>
 #include <mad.h>
 #include "id3tag.h"
@@ -50,16 +55,17 @@ struct _GstMad {
   guchar	*tempbuffer;
   glong		tempsize;	/* used to keep track of partial buffers */
   gboolean	need_sync;
-  guint64	base_time;
-  guint64	framestamp;	/* timestamp-like, but counted in frames */
+  GstClockTime  last_ts;
+  guint64       base_byte_offset;
+  guint64       bytes_consumed;   /* since the base_byte_offset */
   guint64	total_samples;  /* the number of samples since the sync point */
 
   gboolean	restart;
+  guint64       segment_start;
 
   /* info */
   struct mad_header header;
   gboolean	new_header;
-  gboolean	can_seek;
   guint		framecount;
   gint		vbr_average; /* average bitrate */
   guint64	vbr_rate; /* average * framecount */
@@ -73,6 +79,9 @@ struct _GstMad {
   /* negotiated format */
   gint		rate;
   gint		channels;
+
+  GstIndex	*index;
+  gint		 index_id;
 };
 
 struct _GstMadClass {
@@ -112,9 +121,7 @@ GST_PAD_TEMPLATE_FACTORY (mad_src_template_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "mad_src",
-    "audio/raw",
-      "format",   GST_PROPS_STRING ("int"),
-      "law",         GST_PROPS_INT (0),
+    "audio/x-raw-int",
       "endianness",  GST_PROPS_INT (G_BYTE_ORDER),
       "signed",      GST_PROPS_BOOLEAN (TRUE),
       "width",       GST_PROPS_INT (16),
@@ -130,8 +137,10 @@ GST_PAD_TEMPLATE_FACTORY (mad_sink_template_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "mad_sink",
-    "audio/x-mp3",
-    NULL
+    "audio/mpeg",
+      /* we don't need channel/rate ... */
+      "mpegversion", GST_PROPS_INT (1),
+      "layer", GST_PROPS_INT_RANGE (1, 3)
   )
 )
 
@@ -162,11 +171,14 @@ static gboolean	gst_mad_convert_src	(GstPad *pad, GstFormat src_format,
 					 gint64 src_value, GstFormat
 					 *dest_format, gint64 *dest_value);
 
-static void	gst_mad_chain		(GstPad *pad, GstBuffer *buffer);
+static void	gst_mad_chain		(GstPad *pad, GstData *_data);
 
 static GstElementStateReturn
 		gst_mad_change_state (GstElement *element);
 
+static void      gst_mad_set_index (GstElement *element, GstIndex *index);
+static GstIndex* gst_mad_get_index (GstElement *element);
+
 
 static GstElementClass *parent_class = NULL;
 /* static guint gst_mad_signals[LAST_SIGNAL] = { 0 }; */
@@ -261,6 +273,8 @@ gst_mad_class_init (GstMadClass *klass)
   gobject_class->dispose = gst_mad_dispose;
 
   gstelement_class->change_state = gst_mad_change_state;
+  gstelement_class->set_index    = gst_mad_set_index;
+  gstelement_class->get_index    = gst_mad_get_index;
 
   /* init properties */
   /* currently, string representations are used, we might want to change that */
@@ -304,14 +318,15 @@ gst_mad_init (GstMad *mad)
   mad->tempbuffer = g_malloc (MAD_BUFFER_MDLEN * 3);
   mad->tempsize = 0;
   mad->need_sync = TRUE;
-  mad->base_time = 0;
-  mad->framestamp = 0;
+  mad->base_byte_offset = 0;
+  mad->bytes_consumed = 0;
   mad->total_samples = 0;
   mad->new_header = TRUE;
   mad->framecount = 0;
   mad->vbr_average = 0;
   mad->vbr_rate = 0;
   mad->restart = FALSE;
+  mad->segment_start = 0;
   mad->header.mode = -1;
   mad->header.emphasis = -1;
   mad->metadata = NULL;
@@ -328,17 +343,38 @@ gst_mad_dispose (GObject *object)
 {
   GstMad *mad = GST_MAD (object);
 
+  gst_mad_set_index (GST_ELEMENT (object), NULL);
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 
   g_free (mad->tempbuffer);
 }
 
+static void
+gst_mad_set_index (GstElement *element, GstIndex *index)
+{
+  GstMad *mad = GST_MAD (element);
+  
+  mad->index = index;
+
+  if (index)
+    gst_index_get_writer_id (index, GST_OBJECT (element), &mad->index_id);
+}
+
+static GstIndex*
+gst_mad_get_index (GstElement *element)
+{
+  GstMad *mad = GST_MAD (element);
+
+  return mad->index;
+}
+
 static const GstFormat*
 gst_mad_get_formats (GstPad *pad)
 {
   static const GstFormat src_formats[] = {
     GST_FORMAT_BYTES,
-    GST_FORMAT_UNITS,
+    GST_FORMAT_DEFAULT,
     GST_FORMAT_TIME,
     0
   };
@@ -377,8 +413,6 @@ gst_mad_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
           /* multiply by 8 because vbr is in bits/second */
           *dest_value = src_value * 8 * GST_SECOND / mad->vbr_average;
@@ -389,8 +423,6 @@ gst_mad_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
           /* multiply by 8 because vbr is in bits/second */
           *dest_value = src_value * mad->vbr_average / (8 * GST_SECOND);
@@ -421,13 +453,11 @@ gst_mad_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  if (bytes_per_sample == 0)
             return FALSE;
 	  *dest_value = src_value / bytes_per_sample;
           break;
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	{
           gint byterate = bytes_per_sample * mad->frame.header.samplerate;
@@ -441,13 +471,11 @@ gst_mad_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
           res = FALSE;
       }
       break;
-    case GST_FORMAT_UNITS:
+    case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
 	  *dest_value = src_value * bytes_per_sample;
 	  break;
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	  if (mad->frame.header.samplerate == 0)
             return FALSE;
@@ -459,12 +487,10 @@ gst_mad_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
 	  scale = bytes_per_sample;
 	  /* fallthrough */
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  *dest_value = src_value * scale * mad->frame.header.samplerate / GST_SECOND;
           break;
         default:
@@ -501,11 +527,8 @@ gst_mad_src_query (GstPad *pad, GstQueryType type,
     case GST_QUERY_TOTAL:
     {
       switch (*format) {
-	case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-	  /* fallthrough */
 	case GST_FORMAT_BYTES:
-	case GST_FORMAT_UNITS:
+	case GST_FORMAT_DEFAULT:
 	case GST_FORMAT_TIME:
         {
 	  gint64 peer_value;
@@ -546,21 +569,11 @@ gst_mad_src_query (GstPad *pad, GstQueryType type,
     }
     case GST_QUERY_POSITION:
       switch (*format) {
-	case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-	  /* fall through */
 	default:
 	{
-          GstFormat time_format;
-	  gint64 samples;
-
-          time_format = GST_FORMAT_UNITS;
-	  res = gst_pad_convert (pad,
-			GST_FORMAT_TIME, mad->base_time,
-			&time_format, &samples);
 	  /* we only know about our samples, convert to requested format */
 	  res &= gst_pad_convert (pad,
-			  GST_FORMAT_UNITS, mad->total_samples + samples,
+			  GST_FORMAT_DEFAULT, mad->total_samples,
 			  format, value);
 	  break;
 	}
@@ -573,6 +586,122 @@ gst_mad_src_query (GstPad *pad, GstQueryType type,
   return res;
 }
 
+static gboolean
+index_seek (GstMad *mad, GstPad *pad, GstEvent *event)
+{
+  /* since we know the exact byteoffset of the frame,
+     make sure to try bytes first */
+
+  const GstFormat try_all_formats[] = { 
+    GST_FORMAT_BYTES,
+    GST_FORMAT_TIME,
+    0
+  };
+  const GstFormat *try_formats = try_all_formats;
+  const GstFormat *peer_formats;
+
+  GstIndexEntry *entry =
+    gst_index_get_assoc_entry (mad->index, mad->index_id,
+			       GST_INDEX_LOOKUP_BEFORE, 0,
+			       GST_EVENT_SEEK_FORMAT (event),
+			       GST_EVENT_SEEK_OFFSET (event));
+  if (!entry)
+    return FALSE;
+  
+  peer_formats = gst_pad_get_formats (GST_PAD_PEER (mad->sinkpad));
+
+  while (gst_formats_contains (peer_formats, *try_formats)) {
+    gint64 value;
+    GstEvent *seek_event;
+    
+    if (gst_index_entry_assoc_map (entry, *try_formats, &value)) {
+      /* lookup succeeded, create the seek */
+
+      GST_CAT_DEBUG (GST_CAT_SEEK, "index %s %" G_GINT64_FORMAT
+		     " -> %s %" G_GINT64_FORMAT,
+		     gst_format_get_details (GST_EVENT_SEEK_FORMAT (event))->nick,
+		     GST_EVENT_SEEK_OFFSET (event),
+		     gst_format_get_details (*try_formats)->nick,
+		     value);
+
+      seek_event = gst_event_new_seek (*try_formats |
+				       GST_SEEK_METHOD_SET |
+				       GST_SEEK_FLAG_FLUSH, value);
+
+      if (gst_pad_send_event (GST_PAD_PEER (mad->sinkpad), seek_event)) {
+	/* seek worked, we're done, loop will exit */
+	mad->restart = TRUE;
+	g_assert (GST_EVENT_SEEK_FORMAT (event) == GST_FORMAT_TIME);
+	mad->segment_start = GST_EVENT_SEEK_OFFSET (event);
+	return TRUE;
+      }
+    }
+    try_formats++;
+  }
+
+  return FALSE;
+}
+
+static gboolean
+normal_seek (GstMad *mad, GstPad *pad, GstEvent *event)
+{
+  gint64 src_offset;
+  gboolean flush;
+  GstFormat format;
+  const GstFormat *peer_formats;
+  gboolean res;
+  
+  format = GST_FORMAT_TIME;
+  
+  /* first bring the src_format to TIME */
+  if (!gst_pad_convert (pad,
+			GST_EVENT_SEEK_FORMAT (event), GST_EVENT_SEEK_OFFSET (event),
+			&format, &src_offset))
+  {
+    /* didn't work, probably unsupported seek format then */
+    return FALSE;
+  }
+  
+  /* shave off the flush flag, we'll need it later */
+  flush = GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH;
+  
+  /* assume the worst */
+  res = FALSE;
+  
+  peer_formats = gst_pad_get_formats (GST_PAD_PEER (mad->sinkpad));
+  
+  /* while we did not exhaust our seek formats without result */
+  while (peer_formats && *peer_formats && !res) {
+    gint64 desired_offset;
+    
+    format = *peer_formats;
+    
+    /* try to convert requested format to one we can seek with on the sinkpad */
+    if (gst_pad_convert (mad->sinkpad, GST_FORMAT_TIME, src_offset,
+			 &format, &desired_offset))
+      {
+	GstEvent *seek_event;
+	
+	/* conversion succeeded, create the seek */
+	seek_event = gst_event_new_seek (format | GST_SEEK_METHOD_SET | flush,
+					 desired_offset);
+	/* do the seek */
+	if (gst_pad_send_event (GST_PAD_PEER (mad->sinkpad), seek_event)) {
+	  /* seek worked, we're done, loop will exit */
+	  res = TRUE;
+	}
+      }
+    /* at this point, either the seek worked or res == FALSE */
+    if (res)
+      /* we need to break out of the processing loop on flush */
+      mad->restart = flush;
+    
+    peer_formats++;
+  }
+
+  return res;
+}
+
 static gboolean
 gst_mad_src_event (GstPad *pad, GstEvent *event)
 {
@@ -591,62 +720,12 @@ gst_mad_src_event (GstPad *pad, GstEvent *event)
       break;
     /* the all-formats seek logic */
     case GST_EVENT_SEEK:
-    {
-      gint64 src_offset;
-      gboolean flush;
-      GstFormat format;
-      const GstFormat *peer_formats;
-
-      format = GST_FORMAT_TIME;
-
-      /* first bring the src_format to TIME */
-      if (!gst_pad_convert (pad,
-		GST_EVENT_SEEK_FORMAT (event), GST_EVENT_SEEK_OFFSET (event),
-		&format, &src_offset))
-      {
-	/* didn't work, probably unsupported seek format then */
-        res = FALSE;
-        break;
-      }
-
-      /* shave off the flush flag, we'll need it later */
-      flush = GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH;
-
-      /* assume the worst */
-      res = FALSE;
-
-      peer_formats = gst_pad_get_formats (GST_PAD_PEER (mad->sinkpad));
-
-      /* while we did not exhaust our seek formats without result */
-      while (peer_formats && *peer_formats && !res) {
-        gint64 desired_offset;
-
-	format = *peer_formats;
-
-        /* try to convert requested format to one we can seek with on the sinkpad */
-        if (gst_pad_convert (mad->sinkpad, GST_FORMAT_TIME, src_offset,
-			     &format, &desired_offset))
-        {
-          GstEvent *seek_event;
-
-	  /* conversion succeeded, create the seek */
-          seek_event = gst_event_new_seek (format | GST_SEEK_METHOD_SET | flush,
-			                   desired_offset);
-	  /* do the seek */
-          if (gst_pad_send_event (GST_PAD_PEER (mad->sinkpad), seek_event)) {
-	    /* seek worked, we're done, loop will exit */
-	    res = TRUE;
-	  }
-        }
-	/* at this point, either the seek worked or res == FALSE */
-	if (res)
-          /* we need to break out of the processing loop on flush */
-          mad->restart = flush;
-
-	peer_formats++;
-      }
+      if (mad->index)
+	res = index_seek (mad, pad, event);
+      else
+	res = normal_seek (mad, pad, event);
       break;
-    }
+
     default:
       res = FALSE;
       break;
@@ -794,10 +873,7 @@ G_STMT_START{							\
   mad->new_header = FALSE;
 
   if (changed) {
-    if (mad->streaminfo) {
-      gst_caps_unref (mad->streaminfo);
-    }
-    mad->streaminfo = gst_mad_get_streaminfo (mad);
+    gst_caps_replace_sink (&mad->streaminfo, gst_mad_get_streaminfo (mad));
     g_object_notify (G_OBJECT (mad), "streaminfo");
   }
 #undef CHECK_HEADER
@@ -951,6 +1027,8 @@ gst_mad_handle_event (GstPad *pad, GstBuffer *buffer)
       gint n = GST_EVENT_DISCONT_OFFSET_LEN (event);
       gint i;
 
+      mad->total_samples = 0;
+
       for (i = 0; i < n; i++)
       {
 	const GstFormat *formats;
@@ -972,9 +1050,14 @@ gst_mad_handle_event (GstPad *pad, GstBuffer *buffer)
             time = 0;
           }
 
-          /* for now, this is the best we can do. Let's hope a real timestamp
-           * arrives with the next buffer */
-          mad->base_time = time;
+          /* for now, this is the best we can do to get the total number
+	   * of samples */
+          format = GST_FORMAT_DEFAULT;
+          if (!gst_pad_convert (mad->srcpad,
+		                GST_FORMAT_TIME, time, &format, &mad->total_samples))
+          {
+            mad->total_samples = 0;
+          }
 
           gst_event_unref (event);
 
@@ -982,12 +1065,11 @@ gst_mad_handle_event (GstPad *pad, GstBuffer *buffer)
 	  {
             discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME,
 			                           time, NULL);
-            gst_pad_push (mad->srcpad, GST_BUFFER (discont));
+            gst_pad_push (mad->srcpad, GST_DATA (discont));
           }
           break;
         }
       }
-      mad->total_samples = 0;
       mad->tempsize = 0;
       /* we don't need to restart when we get here */
       mad->restart = FALSE;
@@ -1000,13 +1082,26 @@ gst_mad_handle_event (GstPad *pad, GstBuffer *buffer)
   return;
 }
 
-static void
-gst_mad_chain (GstPad *pad, GstBuffer *buffer)
+static gboolean
+gst_mad_check_restart (GstMad *mad)
 {
+  gboolean yes = mad->restart;
+  if (mad->restart) {
+    mad->restart = FALSE;
+    mad->tempsize = 0;
+  }
+  return yes;
+}
+
+static void
+gst_mad_chain (GstPad *pad, GstData *_data)
+{
+  GstBuffer *buffer = GST_BUFFER (_data);
   GstMad *mad;
   gchar *data;
   glong size;
   gboolean new_pts = FALSE;
+  GstClockTime timestamp;
 
   mad = GST_MAD (gst_pad_get_parent (pad));
   g_return_if_fail (GST_IS_MAD (mad));
@@ -1018,13 +1113,20 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
     return;
   }
 
+  gst_mad_check_restart (mad);
+
+  timestamp = GST_BUFFER_TIMESTAMP (buffer);
+
   /* handle timestamps */
-  if (GST_BUFFER_TIMESTAMP (buffer) != -1) {
+  if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
     /* if there is nothing queued (partial buffer), we prepare to set the
      * timestamp on the next buffer */
     if (mad->tempsize == 0) {
-      mad->base_time = GST_BUFFER_TIMESTAMP (buffer);
-      mad->total_samples = 0;
+      /* we have to save the result here because we can't yet convert the timestamp
+       * to a sample offset yet, the samplerate might not be known yet */
+      mad->last_ts = timestamp;
+      mad->base_byte_offset = GST_BUFFER_OFFSET (buffer);
+      mad->bytes_consumed = 0;
     }
     /* else we need to finish the current partial frame with the old timestamp
      * and queue this timestamp for the next frame */
@@ -1062,6 +1164,8 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
       guint nchannels;
       guint nsamples;
       gint rate;
+      guint64 time_offset;
+      guint64 time_duration;
 
       mad_stream_buffer (&mad->stream, mad_input_buffer, mad->tempsize);
 
@@ -1082,15 +1186,13 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
 	                          mad->stream.bufend - mad->stream.this_frame);
 
 	  if (tagsize > mad->tempsize) {
-            GST_INFO (GST_CAT_PLUGIN_INFO,
-		      "mad: got partial id3 tag in buffer, skipping");
+            GST_INFO ("mad: got partial id3 tag in buffer, skipping");
 	  }
 	  else if (tagsize > 0) {
 	    struct id3_tag *tag;
 	    id3_byte_t const *data;
 
-            GST_INFO (GST_CAT_PLUGIN_INFO,
-		      "mad: got ID3 tag size %ld", tagsize);
+            GST_INFO ("mad: got ID3 tag size %ld", tagsize);
 
 	    data = mad->stream.this_frame;
 
@@ -1100,7 +1202,7 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
 
 	    tag = id3_tag_parse (data, tagsize);
 	    if (tag) {
-	      mad->metadata = id3_to_caps (tag);
+              gst_caps_replace_sink (&mad->metadata, id3_to_caps (tag));
 	      id3_tag_delete (tag);
 	      g_object_notify (G_OBJECT (mad), "metadata");
 	    }
@@ -1123,9 +1225,6 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
       rate = mad->frame.header.samplerate;
 #endif
 
-      /* at this point we can accept seek events */
-      mad->can_seek = TRUE;
-
       gst_mad_update_info (mad);
 
       if (mad->channels != nchannels || mad->rate != rate) {
@@ -1137,10 +1236,8 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
         if (gst_pad_try_set_caps (mad->srcpad,
 	    gst_caps_new (
 	      "mad_src",
-              "audio/raw",
+              "audio/x-raw-int",
               gst_props_new (
-	        "format",   GST_PROPS_STRING ("int"),
-                "law",         GST_PROPS_INT (0),
                 "endianness",  GST_PROPS_INT (G_BYTE_ORDER),
                 "signed",      GST_PROPS_BOOLEAN (TRUE),
                 "width",       GST_PROPS_INT (16),
@@ -1149,16 +1246,48 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
                 "channels",    GST_PROPS_INT (nchannels),
                 NULL))) <= 0)
 	{
-	  gst_buffer_unref (buffer);
-          gst_element_error (GST_ELEMENT (mad),
-			     "could not set caps on source pad, aborting...");
-	  return;
+	  if (!gst_pad_recover_caps_error (mad->srcpad, NULL)) {
+	    gst_buffer_unref (buffer);
+	    return;
+	  }
         }
 	mad->channels = nchannels;
 	mad->rate = rate;
       }
 
-      if (GST_PAD_IS_USABLE (mad->srcpad)) {
+      if (mad->frame.header.samplerate == 0) {
+	g_warning ("mad->frame.header.samplerate is 0; timestamps cannot be calculated");
+	time_offset = GST_CLOCK_TIME_NONE;
+	time_duration = GST_CLOCK_TIME_NONE;
+      }
+      else {
+	/* if we have a pending timestamp, we can use it now to calculate the sample offset */
+	if (GST_CLOCK_TIME_IS_VALID (mad->last_ts)) {
+          GstFormat format = GST_FORMAT_DEFAULT;
+          gst_pad_convert (mad->srcpad, GST_FORMAT_TIME, mad->last_ts, &format, &mad->total_samples);
+	  mad->last_ts = GST_CLOCK_TIME_NONE;
+	}
+	time_offset = mad->total_samples * GST_SECOND
+					/ mad->frame.header.samplerate;
+	time_duration = (nsamples * GST_SECOND
+			 / mad->frame.header.samplerate);
+      }
+
+      if (mad->index) {
+	guint64 x_bytes = mad->base_byte_offset + mad->bytes_consumed;
+
+	gst_index_add_association (mad->index, mad->index_id, 0,
+				   GST_FORMAT_BYTES, x_bytes,
+				   GST_FORMAT_TIME, time_offset,
+				   NULL);
+      }
+
+      if (GST_PAD_IS_USABLE (mad->srcpad) &&
+	  mad->segment_start <= (time_offset == GST_CLOCK_TIME_NONE ? 0 : time_offset)) {
+
+	/* for sample accurate seeking, calculate how many samples
+	   to skip and send the remaining pcm samples */
+
         GstBuffer *outbuffer;
         gint16 *outdata;
         mad_fixed_t const *left_ch, *right_ch;
@@ -1170,15 +1299,9 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
         outbuffer = gst_buffer_new_and_alloc (nsamples * nchannels * 2);
         outdata = (gint16 *) GST_BUFFER_DATA (outbuffer);
 
-        if (mad->frame.header.samplerate == 0) {
-	  g_warning ("mad->frame.header.samplerate is 0 !");
-        }
-        else {
-	  guint64 increase;
-	  increase = mad->total_samples * GST_SECOND
-			                / mad->frame.header.samplerate;
-	  GST_BUFFER_TIMESTAMP (outbuffer) = mad->base_time + increase;
-        }
+	GST_BUFFER_TIMESTAMP (outbuffer) = time_offset;
+	GST_BUFFER_DURATION (outbuffer) = time_duration;
+	GST_BUFFER_OFFSET (outbuffer) = mad->total_samples;
 
         /* output sample(s) in 16-bit signed native-endian PCM */
         if (nchannels == 1) {
@@ -1195,7 +1318,7 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
           }
         }
 
-        gst_pad_push (mad->srcpad, outbuffer);
+        gst_pad_push (mad->srcpad, GST_DATA (outbuffer));
       }
 
       mad->total_samples += nsamples;
@@ -1203,14 +1326,13 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
       /* we have a queued timestamp on the incoming buffer that we should
        * use for the next frame */
       if (new_pts) {
+	mad->last_ts = timestamp;
 	new_pts = FALSE;
-        mad->base_time = GST_BUFFER_TIMESTAMP (buffer);
-        mad->total_samples = 0;
+	mad->base_byte_offset = GST_BUFFER_OFFSET (buffer);
+	mad->bytes_consumed = 0;
       }
 
-      if (mad->restart) {
-	mad->restart = FALSE;
-	mad->tempsize = 0;
+      if (gst_mad_check_restart (mad)) {
 	goto end;
       }
 
@@ -1221,6 +1343,7 @@ next:
       /* move out pointer to where mad want the next data */
       mad_input_buffer += consumed;
       mad->tempsize -= consumed;
+      mad->bytes_consumed += consumed;
     }
     memmove (mad->tempbuffer, mad_input_buffer, mad->tempsize);
   }
@@ -1247,17 +1370,16 @@ gst_mad_change_state (GstElement *element)
       mad_frame_init (&mad->frame);
       mad_synth_init (&mad->synth);
       mad->tempsize = 0;
-      mad->can_seek = FALSE;
       mad->total_samples = 0;
       mad->rate = 0;
       mad->channels = 0;
       mad->vbr_average = 0;
-      mad->base_time = 0;
-      mad->framestamp = 0;
+      mad->segment_start = 0;
       mad->new_header = TRUE;
       mad->framecount = 0;
       mad->vbr_rate = 0;
       mad->frame.header.samplerate = 0;
+      mad->last_ts = GST_CLOCK_TIME_NONE;
       if (mad->ignore_crc) options |= MAD_OPTION_IGNORECRC;
       if (mad->half) options |= MAD_OPTION_HALFSAMPLERATE;
       mad_stream_options (&mad->stream, options);
@@ -1273,16 +1395,14 @@ gst_mad_change_state (GstElement *element)
       mad_frame_finish (&mad->frame);
       mad_stream_finish (&mad->stream);
       mad->restart = TRUE;
-      mad->metadata = NULL;
-      mad->streaminfo = NULL;
+      gst_caps_replace (&mad->metadata, NULL);
+      gst_caps_replace (&mad->streaminfo, NULL);
       break;
     case GST_STATE_READY_TO_NULL:
       break;
   }
 
-  parent_class->change_state (element);
-
-  return GST_STATE_SUCCESS;
+  return parent_class->change_state (element);
 }
 
 static gboolean
