@@ -17,24 +17,27 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 
 #include <inttypes.h>
 
 #include "gstmpeg2dec.h"
- 
+
 /* mpeg2dec changed a struct name after 0.3.1, here's a workaround */
 /* mpeg2dec also only defined MPEG2_RELEASE after 0.3.1
 #if MPEG2_RELEASE < MPEG2_VERSION(0,3,2)
 */
 #ifndef MPEG2_RELEASE
-#define MPEG2_VERSION(a,b,c) ((((a)&0xff)<<16)|(((b)&0xff)<<8)|((c)&0xff))
-#define MPEG2_RELEASE MPEG2_VERSION(0,3,1)
 typedef picture_t mpeg2_picture_t;
 typedef gint mpeg2_state_t;
 #define STATE_BUFFER 0
 #endif
 
+GST_DEBUG_CATEGORY_EXTERN (GST_CAT_SEEK);
+
 /* elementfactory information */
 static GstElementDetails gst_mpeg2dec_details = {
   "mpeg1 and mpeg2 video decoder",
@@ -64,21 +67,25 @@ GST_PAD_TEMPLATE_FACTORY (src_template_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "mpeg2dec_src",
-    "video/raw",
-      "format",       GST_PROPS_FOURCC (GST_MAKE_FOURCC ('Y','V','1','2')),
+    "video/x-raw-yuv",
+      "format",       GST_PROPS_LIST (
+	GST_PROPS_FOURCC (GST_MAKE_FOURCC ('Y','V','1','2')),
+	GST_PROPS_FOURCC (GST_MAKE_FOURCC ('I','4','2','0'))
+		      ),
       "width",        GST_PROPS_INT_RANGE (16, 4096),
       "height",       GST_PROPS_INT_RANGE (16, 4096),
       "pixel_width",  GST_PROPS_INT_RANGE (1, 255),
-      "pixel_height", GST_PROPS_INT_RANGE (1, 255)
-  ),
-  GST_CAPS_NEW (
-    "mpeg2dec_src",
-    "video/raw",
-      "format",       GST_PROPS_FOURCC (GST_MAKE_FOURCC ('I','4','2','0')),
-      "width",        GST_PROPS_INT_RANGE (16, 4096),
-      "height",       GST_PROPS_INT_RANGE (16, 4096),
-      "pixel_width",  GST_PROPS_INT_RANGE (1, 255),
-      "pixel_height", GST_PROPS_INT_RANGE (1, 255)
+      "pixel_height", GST_PROPS_INT_RANGE (1, 255),
+      "framerate",    GST_PROPS_LIST (
+			GST_PROPS_FLOAT (24/1.001),
+			GST_PROPS_FLOAT (24.),
+			GST_PROPS_FLOAT (25.),
+			GST_PROPS_FLOAT (30/1.001),
+			GST_PROPS_FLOAT (30.),
+			GST_PROPS_FLOAT (50.),
+			GST_PROPS_FLOAT (60/1.001),
+			GST_PROPS_FLOAT (60.)
+		      )
   )
 );
 
@@ -100,6 +107,7 @@ GST_PAD_TEMPLATE_FACTORY (sink_template_factory,
   GST_CAPS_NEW (
     "mpeg2dec_sink",
     "video/mpeg",
+      /* width/height/framerate not needed */
       "mpegversion",  GST_PROPS_INT_RANGE (1, 2),
       "systemstream", GST_PROPS_BOOLEAN (FALSE)
   )
@@ -137,7 +145,7 @@ static gboolean 	gst_mpeg2dec_convert_src 	(GstPad *pad, GstFormat src_format, g
 static GstElementStateReturn
 			gst_mpeg2dec_change_state	(GstElement *element);
 
-static void		gst_mpeg2dec_chain		(GstPad *pad, GstBuffer *buffer);
+static void		gst_mpeg2dec_chain		(GstPad *pad, GstData *_data);
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_mpeg2dec_signals[LAST_SIGNAL] = { 0 };*/
@@ -317,7 +325,12 @@ static gboolean
 gst_mpeg2dec_negotiate_format (GstMpeg2dec *mpeg2dec)
 {
   GstCaps *allowed;
-  GstCaps *trylist;
+  GstCaps *intersect, *trylist, *head, *to_intersect;
+
+  if (!GST_PAD_IS_LINKED (mpeg2dec->srcpad)) {
+    mpeg2dec->format = MPEG2DEC_FORMAT_I420;
+    return TRUE;
+  }
 
   /* we what we are allowed to do */
   allowed = gst_pad_get_allowed_caps (mpeg2dec->srcpad);
@@ -325,23 +338,29 @@ gst_mpeg2dec_negotiate_format (GstMpeg2dec *mpeg2dec)
   if (!allowed) {
     allowed = GST_CAPS_NEW (
                 "mpeg2dec_negotiate",
-                "video/raw",
+                "video/x-raw-yuv",
                	  "format",        GST_PROPS_FOURCC (GST_STR_FOURCC ("I420"))
 	      );
   }
 
+  to_intersect = GST_CAPS_NEW (
+                   "mpeg2dec_negotiate",
+                      "video/x-raw-yuv",
+                      "width",        GST_PROPS_INT (mpeg2dec->width),
+ 	              "height",       GST_PROPS_INT (mpeg2dec->height),
+                      "pixel_width",  GST_PROPS_INT (mpeg2dec->pixel_width),
+ 	              "pixel_height", GST_PROPS_INT (mpeg2dec->pixel_height),
+		      "framerate",    GST_PROPS_FLOAT (1. * GST_SECOND / mpeg2dec->frame_period)
+                 );
+
   /* try to fix our height */
-  trylist = gst_caps_intersect (allowed,
-                                GST_CAPS_NEW (
-                                  "mpeg2dec_negotiate",
-                                     "video/raw",
-                          	     "width",        GST_PROPS_INT (mpeg2dec->width),
- 	                             "height",       GST_PROPS_INT (mpeg2dec->height),
-                          	     "pixel_width",  GST_PROPS_INT (mpeg2dec->pixel_width),
- 	                             "pixel_height", GST_PROPS_INT (mpeg2dec->pixel_height)
-                                ));
+  intersect = gst_caps_intersect (allowed, to_intersect);
+  gst_caps_unref (allowed);
+  gst_caps_unref (to_intersect);
+
   /* prepare for looping */
-  trylist = gst_caps_normalize (trylist);
+  head = trylist = gst_caps_normalize (intersect);
+  gst_caps_unref (intersect);
 
   while (trylist) {
     GstCaps *to_try = gst_caps_copy_1 (trylist);
@@ -361,8 +380,11 @@ gst_mpeg2dec_negotiate_format (GstMpeg2dec *mpeg2dec)
       }
       break;
     }
+
     trylist = trylist->next;
   }
+  gst_caps_unref (head);
+
   /* oops list exhausted and nothing was found... */
   if (!trylist) {
     return FALSE;
@@ -391,10 +413,7 @@ update_streaminfo (GstMpeg2dec *mpeg2dec)
                        "application/x-gst-streaminfo",
                         props);
 
-  if (mpeg2dec->streaminfo)
-    gst_caps_unref (mpeg2dec->streaminfo);
-
-  mpeg2dec->streaminfo = caps;
+  gst_caps_replace_sink (&mpeg2dec->streaminfo, caps);
   g_object_notify (G_OBJECT (mpeg2dec), "streaminfo");
 }
 
@@ -419,8 +438,9 @@ gst_mpeg2dec_flush_decoder (GstMpeg2dec *mpeg2dec)
 }
 
 static void
-gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
+gst_mpeg2dec_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstMpeg2dec *mpeg2dec = GST_MPEG2DEC (gst_pad_get_parent (pad));
   guint32 size;
   guint8 *data, *end;
@@ -435,7 +455,7 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_DISCONTINUOUS:
       {
-	GST_DEBUG (GST_CAT_EVENT, "discont"); 
+	GST_DEBUG ("discont"); 
         mpeg2dec->next_time = 0;
         mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_PICTURE;
 	gst_mpeg2dec_flush_decoder (mpeg2dec);
@@ -462,29 +482,25 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
   if (pts != -1) {
     gint64 mpeg_pts = GSTTIME_TO_MPEGTIME (pts);
 
-    GST_DEBUG (0, "have pts: %lld (%lld)", 
+    GST_DEBUG ("have pts: %" G_GINT64_FORMAT " (%" G_GINT64_FORMAT ")", 
 		  mpeg_pts, MPEGTIME_TO_GSTTIME (mpeg_pts));
 
-#if MPEG2_RELEASE >= MPEG2_VERSION(0,4,0)
-    mpeg2_tag_picture (mpeg2dec->decoder, mpeg_pts&0xffffffff, mpeg_pts>>32);
-#else
     mpeg2_pts (mpeg2dec->decoder, mpeg_pts);
-#endif
   }
   else {
-    GST_DEBUG (GST_CAT_CLOCK, "no pts");
+    GST_DEBUG ("no pts");
   }
 
-  GST_DEBUG (0, "calling _buffer");
+  GST_DEBUG ("calling _buffer");
   mpeg2_buffer (mpeg2dec->decoder, data, end);
-  GST_DEBUG (0, "calling _buffer done");
+  GST_DEBUG ("calling _buffer done");
 
   while (!done) {
     gboolean slice = FALSE;
 
-    GST_DEBUG (0, "calling parse");
+    GST_DEBUG ("calling parse");
     state = mpeg2_parse (mpeg2dec->decoder);
-    GST_DEBUG (0, "parse state %d", state);
+    GST_DEBUG ("parse state %d", state);
     switch (state) {
       case STATE_SEQUENCE:
       {
@@ -495,14 +511,23 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 	mpeg2dec->total_frames = 0;
         mpeg2dec->frame_period = info->sequence->frame_period * GST_USECOND / 27;
 
-	GST_DEBUG (0, "sequence flags: %d, frame period: %d", 
+	GST_DEBUG ("sequence flags: %d, frame period: %d", 
 		      info->sequence->flags, info->sequence->frame_period);
+	GST_DEBUG ("profile: %02x, colour_primaries: %d", 
+		      info->sequence->profile_level_id, info->sequence->colour_primaries);
+	GST_DEBUG ("transfer chars: %d, matrix coef: %d", 
+		      info->sequence->transfer_characteristics, info->sequence->matrix_coefficients);
 
 	if (!gst_mpeg2dec_negotiate_format (mpeg2dec)) {
           gst_element_error (GST_ELEMENT (mpeg2dec), "could not negotiate format");
 	  goto exit;
 	}
 
+	/* now that we've negotiated, try to get a bufferpool */
+	mpeg2dec->peerpool = gst_pad_get_bufferpool (mpeg2dec->srcpad);
+	if (mpeg2dec->peerpool)
+	  GST_INFO ( "got pool %p", mpeg2dec->peerpool);
+
 	update_streaminfo (mpeg2dec);
 
 	if (!mpeg2dec->have_fbuf) {
@@ -523,7 +548,7 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
         break;
       }
       case STATE_SEQUENCE_REPEATED:
-	GST_DEBUG (0, "sequence repeated");
+	GST_DEBUG ("sequence repeated");
       case STATE_GOP:
         break;
       case STATE_PICTURE:
@@ -536,7 +561,7 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 	}
 	outbuf = gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
 
-	GST_DEBUG (0, "picture %d, %p, %lld, %lld", 
+	GST_DEBUG ("picture %d, %p, %" G_GINT64_FORMAT ", %" G_GINT64_FORMAT, 
 			key_frame, outbuf, GST_BUFFER_OFFSET (outbuf), pts);
 
 	if (mpeg2dec->discont_state == MPEG2DEC_DISC_NEW_PICTURE && key_frame)
@@ -550,16 +575,13 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
         break;
       }
       case STATE_SLICE_1ST:
-	GST_DEBUG (0, "slice 1st");
+	GST_DEBUG ("slice 1st");
         break;
       case STATE_PICTURE_2ND:
-	GST_DEBUG (0, "picture second");
+	GST_DEBUG ("picture second");
         break;
       case STATE_SLICE:
 	slice = TRUE;
-#if MPEG2_RELEASE >= MPEG2_VERSION (0, 4, 0)
-      case STATE_INVALID_END:
-#endif
       case STATE_END:
       {
 	GstBuffer *outbuf = NULL;
@@ -568,7 +590,7 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 	if (!slice) {
 	  mpeg2dec->need_sequence = TRUE;
 	}
-	GST_DEBUG (0, "picture end %p %p %p %p", info->display_fbuf, info->display_picture, info->current_picture,
+	GST_DEBUG ("picture end %p %p %p %p", info->display_fbuf, info->display_picture, info->current_picture,
 			(info->display_fbuf ? info->display_fbuf->id : NULL));
 
 	if (info->display_fbuf && info->display_fbuf->id) {
@@ -579,7 +601,7 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 	  picture = info->display_picture;
 
 	  key_frame = (picture->flags & PIC_MASK_CODING_TYPE) == PIC_FLAG_CODING_TYPE_I;
-	  GST_DEBUG (0, "picture keyfame %d", key_frame);
+	  GST_DEBUG ("picture keyfame %d", key_frame);
 
 	  if (key_frame) {
             GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_KEY_UNIT);
@@ -587,27 +609,22 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 	  if (mpeg2dec->discont_state == MPEG2DEC_DISC_NEW_KEYFRAME && key_frame)
 	    mpeg2dec->discont_state = MPEG2DEC_DISC_NONE;
 
-#if MPEG2_RELEASE < MPEG2_VERSION(0,4,0)
 	  if (picture->flags & PIC_FLAG_PTS) {
             GstClockTime time = MPEGTIME_TO_GSTTIME (picture->pts);
-#else
-          if (picture->flags & PIC_FLAG_TAGS) {
-            GstClockTime time = MPEGTIME_TO_GSTTIME ((guint64) picture->tag2 << 32 | picture->tag);
-#endif
 
-	    GST_DEBUG (0, "picture had pts %lld", time);
+	    GST_DEBUG ("picture had pts %" G_GINT64_FORMAT, time);
             GST_BUFFER_TIMESTAMP (outbuf) = time;
 
             mpeg2dec->next_time = time;
 	  }
 	  else {
-	    GST_DEBUG (0, "picture didn't have pts using %lld", mpeg2dec->next_time);
+	    GST_DEBUG ("picture didn't have pts using %" G_GINT64_FORMAT, mpeg2dec->next_time);
             GST_BUFFER_TIMESTAMP (outbuf) = mpeg2dec->next_time;
 	  }
           mpeg2dec->next_time += (mpeg2dec->frame_period * picture->nb_fields) >> 1;
 
 
-	  GST_DEBUG (0, "picture: %s %s fields:%d off:%lld ts:%lld", 
+	  GST_DEBUG ("picture: %s %s fields:%d off:%" G_GINT64_FORMAT " ts:%" G_GINT64_FORMAT, 
 			  (picture->flags & PIC_FLAG_TOP_FIELD_FIRST ? "tff " : "    "),
 			  (picture->flags & PIC_FLAG_PROGRESSIVE_FRAME ? "prog" : "    "),
 			  picture->nb_fields, 
@@ -631,7 +648,9 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 	    gst_buffer_unref (outbuf);
 	  }
 	  else {
-	    gst_pad_push (mpeg2dec->srcpad, outbuf);
+	    /* TODO set correct offset here based on frame number */
+	    GST_BUFFER_DURATION (outbuf) = mpeg2dec->frame_period;
+	    gst_pad_push (mpeg2dec->srcpad, GST_DATA (outbuf));
 	  }
 	}
 	if (info->discard_fbuf && info->discard_fbuf->id) {
@@ -667,7 +686,7 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 
         memcpy (GST_BUFFER_DATA (udbuf), info->user_data, info->user_data_len);
 
-        gst_pad_push (mpeg2dec->userdatapad, udbuf);
+        gst_pad_push (mpeg2dec->userdatapad, GST_DATA (udbuf));
       }
     }
   }
@@ -704,8 +723,6 @@ gst_mpeg2dec_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	  if (info->sequence && info->sequence->byte_rate) {
             *dest_value = GST_SECOND * src_value / info->sequence->byte_rate;
@@ -717,8 +734,6 @@ gst_mpeg2dec_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
 	  if (info->sequence && info->sequence->byte_rate) {
             *dest_value = src_value * info->sequence->byte_rate / GST_SECOND;
@@ -740,7 +755,7 @@ gst_mpeg2dec_get_src_formats (GstPad *pad)
   static const GstFormat formats[] = {
     GST_FORMAT_BYTES,
     GST_FORMAT_TIME,
-    GST_FORMAT_UNITS,
+    GST_FORMAT_DEFAULT,
     0
   };
   return formats;
@@ -765,8 +780,6 @@ gst_mpeg2dec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
         default:
           res = FALSE;
@@ -774,11 +787,9 @@ gst_mpeg2dec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
 	  scale = 6 * (mpeg2dec->width * mpeg2dec->height >> 2);
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  if (info->sequence && mpeg2dec->frame_period) {
             *dest_value = src_value * scale / mpeg2dec->frame_period;
             break;
@@ -787,10 +798,8 @@ gst_mpeg2dec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
           res = FALSE;
       }
       break;
-    case GST_FORMAT_UNITS:
+    case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
           *dest_value = src_value * mpeg2dec->frame_period;
 	  break;
@@ -832,12 +841,9 @@ gst_mpeg2dec_src_query (GstPad *pad, GstQueryType type,
     case GST_QUERY_TOTAL:
     {
       switch (*format) {
-        case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-          /* fallthrough */
         case GST_FORMAT_TIME:
         case GST_FORMAT_BYTES:
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	{
           res = FALSE;
 
@@ -880,9 +886,6 @@ gst_mpeg2dec_src_query (GstPad *pad, GstQueryType type,
     case GST_QUERY_POSITION:
     {
       switch (*format) {
-        case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-          /* fallthrough */
         default:
           res = gst_pad_convert (pad,
                           GST_FORMAT_TIME, mpeg2dec->next_time,
@@ -942,6 +945,13 @@ index_seek (GstPad *pad, GstEvent *event)
       if (gst_index_entry_assoc_map (entry, *try_formats, &value)) {
         GstEvent *seek_event;
 
+	GST_CAT_DEBUG (GST_CAT_SEEK, "index %s %" G_GINT64_FORMAT
+		       " -> %s %" G_GINT64_FORMAT,
+		       gst_format_get_details (GST_EVENT_SEEK_FORMAT (event))->nick,
+		       GST_EVENT_SEEK_OFFSET (event),
+		       gst_format_get_details (*try_formats)->nick,
+		       value);
+
         /* lookup succeeded, create the seek */
         seek_event = gst_event_new_seek (*try_formats | GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH, value);
         /* do the seekk */
@@ -1083,10 +1093,12 @@ gst_mpeg2dec_change_state (GstElement *element)
       break;
     }
     case GST_STATE_PAUSED_TO_PLAYING:
-      /* try to get a bufferpool */
-      mpeg2dec->peerpool = gst_pad_get_bufferpool (mpeg2dec->srcpad);
-      if (mpeg2dec->peerpool)
-        GST_INFO (GST_CAT_PLUGIN_INFO, "got pool %p", mpeg2dec->peerpool);
+      /* if we've negotiated caps, try to get a bufferpool */
+      if (mpeg2dec->peerpool == NULL && mpeg2dec->width > 0) {
+	mpeg2dec->peerpool = gst_pad_get_bufferpool (mpeg2dec->srcpad);
+	if (mpeg2dec->peerpool)
+	  GST_INFO ( "got pool %p", mpeg2dec->peerpool);
+      }
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
       /* need to clear things we get from other plugins, since we could be reconnected */
@@ -1097,6 +1109,7 @@ gst_mpeg2dec_change_state (GstElement *element)
       break;
     case GST_STATE_PAUSED_TO_READY:
       gst_mpeg2dec_close_decoder (mpeg2dec);
+      gst_caps_replace (&mpeg2dec->streaminfo, NULL);
       break;
     case GST_STATE_READY_TO_NULL:
       break;
@@ -1166,5 +1179,3 @@ GstPluginDesc plugin_desc = {
   "mpeg2dec",
   plugin_init
 };
-
-
