@@ -17,17 +17,21 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <inttypes.h>
 
 #include "gstsnapshot.h"
+#include <gst/video/video.h>
 
 #define MAX_HEIGHT	2048
 
 /* elementfactory information */
 static GstElementDetails snapshot_details = {
   "snapshot",
-  "Filter/Effect",
+  "Filter/Video",
   "LGPL",
   "Dump a frame to a png file",
   VERSION,
@@ -39,15 +43,14 @@ GST_PAD_TEMPLATE_FACTORY (snapshot_src_factory,
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
+  gst_caps_new (
    "snapshot_src",
-   "video/raw",
-     "format",     GST_PROPS_LIST (
+   "video/x-raw-yuv",
+     GST_VIDEO_YUV_PAD_TEMPLATE_PROPS (
+		   GST_PROPS_LIST (
                      GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
 		     GST_PROPS_FOURCC (GST_STR_FOURCC ("YUY2"))
-                   ),	          
-     "width",      GST_PROPS_INT_RANGE(0, G_MAXINT),
-     "height",     GST_PROPS_INT_RANGE(0, G_MAXINT) 
+                   )) 
   )
 )
 
@@ -55,15 +58,14 @@ GST_PAD_TEMPLATE_FACTORY (snapshot_sink_factory,
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
+  gst_caps_new (
    "snapshot_src",
-   "video/raw",
-       "format",     GST_PROPS_LIST (
-                       GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
-                       GST_PROPS_FOURCC (GST_STR_FOURCC ("YUY2"))
-                     ),
-       "width",      GST_PROPS_INT_RANGE(0, G_MAXINT),
-       "height",     GST_PROPS_INT_RANGE(0, G_MAXINT)
+   "video/x-raw-yuv",
+     GST_VIDEO_YUV_PAD_TEMPLATE_PROPS (
+		   GST_PROPS_LIST (
+                     GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
+		     GST_PROPS_FOURCC (GST_STR_FOURCC ("YUY2"))
+                   )) 
   )
 )
 
@@ -85,7 +87,7 @@ static GType 	gst_snapshot_get_type 	(void);
 static void	gst_snapshot_class_init	(GstSnapshotClass *klass);
 static void	gst_snapshot_init	(GstSnapshot *snapshot);
 
-static void	gst_snapshot_chain	(GstPad *pad, GstBuffer *buf);
+static void	gst_snapshot_chain	(GstPad *pad, GstData *_data);
 
 static void	gst_snapshot_set_property	(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void	gst_snapshot_get_property	(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
@@ -169,28 +171,30 @@ gst_snapshot_sinkconnect (GstPad *pad, GstCaps *caps)
 {
   GstSnapshot *filter;
   GstCaps *from_caps, *to_caps;
+  gfloat fps;
 
   filter = GST_SNAPSHOT (gst_pad_get_parent (pad));
 
   if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_CONNECT_DELAYED;
+    return GST_PAD_LINK_DELAYED;
 
   gst_caps_get_int (caps, "width", &filter->width);
   gst_caps_get_int (caps, "height", &filter->height);
+  gst_caps_get_float (caps, "framerate", &fps);
   gst_caps_get_fourcc_int (caps, "format", &filter->format);
   filter->to_bpp = 24;
 
 
   to_caps = GST_CAPS_NEW (
     "snapshot_conversion",
-    "video/raw",
-      "format",   GST_PROPS_FOURCC (GST_STR_FOURCC ("RGB ")),
-      "width",    GST_PROPS_INT( filter->width ),
-      "height",   GST_PROPS_INT( filter->height ),
+    "video/x-raw-rgb",
+      "width",      GST_PROPS_INT( filter->width ),
+      "height",     GST_PROPS_INT( filter->height ),
       "red_mask",   GST_PROPS_INT (0x0000FF),
       "green_mask", GST_PROPS_INT (0x00FF00),
       "blue_mask",  GST_PROPS_INT (0xFF0000),
-      "bpp",        GST_PROPS_INT( 24 ) 
+      "bpp",        GST_PROPS_INT( 24 ),
+      "framerate",  GST_PROPS_FLOAT (fps)
   );
 
   switch ( filter->format )
@@ -199,10 +203,11 @@ gst_snapshot_sinkconnect (GstPad *pad, GstCaps *caps)
     case GST_MAKE_FOURCC('I','4','2','0'):
        from_caps = GST_CAPS_NEW (
          "snapshot_from", 
-         "video/raw",
-         "format", GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
-         "width",    GST_PROPS_INT( filter->width ),
-         "height",   GST_PROPS_INT( filter->height )
+         "video/x-raw-yuv",
+         "format",    GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
+         "width",     GST_PROPS_INT( filter->width ),
+         "height",    GST_PROPS_INT( filter->height ),
+	 "framerate", GST_PROPS_FLOAT (fps)
        );
 
        filter->converter = gst_colorspace_yuv2rgb_get_converter ( from_caps, to_caps );
@@ -223,14 +228,14 @@ gst_snapshot_sinkconnect (GstPad *pad, GstCaps *caps)
       
   gst_pad_try_set_caps (filter->srcpad, caps);
 
-  return GST_PAD_CONNECT_OK;
+  return GST_PAD_LINK_OK;
 }
 
 static void
 gst_snapshot_init (GstSnapshot *snapshot)
 {
   snapshot->sinkpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (snapshot_sink_factory), "sink");
-  gst_pad_set_connect_function (snapshot->sinkpad, gst_snapshot_sinkconnect);
+  gst_pad_set_link_function (snapshot->sinkpad, gst_snapshot_sinkconnect);
   gst_pad_set_chain_function (snapshot->sinkpad, gst_snapshot_chain);
   gst_element_add_pad (GST_ELEMENT (snapshot), snapshot->sinkpad);
 
@@ -244,8 +249,9 @@ gst_snapshot_init (GstSnapshot *snapshot)
 
 
 static void
-gst_snapshot_chain (GstPad *pad, GstBuffer *buf)
+gst_snapshot_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstSnapshot *snapshot;
   guchar *data, *data_to_convert, *buffer_i420, *data_converted;
   gulong size,image_size;
@@ -263,7 +269,7 @@ gst_snapshot_chain (GstPad *pad, GstBuffer *buf)
   data = GST_BUFFER_DATA(buf);
   size = GST_BUFFER_SIZE(buf);
 
-  GST_DEBUG (0,"snapshot: have buffer of %d\n", GST_BUFFER_SIZE(buf));
+  GST_DEBUG ("snapshot: have buffer of %d\n", GST_BUFFER_SIZE(buf));
 
   outbuf = gst_buffer_new();
   GST_BUFFER_DATA(outbuf) = g_malloc(GST_BUFFER_SIZE(buf));
@@ -278,7 +284,7 @@ gst_snapshot_chain (GstPad *pad, GstBuffer *buf)
 
     if ( snapshot->format == GST_MAKE_FOURCC('Y','U','Y','2') )
     {
-      GST_DEBUG(0, "YUY2 => RGB\n");
+      GST_DEBUG ("YUY2 => RGB\n");
       buffer_i420 = g_malloc ((image_size * (snapshot->to_bpp/8)) );
       gst_colorspace_yuy2_to_i420( data, buffer_i420, snapshot->width, snapshot->height);
       data_to_convert = buffer_i420;
@@ -288,7 +294,7 @@ gst_snapshot_chain (GstPad *pad, GstBuffer *buf)
 
     gst_colorspace_convert (snapshot->converter, data_to_convert, data_converted); 
 
-    GST_INFO (0,"dumpfile : %s\n", snapshot->location );
+    GST_INFO ("dumpfile : %s\n", snapshot->location );
     fp = fopen( snapshot->location, "wb" );
     if ( fp == NULL )
       g_warning(" Can not open %s\n", snapshot->location );
@@ -325,7 +331,7 @@ gst_snapshot_chain (GstPad *pad, GstBuffer *buf)
     }
   }
 
-  gst_pad_push(snapshot->srcpad,buf );
+  gst_pad_push(snapshot->srcpad,GST_DATA (buf ));
 }
 
 static void
