@@ -18,6 +18,9 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -69,7 +72,7 @@ gst_vorbisenc_get_formats (GstPad *pad)
   };
   static const GstFormat sink_formats[] = {
     GST_FORMAT_BYTES,
-    GST_FORMAT_UNITS,
+    GST_FORMAT_DEFAULT,
     GST_FORMAT_TIME, 
     0
   };
@@ -85,7 +88,7 @@ gst_vorbisenc_get_formats (GstPad *pad)
 static void 		gst_vorbisenc_class_init 	(VorbisEncClass *klass);
 static void 		gst_vorbisenc_init 		(VorbisEnc *vorbisenc);
 
-static void 		gst_vorbisenc_chain 		(GstPad *pad, GstBuffer *buf);
+static void 		gst_vorbisenc_chain 		(GstPad *pad, GstData *_data);
 static gboolean 	gst_vorbisenc_setup 		(VorbisEnc *vorbisenc);
 
 static void 		gst_vorbisenc_get_property 	(GObject *object, guint prop_id, 
@@ -209,8 +212,6 @@ gst_vorbisenc_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
           *dest_value = src_value * GST_SECOND / avg;
           break;
@@ -220,8 +221,6 @@ gst_vorbisenc_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
           *dest_value = src_value * avg / GST_SECOND;
           break;
@@ -251,13 +250,11 @@ gst_vorbisenc_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  if (bytes_per_sample == 0)
             return FALSE;
 	  *dest_value = src_value / bytes_per_sample;
           break;
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	{
           gint byterate = bytes_per_sample * vorbisenc->frequency;
@@ -271,13 +268,11 @@ gst_vorbisenc_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
           res = FALSE;
       }
       break;
-    case GST_FORMAT_UNITS:
+    case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
 	  *dest_value = src_value * bytes_per_sample;
 	  break;
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	  if (vorbisenc->frequency == 0)
             return FALSE;
@@ -289,12 +284,10 @@ gst_vorbisenc_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
 	  scale = bytes_per_sample;
 	  /* fallthrough */
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  *dest_value = src_value * scale * vorbisenc->frequency / GST_SECOND;
           break;
         default:
@@ -331,9 +324,6 @@ gst_vorbisenc_src_query (GstPad *pad, GstQueryType type,
     case GST_QUERY_TOTAL:
     {
       switch (*format) {
-	case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-	  /* fallthrough */
 	case GST_FORMAT_BYTES:
 	case GST_FORMAT_TIME:
         {
@@ -375,9 +365,6 @@ gst_vorbisenc_src_query (GstPad *pad, GstQueryType type,
     }
     case GST_QUERY_POSITION:
       switch (*format) {
-	case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-	  /* fall through */
 	default:
 	{
 	  /* we only know about our samples, convert to requested format */
@@ -430,13 +417,13 @@ gst_vorbisenc_init (VorbisEnc * vorbisenc)
   vorbisenc->metadata = GST_CAPS_NEW (
 		  "vorbisenc_metadata",
                   "application/x-gst-metadata",
-                    "comment",  GST_PROPS_STRING ("Track encoded with GStreamer"),
-		    "date",     GST_PROPS_STRING (""),
-		    "tracknum", GST_PROPS_STRING (""),
-		    "title",    GST_PROPS_STRING (""),
-		    "artist",   GST_PROPS_STRING (""),
-		    "album",    GST_PROPS_STRING (""),
-		    "genre",    GST_PROPS_STRING ("")
+                    "DESCRIPTION",	GST_PROPS_STRING ("Track encoded with GStreamer"),
+		    "DATE",     	GST_PROPS_STRING (""),
+		    "TRACKNUMBER", 	GST_PROPS_STRING (""),
+		    "TITLE",    	GST_PROPS_STRING (""),
+		    "ARTIST",   	GST_PROPS_STRING (""),
+		    "ALBUM",    	GST_PROPS_STRING (""),
+		    "GENRE",    	GST_PROPS_STRING ("")
 		  );
 
 
@@ -460,19 +447,16 @@ gst_vorbisenc_add_metadata (VorbisEnc *vorbisenc, GstCaps *caps)
     prop = (GstPropsEntry*)(props->data);
     props = g_list_next(props);
 
-    if (gst_props_entry_get_type (prop) == GST_PROPS_STRING_TYPE) {
+    if (gst_props_entry_get_props_type (prop) == GST_PROPS_STRING_TYPE) {
       const gchar *name = gst_props_entry_get_name (prop);
       const gchar *value;
-      char *upper_name;
 
       gst_props_entry_get_string (prop, &value);
 
       if (!value || strlen (value) == 0)
 	continue;
 
-      upper_name = g_ascii_strup (name, -1);
-      vorbis_comment_add_tag (&vorbisenc->vc, g_strdup (upper_name), g_strdup (value));
-      g_free (upper_name);
+      vorbis_comment_add_tag (&vorbisenc->vc, g_strdup (name), g_strdup (value));
     }
   }
 }
@@ -666,13 +650,13 @@ gst_vorbisenc_write_page (VorbisEnc *vorbisenc, ogg_page *page)
 			             page->body,
 	        		     page->body_len);
 
-  GST_DEBUG (0, "vorbisenc: encoded buffer of %d bytes", 
+  GST_DEBUG ("vorbisenc: encoded buffer of %d bytes", 
 			GST_BUFFER_SIZE (outbuf));
 
   vorbisenc->bytes_out += GST_BUFFER_SIZE (outbuf);
 
   if (GST_PAD_IS_USABLE (vorbisenc->srcpad)) {
-    gst_pad_push (vorbisenc->srcpad, outbuf);
+    gst_pad_push (vorbisenc->srcpad, GST_DATA (outbuf));
   }
   else {
     gst_buffer_unref (outbuf);
@@ -680,8 +664,9 @@ gst_vorbisenc_write_page (VorbisEnc *vorbisenc, ogg_page *page)
 }
 
 static void
-gst_vorbisenc_chain (GstPad * pad, GstBuffer * buf)
+gst_vorbisenc_chain (GstPad * pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   VorbisEnc *vorbisenc;
 
   g_return_if_fail (pad != NULL);
@@ -791,7 +776,7 @@ gst_vorbisenc_chain (GstPad * pad, GstBuffer * buf)
     vorbis_block_clear (&vorbisenc->vb);
     vorbis_dsp_clear (&vorbisenc->vd);
     vorbis_info_clear (&vorbisenc->vi);
-    gst_pad_push (vorbisenc->srcpad, GST_BUFFER (gst_event_new (GST_EVENT_EOS)));
+    gst_pad_push (vorbisenc->srcpad, GST_DATA (gst_event_new (GST_EVENT_EOS)));
     gst_element_set_eos (GST_ELEMENT (vorbisenc));
   }
 }
