@@ -17,11 +17,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <gst/gst.h>
 #include <vorbis/codec.h>
 #include <vorbis/vorbisfile.h>
-#include <gst/bytestream/bytestream.h>
+#include <gst/gstbytestream.h>
 
 #define GST_TYPE_VORBISFILE \
   (vorbisfile_get_type())
@@ -38,28 +41,30 @@ typedef struct _VorbisFile VorbisFile;
 typedef struct _VorbisFileClass VorbisFileClass;
 
 struct _VorbisFile {
-  GstElement element;
+  GstElement 	 element;
 
-  GstPad *sinkpad,*srcpad;
+  GstPad 	*sinkpad,
+  		*srcpad;
   GstByteStream *bs;
 
   OggVorbis_File vf;
-  gint current_link;
+  gint 		 current_link;
 
-  gboolean restart;
-  gboolean need_discont;
-  gboolean eos;
-  gboolean seek_pending;
-  gint64 seek_value;
-  GstFormat seek_format;
-  gboolean seek_accurate;
+  gboolean 	 restart;
+  gboolean 	 need_discont;
+  gboolean 	 eos;
+  gboolean 	 seek_pending;
+  gint64 	 seek_value;
+  GstFormat 	 seek_format;
+  gboolean 	 seek_accurate;
 
-  gboolean may_eos;
-  guint64 total_bytes;
-  guint64 offset;
+  gboolean 	 may_eos;
+  guint64 	 total_bytes;
+  guint64 	 offset;
+  gulong	 blocksize;
 
-  GstCaps *metadata;
-  GstCaps *streaminfo;
+  GstCaps 	*metadata;
+  GstCaps 	*streaminfo;
 };
 
 struct _VorbisFileClass {
@@ -76,7 +81,7 @@ GstElementDetails vorbisfile_details =
 {
   "Ogg Vorbis decoder",
   "Codec/Audio/Decoder",
-  "GPL",
+  "LGPL",
   "Decodes OGG Vorbis audio using the vorbisfile API",
   VERSION,
   "Monty <monty@xiph.org>, " 
@@ -90,9 +95,12 @@ enum
   LAST_SIGNAL
 };
 
+#define DEFAULT_BLOCKSIZE 4096
+
 enum
 {
   ARG_0,
+  ARG_BLOCKSIZE,
   ARG_METADATA,
   ARG_STREAMINFO
 };
@@ -175,6 +183,9 @@ gst_vorbisfile_class_init (VorbisFileClass * klass)
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BLOCKSIZE,
+    g_param_spec_ulong ("blocksize", "Block size", "Size in bytes to read per buffer",
+                        1, G_MAXULONG, DEFAULT_BLOCKSIZE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_METADATA,
     g_param_spec_boxed ("metadata", "Metadata", "(logical) Stream metadata",
                          GST_TYPE_CAPS, G_PARAM_READABLE));
@@ -217,6 +228,7 @@ gst_vorbisfile_init (VorbisFile * vorbisfile)
   vorbisfile->metadata = NULL;
   vorbisfile->streaminfo = NULL;
   vorbisfile->current_link = -1;
+  vorbisfile->blocksize = DEFAULT_BLOCKSIZE;
 }
 
 /* the next four functions are the ov callbacks we provide to vorbisfile
@@ -227,11 +239,11 @@ gst_vorbisfile_read (void *ptr, size_t size, size_t nmemb, void *datasource)
 {
   guint32 got_bytes;
   guint8 *data;
-  size_t read_size = size * nmemb;
+  guint32 read_size = size * nmemb;
 
   VorbisFile *vorbisfile = GST_VORBISFILE (datasource);
 
-  GST_DEBUG (0, "read %d", read_size);
+  GST_DEBUG ("read %d", read_size);
 
   /* make sure we don't go to EOS */
   if (!vorbisfile->may_eos && vorbisfile->total_bytes && 
@@ -246,6 +258,8 @@ gst_vorbisfile_read (void *ptr, size_t size, size_t nmemb, void *datasource)
   do {
     got_bytes = gst_bytestream_peek_bytes (vorbisfile->bs, &data, read_size);
 
+    GST_DEBUG ("peek returned  %d", got_bytes);
+
     if (got_bytes == 0) {
       GstEvent *event;
       guint32 avail;
@@ -254,20 +268,23 @@ gst_vorbisfile_read (void *ptr, size_t size, size_t nmemb, void *datasource)
 
       switch (GST_EVENT_TYPE (event)) {
 	case GST_EVENT_EOS:
-	  GST_DEBUG (0, "eos");
+	  GST_DEBUG ("eos");
           vorbisfile->eos = TRUE;
           gst_event_unref (event);
           goto done;
 	case GST_EVENT_DISCONTINUOUS:
-	  GST_DEBUG (0, "discont");
+	  GST_DEBUG ("discont");
 	  vorbisfile->need_discont = TRUE;
 	default:
+	  GST_DEBUG ("unknown event %d", GST_EVENT_TYPE (event));
           break;
       }
       gst_event_unref (event);
     }
   } while (got_bytes == 0);
 
+  GST_DEBUG ("read %d got %d bytes", read_size, got_bytes);
+
   memcpy (ptr, data, got_bytes);
   gst_bytestream_flush_fast (vorbisfile->bs, got_bytes);
 
@@ -285,12 +302,11 @@ gst_vorbisfile_seek (void *datasource, int64_t offset, int whence)
   guint64 pending_offset = vorbisfile->offset;
   gboolean need_total = FALSE;
 
-
   if (!vorbisfile->vf.seekable) {
     return -1;
   }
   
-  GST_DEBUG (0, "seek %lld %d", offset, whence);
+  GST_DEBUG ("seek %" G_GINT64_FORMAT " %d", offset, whence);
 
   if (whence == SEEK_SET) {
     method = GST_SEEK_METHOD_SET;
@@ -322,7 +338,7 @@ gst_vorbisfile_seek (void *datasource, int64_t offset, int whence)
 static int
 gst_vorbisfile_close (void *datasource)
 {
-  GST_DEBUG (0, "close");
+  GST_DEBUG ("close");
   return 0;
 }
 
@@ -334,7 +350,7 @@ gst_vorbisfile_tell (void *datasource)
 
   result = gst_bytestream_tell (vorbisfile->bs);
 
-  GST_DEBUG (0, "tell %ld", result);
+  GST_DEBUG ("tell %ld", result);
 
   return result;
 }
@@ -441,6 +457,8 @@ static gboolean
 gst_vorbisfile_new_link (VorbisFile *vorbisfile, gint link)
 {
   vorbis_info *vi = ov_info (&vorbisfile->vf, link);
+  GstCaps *caps;
+  gboolean res = TRUE;
 
   /* new logical bitstream */
   vorbisfile->current_link = link;
@@ -448,23 +466,24 @@ gst_vorbisfile_new_link (VorbisFile *vorbisfile, gint link)
   gst_vorbisfile_update_metadata (vorbisfile, link);
   gst_vorbisfile_update_streaminfo (vorbisfile, link);
       
-  if (gst_pad_try_set_caps (vorbisfile->srcpad,
-                   GST_CAPS_NEW ("vorbisdec_src",
-                                   "audio/raw",    
-                                     "format",     GST_PROPS_STRING ("int"),
-                                     "law",        GST_PROPS_INT (0),
-                                     "endianness", GST_PROPS_INT (G_BYTE_ORDER),
-                                     "signed",     GST_PROPS_BOOLEAN (TRUE),
-                                     "width",      GST_PROPS_INT (16),
-                                     "depth",      GST_PROPS_INT (16),
-                                     "rate",       GST_PROPS_INT (vi->rate),
-                                     "channels",   GST_PROPS_INT (vi->channels)
-                                )) <= 0) 
-  {
-     return FALSE;
+  caps = GST_CAPS_NEW ("vorbisdec_src",
+                       "audio/x-raw-int",    
+                         "endianness", GST_PROPS_INT (G_BYTE_ORDER),
+                         "signed",     GST_PROPS_BOOLEAN (TRUE),
+                         "width",      GST_PROPS_INT (16),
+                         "depth",      GST_PROPS_INT (16),
+                         "rate",       GST_PROPS_INT (vi->rate),
+                         "channels",   GST_PROPS_INT (vi->channels));
+  gst_caps_ref (caps);
+
+  if (gst_pad_try_set_caps (vorbisfile->srcpad, caps) <= 0) {
+    if (!gst_pad_recover_caps_error (vorbisfile->srcpad, caps))
+      res = FALSE;
   }
 
-  return TRUE;
+  gst_caps_unref (caps);
+
+  return res;
 }
 
 static void
@@ -473,8 +492,6 @@ gst_vorbisfile_loop (GstElement *element)
   VorbisFile *vorbisfile = GST_VORBISFILE (element);
   GstBuffer *outbuf;
   long ret;
-  GstClockTime time;
-  gint64 samples;
   gint link;
 
   /* this function needs to go first since you don't want to be messing
@@ -485,7 +502,7 @@ gst_vorbisfile_loop (GstElement *element)
     vorbisfile->may_eos = FALSE;
     vorbisfile->vf.seekable = gst_bytestream_seek (vorbisfile->bs, 0, 
 		                                   GST_SEEK_METHOD_SET);
-    GST_DEBUG (GST_CAT_PLUGIN_INFO, "vorbisfile: seekable: %s\n",
+    GST_DEBUG ("vorbisfile: seekable: %s",
 	       vorbisfile->vf.seekable ? "yes" : "no");
 
     /* open our custom vorbisfile data object with the callbacks we provide */
@@ -501,7 +518,6 @@ gst_vorbisfile_loop (GstElement *element)
 
   if (vorbisfile->seek_pending) {
     /* get time to seek to in seconds */
-
     switch (vorbisfile->seek_format) {
       case GST_FORMAT_TIME:
       {
@@ -519,7 +535,7 @@ gst_vorbisfile_loop (GstElement *element)
 	}
 	break;
       }
-      case GST_FORMAT_UNITS:
+      case GST_FORMAT_DEFAULT:
 	if (vorbisfile->seek_accurate) {
           if (ov_pcm_seek (&vorbisfile->vf, vorbisfile->seek_value) == 0) {
             vorbisfile->need_discont = TRUE;
@@ -555,17 +571,12 @@ gst_vorbisfile_loop (GstElement *element)
   /* we update the caps for each logical stream */
   if (vorbisfile->vf.current_link != vorbisfile->current_link) {
     if (!gst_vorbisfile_new_link (vorbisfile, vorbisfile->vf.current_link)) {
-      gst_element_error (GST_ELEMENT (vorbisfile), "could not negotiate format");
+      return;
     }
     return;
   }
 
-  outbuf = gst_buffer_new ();
-  GST_BUFFER_DATA (outbuf) = g_malloc (4096);
-  GST_BUFFER_SIZE (outbuf) = 4096;
-
-  /* get current time for discont and buffer timestamp */
-  time = (GstClockTime) (ov_time_tell (&vorbisfile->vf) * GST_SECOND);
+  outbuf = gst_buffer_new_and_alloc (vorbisfile->blocksize);
 
   ret = ov_read (&vorbisfile->vf, 
 		 GST_BUFFER_DATA (outbuf), GST_BUFFER_SIZE (outbuf), 
@@ -573,7 +584,7 @@ gst_vorbisfile_loop (GstElement *element)
 		 sizeof (gint16), 1, &link);
 
   if (ret == 0) {
-    GST_DEBUG (0, "eos");
+    GST_DEBUG ("eos");
     /* send EOS event */
     /*ov_clear (&vorbisfile->vf);*/
     vorbisfile->restart = TRUE;
@@ -581,7 +592,7 @@ gst_vorbisfile_loop (GstElement *element)
     /* if the pad is not usable, don't push it out */
     if (GST_PAD_IS_USABLE (vorbisfile->srcpad)) {
       gst_pad_push (vorbisfile->srcpad, 
-		    GST_BUFFER (gst_event_new (GST_EVENT_EOS)));
+		    GST_DATA (gst_event_new (GST_EVENT_EOS)));
     }
     gst_element_set_eos (element);
     return;
@@ -592,6 +603,13 @@ gst_vorbisfile_loop (GstElement *element)
     return;
   }
   else {
+    GstClockTime time;
+    gint64 samples;
+    
+   /* get stream stats */
+    samples = (gint64) (ov_pcm_tell (&vorbisfile->vf));
+    time = (GstClockTime) (ov_time_tell (&vorbisfile->vf) * GST_SECOND);
+
     if (vorbisfile->need_discont) {
       GstEvent *discont;
 
@@ -599,27 +617,36 @@ gst_vorbisfile_loop (GstElement *element)
 
       /* if the pad is not usable, don't push it out */
       if (GST_PAD_IS_USABLE (vorbisfile->srcpad)) {
-        /* get stream stats */
-        samples = (gint64) (ov_pcm_tell (&vorbisfile->vf));
 
         discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time, 
-		    			     GST_FORMAT_UNITS, samples, NULL); 
+		    			     GST_FORMAT_DEFAULT, samples, NULL); 
 
-        gst_pad_push (vorbisfile->srcpad, GST_BUFFER (discont));
+        gst_pad_push (vorbisfile->srcpad, GST_DATA (discont));
       }
     }
 
     GST_BUFFER_SIZE (outbuf) = ret;
     GST_BUFFER_TIMESTAMP (outbuf) = time;
+    GST_BUFFER_OFFSET (outbuf) = samples;
+    {
+      GstClockTime duration;
+      GstFormat format;
+
+      format = GST_FORMAT_TIME;
+      gst_pad_convert (vorbisfile->srcpad,
+			     GST_FORMAT_BYTES, ret,
+			     &format, &duration);
+      GST_BUFFER_DURATION (outbuf) = duration;
+    }
 
     vorbisfile->may_eos = TRUE;
 
     if (!vorbisfile->vf.seekable) {
-      vorbisfile->total_bytes += GST_BUFFER_SIZE (outbuf);
+      vorbisfile->total_bytes += ret;
     }
   
     if (GST_PAD_IS_USABLE (vorbisfile->srcpad)) 
-      gst_pad_push (vorbisfile->srcpad, outbuf);
+      gst_pad_push (vorbisfile->srcpad, GST_DATA (outbuf));
     else
       gst_buffer_unref (outbuf);
   }
@@ -631,7 +658,7 @@ gst_vorbisfile_get_formats (GstPad *pad)
   static GstFormat src_formats[] = {
     GST_FORMAT_TIME,
     GST_FORMAT_BYTES,
-    GST_FORMAT_UNITS,
+    GST_FORMAT_DEFAULT,
     0,
     0
   };
@@ -661,16 +688,13 @@ gst_vorbisfile_src_convert (GstPad *pad,
   
   vorbisfile = GST_VORBISFILE (gst_pad_get_parent (pad));
 
-  if (*dest_format == GST_FORMAT_DEFAULT)
-    *dest_format = GST_FORMAT_TIME;
-
   vi = ov_info (&vorbisfile->vf, -1);
   bytes_per_sample = vi->channels * 2;
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
           *dest_value = src_value / (vi->channels * 2);
           break;
         case GST_FORMAT_TIME:
@@ -685,7 +709,7 @@ gst_vorbisfile_src_convert (GstPad *pad,
         default:
           res = FALSE;
       }
-    case GST_FORMAT_UNITS:
+    case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
 	  *dest_value = src_value * bytes_per_sample;
@@ -703,7 +727,7 @@ gst_vorbisfile_src_convert (GstPad *pad,
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
 	  scale = bytes_per_sample;
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  *dest_value = src_value * scale * vi->rate / GST_SECOND;
           break;
         default:
@@ -721,7 +745,7 @@ gst_vorbisfile_src_convert (GstPad *pad,
           case GST_FORMAT_BYTES:
             res = FALSE;
             break;
-          case GST_FORMAT_UNITS:
+          case GST_FORMAT_DEFAULT:
 	    if (src_value > vorbisfile->vf.links) {
 	      src_value = vorbisfile->vf.links;
 	    }
@@ -769,9 +793,6 @@ gst_vorbisfile_sink_convert (GstPad *pad,
   
   vorbisfile = GST_VORBISFILE (gst_pad_get_parent (pad));
 
-  if (*dest_format == GST_FORMAT_DEFAULT)
-    *dest_format = GST_FORMAT_TIME;
-
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
@@ -840,7 +861,7 @@ gst_vorbisfile_src_query (GstPad *pad, GstQueryType type,
     case GST_QUERY_TOTAL:
     {
       switch (*format) {
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
           if (vorbisfile->vf.seekable)
 	    *value = ov_pcm_total (&vorbisfile->vf, -1);
 	  else
@@ -852,9 +873,6 @@ gst_vorbisfile_src_query (GstPad *pad, GstQueryType type,
 	  else
 	    return FALSE;
 	  break;
-        case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-          /* fall through */
         case GST_FORMAT_TIME:
           if (vorbisfile->vf.seekable)
 	    *value = (gint64) (ov_time_total (&vorbisfile->vf, -1) * GST_SECOND);
@@ -876,9 +894,6 @@ gst_vorbisfile_src_query (GstPad *pad, GstQueryType type,
     }
     case GST_QUERY_POSITION:
       switch (*format) {
-        case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-          /* fall through */
         case GST_FORMAT_TIME:
           if (vorbisfile->vf.seekable)
 	    *value = (gint64) (ov_time_tell (&vorbisfile->vf) * GST_SECOND);
@@ -892,7 +907,7 @@ gst_vorbisfile_src_query (GstPad *pad, GstQueryType type,
 	  else
             *value = vorbisfile->total_bytes;
 	  break;
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
           if (vorbisfile->vf.seekable)
 	    *value = ov_pcm_tell (&vorbisfile->vf);
 	  else
@@ -944,11 +959,11 @@ gst_vorbisfile_src_event (GstPad *pad, GstEvent *event)
       vorbis_info *vi;
       GstFormat format;
   
-      GST_DEBUG (GST_CAT_EVENT, "vorbisfile: handling seek event on pad %s:%s",
+      GST_DEBUG ("vorbisfile: handling seek event on pad %s:%s",
 		 GST_DEBUG_PAD_NAME (pad));
       if (!vorbisfile->vf.seekable) {
 	gst_event_unref (event);
-	GST_DEBUG (GST_CAT_EVENT, "vorbis stream is not seekable");
+	GST_DEBUG ("vorbis stream is not seekable");
         return FALSE;
       }
 
@@ -966,13 +981,13 @@ gst_vorbisfile_src_event (GstPad *pad, GstEvent *event)
 	case GST_FORMAT_BYTES:
           vi = ov_info (&vorbisfile->vf, -1);
 	  if (vi->channels == 0) {
-	    GST_DEBUG (GST_CAT_EVENT, "vorbis stream has 0 channels ?");
+	    GST_DEBUG ("vorbis stream has 0 channels ?");
 	    res = FALSE;
 	    goto done; 
 	  }
           offset /= vi->channels * 2;
 	  /* fallthrough */
-	case GST_FORMAT_UNITS:
+	case GST_FORMAT_DEFAULT:
 	  vorbisfile->seek_pending = TRUE;
 	  vorbisfile->seek_value = offset;
 	  vorbisfile->seek_format = format;
@@ -989,7 +1004,7 @@ gst_vorbisfile_src_event (GstPad *pad, GstEvent *event)
 	  }
 	  else
 	  {
-	    GST_DEBUG (GST_CAT_EVENT, "unhandled seek format");
+	    GST_DEBUG ("unhandled seek format");
 	    res = FALSE;
 	  }
 	  break;
@@ -1047,6 +1062,9 @@ gst_vorbisfile_set_property (GObject *object, guint prop_id,
   vorbisfile = GST_VORBISFILE (object);
 
   switch (prop_id) {
+    case ARG_BLOCKSIZE:
+      vorbisfile->blocksize = g_value_get_ulong (value);
+      break;
     default:
       g_warning ("Unknown property id\n");
   }
@@ -1063,6 +1081,9 @@ gst_vorbisfile_get_property (GObject *object, guint prop_id,
   vorbisfile = GST_VORBISFILE (object);
 
   switch (prop_id) {
+    case ARG_BLOCKSIZE:
+      g_value_set_ulong (value, vorbisfile->blocksize);
+      break;
     case ARG_METADATA:
       g_value_set_boxed (value, vorbisfile->metadata);
       break;
