@@ -1,12 +1,29 @@
+/* GStreamer
+ * Copyright (C) <2002> Thomas Vander Stichele <thomas@apestaart.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 /*
  * this library handles interaction with GConf
  */
 
 #include "gconf.h"
 
-#ifndef GST_GCONF_DIR
-#error "GST_GCONF_DIR is not defined !"
-#endif
+#define GST_GCONF_DIR "/system/gstreamer"
 
 static GConfClient *_gst_gconf_client = NULL; /* GConf connection */
 
@@ -80,7 +97,7 @@ gst_gconf_get_string (const gchar *key)
 
   if (error)
   {
-    g_warning ("gst_gconf_get_string: error: %s\n", error->message);
+    g_print ("gst_gconf_get_string: error: %s\n", error->message);
     g_error_free (error);
   }
   /* FIXME: decide if we want to strdup this value; if we do, check for NULL */
@@ -103,7 +120,7 @@ gst_gconf_set_string (const gchar *key, const gchar *value)
   gconf_client_set_string (gst_gconf_get_client (), full_key, value, &error);
   if (error)
   {
-    g_warning ("gst_gconf_set_string: error: %s\n", error->message);
+    g_print ("gst_gconf_set_string: error: %s\n", error->message);
     g_error_free (error);
   }
   g_free (full_key);
@@ -187,7 +204,7 @@ gst_gconf_get_default_audio_sink (void)
     if (!ret)
       g_warning ("No GConf default audio sink key and osssink doesn't work");
     else
-      g_warning ("GConf audio sink not found, using osssink");
+      g_print ("GConf audio sink not found, using osssink");
   }
 
   return ret;
@@ -213,7 +230,7 @@ gst_gconf_get_default_video_sink (void)
     if (!ret)
       g_warning ("No GConf default video sink key and xvideosink doesn't work");
     else
-      g_warning ("GConf video sink not found, using xvideosink");
+      g_print ("GConf video sink not found, using xvideosink");
   }
 
   return ret;
@@ -239,7 +256,7 @@ gst_gconf_get_default_audio_src (void)
     if (!ret)
       g_warning ("No GConf default audio src key and osssrc doesn't work");
     else
-      g_warning ("GConf audio src not found, using osssrc");
+      g_print ("GConf audio src not found, using osssrc");
   }
 
   return ret;
@@ -266,33 +283,33 @@ gst_gconf_get_default_video_src (void)
     if (!ret)
       g_warning ("No GConf default video src key and videotestrc doesn't work");
     else
-      g_warning ("GConf video src not found, using videotestrc");
+      g_print ("GConf video src not found, using videotestrc");
   }
 
   return ret;
 }
 
 /**
- * gst_gconf_get_default_visualisation_element:
+ * gst_gconf_get_default_visualization_element:
  *
- * Render visualisation bin from GStreamer GConf key : "default/visualisation".
- * If key is invalid goom is used as default visualisation element.
+ * Render visualization bin from GStreamer GConf key : "default/visualization".
+ * If key is invalid goom is used as default visualization element.
  *
- * Returns: a #GstElement containing the visualisation bin, or NULL if
+ * Returns: a #GstElement containing the visualization bin, or NULL if
  * everything failed.
  */
 GstElement *
-gst_gconf_get_default_visualisation_element (void)
+gst_gconf_get_default_visualization_element (void)
 {
-  GstElement *ret = gst_gconf_render_bin_from_key ("default/visualisation");
+  GstElement *ret = gst_gconf_render_bin_from_key ("default/visualization");
   
   if (!ret) {
     ret = gst_element_factory_make ("goom", NULL);
   
     if (!ret)
-      g_warning ("No GConf default visualisation plugin key and goom doesn't work");
+      g_warning ("No GConf default visualization plugin key and goom doesn't work");
     else
-      g_warning ("GConf visualisation plugin not found, using goom");
+      g_print ("GConf visualization plugin not found, using goom");
   }
 
   return ret;
@@ -312,4 +329,3 @@ GstPluginDesc plugin_desc = {
         "gstgconf",
 	  plugin_init
 };
-
