@@ -21,6 +21,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <gst/gst.h>
 #include "gsteffectv.h"
@@ -88,7 +91,7 @@ static void 	gst_edgetv_set_property 	(GObject * object, guint prop_id,
 static void 	gst_edgetv_get_property 	(GObject * object, guint prop_id,
 					  	 GValue * value, GParamSpec * pspec);
 
-static void 	gst_edgetv_chain 		(GstPad * pad, GstBuffer * buf);
+static void 	gst_edgetv_chain 		(GstPad * pad, GstData *_data);
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_edgetv_signals[LAST_SIGNAL] = { 0 }; */
@@ -150,7 +153,7 @@ gst_edgetv_sinkconnect (GstPad * pad, GstCaps * caps)
   filter->map = (guint32 *)g_malloc (filter->map_width * filter->map_height * sizeof(guint32) * 2);
   bzero(filter->map, filter->map_width * filter->map_height * sizeof(guint32) * 2);
 
-  return gst_pad_try_set_caps (filter->srcpad, caps);
+  return gst_pad_try_set_caps (filter->srcpad, gst_caps_ref (caps));
 }
 
 static void
@@ -168,8 +171,9 @@ gst_edgetv_init (GstEdgeTV * filter)
 }
 
 static void
-gst_edgetv_chain (GstPad * pad, GstBuffer * buf)
+gst_edgetv_chain (GstPad * pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstEdgeTV *filter;
   int x, y;
   int r, g, b;
@@ -266,7 +270,7 @@ gst_edgetv_chain (GstPad * pad, GstBuffer * buf)
   }
   gst_buffer_unref (buf);
 
-  gst_pad_push (filter->srcpad, outbuf);
+  gst_pad_push (filter->srcpad, GST_DATA (outbuf));
 }
 
 static void
