@@ -18,6 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gst/gst.h>
 #include "gstfilter.h"
 #include "iir.h"
@@ -86,7 +89,7 @@ static void gst_iir_set_property	(GObject * object, guint prop_id,
 static void gst_iir_get_property	(GObject * object, guint prop_id,
                                          GValue * value, GParamSpec * pspec);
 
-static void gst_iir_chain		(GstPad * pad, GstBuffer * buf);
+static void gst_iir_chain		(GstPad * pad, GstData *_data);
 static GstPadLinkReturn
        gst_iir_sink_connect 		(GstPad * pad, GstCaps * caps);
 
@@ -172,7 +175,7 @@ gst_iir_sink_connect (GstPad * pad, GstCaps * caps)
   if (!GST_CAPS_IS_FIXED (caps))
     return GST_PAD_LINK_DELAYED;
  
-  set_retval = gst_pad_try_set_caps(filter->srcpad, caps);  
+  set_retval = gst_pad_try_set_caps(filter->srcpad, gst_caps_ref (caps));  
   if (set_retval > 0) {
     /* connection works, so init the filter */
     /* FIXME: remember to free it */
@@ -185,8 +188,9 @@ gst_iir_sink_connect (GstPad * pad, GstCaps * caps)
 }
 
 static void
-gst_iir_chain (GstPad * pad, GstBuffer * buf)
+gst_iir_chain (GstPad * pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstIIR *filter;
   gfloat *src;
   int i;
@@ -202,7 +206,7 @@ gst_iir_chain (GstPad * pad, GstBuffer * buf)
   for (i = 0; i < GST_BUFFER_SIZE (buf) / sizeof (gfloat); ++i)
     *(src + i) = (gfloat) IIR_filter (filter->state, (double) *(src + i));
 
-  gst_pad_push (filter->srcpad, buf);
+  gst_pad_push (filter->srcpad, GST_DATA (buf));
 }
 
 static void
