@@ -17,14 +17,18 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 
 #include "flx_fmt.h"
 #include "gstflxdec.h"
+#include <gst/video/video.h>
 
 #define JIFFIE  (GST_SECOND/70)
 
-static GstCaps* flxdec_type_find(GstBuffer *buf, gpointer private);
+static GstCaps* flxdec_type_find (GstByteStream *bs, gpointer private);
 
 /* flx element information */
 static GstElementDetails flxdec_details = {
@@ -33,7 +37,7 @@ static GstElementDetails flxdec_details = {
   "LGPL",
   "FLX decoder",
   VERSION,
-  "Sepp Wijnands <mrrazz@garbage-coderz.net>"
+  "Sepp Wijnands <mrrazz@garbage-coderz.net>",
   "(C) 2001",
 };
 
@@ -61,7 +65,7 @@ GST_PAD_TEMPLATE_FACTORY (sink_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "flxdec_sink",         
-    "video/fli",
+    "video/x-fli",
      NULL
   )
 )
@@ -73,16 +77,16 @@ GST_PAD_TEMPLATE_FACTORY (src_video_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "src_video",
-    "video/raw",
-      "format",       GST_PROPS_FOURCC (GST_MAKE_FOURCC ('R', 'G', 'B', ' ')),
+    "video/x-raw-rgb",
         "bpp",        GST_PROPS_INT (32),
         "depth",      GST_PROPS_INT (32),
-        "endianness", GST_PROPS_INT (G_LITTLE_ENDIAN),
-        "red_mask",   GST_PROPS_INT (0x00ff0000),
-        "green_mask", GST_PROPS_INT (0x0000ff00),
-        "blue_mask",  GST_PROPS_INT (0x000000ff),
+        "endianness", GST_PROPS_INT (G_BIG_ENDIAN),
+        "red_mask",   GST_PROPS_INT (R_MASK_32),
+        "green_mask", GST_PROPS_INT (G_MASK_32),
+        "blue_mask",  GST_PROPS_INT (B_MASK_32),
         "width",      GST_PROPS_INT_RANGE(320, 1280), 
-        "height",     GST_PROPS_INT_RANGE(200, 1024)
+        "height",     GST_PROPS_INT_RANGE(200, 1024),
+        "framerate",  GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT)
   )
 )
 
@@ -109,23 +113,33 @@ static void 	flx_decode_delta_flc	(GstFlxDec *, guchar *, guchar *);
 static GstElementClass *parent_class = NULL;
 
 static GstCaps* 
-flxdec_type_find (GstBuffer *buf, gpointer private)
+flxdec_type_find (GstByteStream *bs, gpointer private)
 {
-  guchar *data = GST_BUFFER_DATA(buf);
-  GstCaps *new;
+  GstBuffer *buf = NULL;
+  GstCaps *new = NULL;
 
-  /* check magic */
-  if ((data[4] == 0x11 || data[4] == 0x12
-       || data[4] == 0x30 || data[4] == 0x44) && data[5] == 0xaf) {
+  if (gst_bytestream_peek (bs, &buf, 134) == 134) {
+    guint8 *data = GST_BUFFER_DATA (buf);
+
+    /* check magic */
+    if ((data[4] == 0x11 || data[4] == 0x12 ||
+         data[4] == 0x30 || data[4] == 0x44) &&
+        data[5] == 0xaf) {
       /* check the frame type of the first frame */
       if ((data[132] == 0x00 || data[132] == 0xfa) && data[133] == 0xf1) {
-        g_print("GstFlxDec: found supported flx format\n");
-        new = gst_caps_new("flxdec_type_find","video/fli", NULL);
-        return new;
+        GST_DEBUG ("GstFlxDec: found supported flx format");
+        new = gst_caps_new ("flxdec_type_find",
+			    "video/x-fli",
+			      NULL);
       }
+    }
   }
-  
-  return NULL; 
+
+  if (buf != NULL) {
+    gst_buffer_unref (buf);
+  }
+
+  return new;
 }
 
 
@@ -467,7 +481,7 @@ gst_flxdec_loop (GstElement *element)
   g_return_if_fail (element != NULL);
   g_return_if_fail (GST_IS_FLXDEC(element));
 
-  GST_DEBUG (0, "entering loop function");
+  GST_DEBUG ("entering loop function");
   
   flxdec = GST_FLXDEC(element);
 
@@ -513,20 +527,18 @@ gst_flxdec_loop (GstElement *element)
     }
     
     gst_pad_try_set_caps (flxdec->srcpad,
-  		gst_caps_new (
+  		GST_CAPS_NEW (
   	  	  "src_video",
-	  	  "video/raw",
-		  gst_props_new (
-		    "format",       GST_PROPS_FOURCC (GST_MAKE_FOURCC ('R', 'G', 'B', ' ')),
-		      "bpp",        GST_PROPS_INT (32),
-		      "depth",      GST_PROPS_INT (32),
-		      "endianness", GST_PROPS_INT (G_LITTLE_ENDIAN),
-		      "red_mask",   GST_PROPS_INT (0x00ff0000),
-		      "green_mask", GST_PROPS_INT (0x0000ff00),
-		      "blue_mask",  GST_PROPS_INT (0x000000ff),
-  		      "width",      GST_PROPS_INT (flxh->width), 
-  		      "height",     GST_PROPS_INT (flxh->height),
-		    NULL)));
+	  	  "video/x-raw-rgb",
+	            "bpp",        GST_PROPS_INT (32),
+		    "depth",      GST_PROPS_INT (32),
+		    "endianness", GST_PROPS_INT (G_BIG_ENDIAN),
+		    "red_mask",   GST_PROPS_INT (R_MASK_32),
+		    "green_mask", GST_PROPS_INT (G_MASK_32),
+		    "blue_mask",  GST_PROPS_INT (B_MASK_32),
+  		    "width",      GST_PROPS_INT (flxh->width), 
+  		    "height",     GST_PROPS_INT (flxh->height),
+		    "framerate",  GST_PROPS_FLOAT (GST_SECOND/flxdec->frame_time)));
 
     if (flxh->depth <= 8) 
       flxdec->converter = flx_colorspace_converter_new(flxh->width, flxh->height);
@@ -597,7 +609,7 @@ gst_flxdec_loop (GstElement *element)
 	GST_BUFFER_TIMESTAMP (out) = flxdec->next_time;
 	flxdec->next_time += flxdec->frame_time;
 
-        gst_pad_push(flxdec->srcpad, out);
+        gst_pad_push(flxdec->srcpad, GST_DATA (out));
         
         break;
     }
@@ -678,10 +690,6 @@ plugin_init (GModule *module, GstPlugin *plugin)
   GstElementFactory *factory;
   GstTypeFactory *type;
 
-  /* this filter needs the bytestream package */
-  if (!gst_library_load ("gstbytestream"))
-    return FALSE;
-
   factory = gst_element_factory_new("flxdec", GST_TYPE_FLXDEC, &flxdec_details);
   g_return_val_if_fail(factory != NULL, FALSE);
   gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
