@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) <2001> Steve Baker <stevebaker_org@yahoo.co.uk>
+ * Copyright (C) 2002, 2003 Andy Wingo <wingo at pobox dot com>
  *
  * float2int.c
  *
@@ -19,9 +20,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gst/floatcast/floatcast.h>
 
 #include <gstfloat2int.h>
+#include <gst/audio/audio.h>
 
 /* elementfactory information */
 static GstElementDetails float2int_details = {
@@ -30,19 +35,18 @@ static GstElementDetails float2int_details = {
   "LGPL",
   "Convert from floating point to integer audio data",
   VERSION,
-  "Steve Baker <stevebaker_org@yahoo.co.uk>",
-  "(C) 2001",
+  "Steve Baker <stevebaker_org@yahoo.co.uk>, Andy Wingo <wingo at pobox dot com>",
+  "(C) 2001, 2002, 2003",
 };
 
 
-enum {
-  /* FILL ME */
-  LAST_SIGNAL
-};
+#define GST_FLOAT2INT_DEFAULT_BUFFER_FRAMES 256
+#if 0
+#define F2I_DEBUG(...) g_message (__VA_ARGS__)
+#else
+#define F2I_DEBUG(...)
+#endif
 
-enum {
-  ARG_0,
-};
 
 GST_PAD_TEMPLATE_FACTORY (float2int_sink_factory,
   "sink%d",
@@ -50,13 +54,12 @@ GST_PAD_TEMPLATE_FACTORY (float2int_sink_factory,
   GST_PAD_REQUEST,
   GST_CAPS_NEW (
     "float2int_sink",
-    "audio/raw",
-    "rate",       GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "format",     GST_PROPS_STRING ("float"),
-    "layout",     GST_PROPS_STRING ("gfloat"),
-    "intercept",  GST_PROPS_FLOAT(0.0),
-    "slope",      GST_PROPS_FLOAT(1.0),
-    "channels",   GST_PROPS_INT (1)
+    "audio/x-raw-float",
+    "rate",		GST_PROPS_INT_RANGE (1, G_MAXINT),
+    "width",		GST_PROPS_INT (32),
+    "endianness",	GST_PROPS_INT (G_BYTE_ORDER),
+    "buffer-frames",	GST_PROPS_INT_RANGE (1, G_MAXINT),
+    "channels",		GST_PROPS_INT (1)
   )
 );
 
@@ -66,11 +69,9 @@ GST_PAD_TEMPLATE_FACTORY (float2int_src_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "float2int_src",
-    "audio/raw",
-    "format",     GST_PROPS_STRING ("int"),
+    "audio/x-raw-int",
     "channels",   GST_PROPS_INT_RANGE (1, G_MAXINT),
     "rate",       GST_PROPS_INT_RANGE (1, G_MAXINT),
-    "law",        GST_PROPS_INT (0),
     "endianness", GST_PROPS_INT (G_BYTE_ORDER),
     "width",      GST_PROPS_INT (16),
     "depth",      GST_PROPS_INT (16),
@@ -83,7 +84,7 @@ static GstPadTemplate *srctempl, *sinktempl;
 static void	gst_float2int_class_init		(GstFloat2IntClass *klass);
 static void	gst_float2int_init			(GstFloat2Int *plugin);
 
-static GstPadLinkReturn gst_float2int_connect	(GstPad *pad, GstCaps *caps);
+static GstPadLinkReturn gst_float2int_link		(GstPad *pad, GstCaps *caps);
 
 static GstPad*  gst_float2int_request_new_pad		(GstElement *element, GstPadTemplate *temp,
                                                          const gchar *unused);
@@ -138,7 +139,7 @@ gst_float2int_init (GstFloat2Int *plugin)
 
   plugin->srcpad = gst_pad_new_from_template (srctempl, "src");
   gst_element_add_pad (GST_ELEMENT (plugin), plugin->srcpad);
-  gst_pad_set_link_function (plugin->srcpad, gst_float2int_connect);
+  gst_pad_set_link_function (plugin->srcpad, gst_float2int_link);
 
   plugin->numchannels = 0;
   plugin->channelcount = 0;
@@ -148,26 +149,14 @@ gst_float2int_init (GstFloat2Int *plugin)
 static GstBufferPool*
 gst_float2int_get_bufferpool (GstPad *pad)
 {
-  static GstBufferPool *bp = NULL;
-  GstBuffer *buffer;
+  gint frames;
   GstFloat2Int *filter = (GstFloat2Int *) gst_pad_get_parent (pad);
 
-  if (!filter->pool)
-    if (! (filter->pool = gst_pad_get_bufferpool (filter->srcpad)))
-      filter->pool = gst_buffer_pool_get_default (sizeof(gint16) * 1024 * filter->numchannels, 5);
-
-  if (!bp) {
-    /* create a new buffer pool with the same number of frames as the source
-       pad's buffer pool */
-    if (!(buffer = gst_buffer_new_from_pool (filter->pool, 0, 0)))
-      return NULL;
-    
-    bp = gst_buffer_pool_get_default (GST_BUFFER_SIZE (buffer) / sizeof (gint16) / filter->numchannels * sizeof (gfloat), 8);
-
-    gst_buffer_unref (buffer);
-  }
+  frames = filter->buffer_frames;
+  if (!frames)
+    frames = GST_FLOAT2INT_DEFAULT_BUFFER_FRAMES;
   
-  return bp;
+  return gst_buffer_pool_get_default (frames * sizeof (float), 5);
 }
 
 /* this is a very brittle function. it's hard to come up with a "correct
@@ -186,16 +175,19 @@ gst_float2int_get_bufferpool (GstPad *pad)
    programmer error more than anything.
  */
 static GstPadLinkReturn
-gst_float2int_connect (GstPad *pad, GstCaps *caps)
+gst_float2int_link (GstPad *pad, GstCaps *caps)
 {
   GstFloat2Int *filter;
-  GstCaps *intcaps = NULL, *floatcaps = NULL;
+  GstCaps *intcaps, *floatcaps;
   GSList *l;
-  gint rate, channels;
+  gint rate, channels, buffer_frames;
   
   filter = GST_FLOAT2INT (GST_PAD_PARENT (pad));
   
-  /* we have two variables to play with: "rate" and "channels" on the int side. */
+  /* we have three variables to play with:
+     "rate": both sides
+     "buffer-frames": float side
+     "channels": int side. */
   if (GST_CAPS_IS_FIXED (caps)) {
     if (pad == filter->srcpad) {
       /* it's the int pad */
@@ -203,54 +195,62 @@ gst_float2int_connect (GstPad *pad, GstCaps *caps)
         gst_caps_get_int (caps, "channels", &channels);
 
         if (channels != filter->numchannels) {
-          GST_DEBUG (0, "Tried to set caps with %d channels but there are %d source pads",
+          GST_DEBUG ("Tried to set caps with %d channels but there are %d source pads",
                      channels, filter->numchannels);
           return GST_PAD_LINK_REFUSED;
         }
         
+        if (!filter->buffer_frames)
+          filter->buffer_frames = GST_FLOAT2INT_DEFAULT_BUFFER_FRAMES;
+        
         floatcaps = gst_caps_copy (gst_pad_template_get_caps (float2int_sink_factory ()));
-        /* FIXME: we really shouldn't have to convert to int and back */
         gst_caps_get_int (caps, "rate", &filter->rate);
-        /* g_print("float2int got caps with rate %d\n", rate); */
         gst_caps_set (floatcaps, "rate", GST_PROPS_INT (filter->rate), NULL);
-        /* we now know that the caps are fixed. let's set them. this is a hack
-           but oh well. */
-	GST_CAPS_FLAG_SET (floatcaps, GST_CAPS_FIXED);
+        gst_caps_set (floatcaps, "buffer-frames", GST_PROPS_INT (filter->buffer_frames), NULL);
         
         for (l=filter->channels; l; l=l->next)
           if (gst_pad_try_set_caps (GST_FLOAT2INT_CHANNEL (l)->sinkpad, floatcaps) <= 0)
             return GST_PAD_LINK_REFUSED;
         
-        /* FIXME: refcounting? */
-        filter->floatcaps = floatcaps;
-        filter->intcaps = caps;
+        filter->floatcaps = gst_caps_ref (floatcaps);
+        filter->intcaps = gst_caps_ref (caps);
         return GST_PAD_LINK_OK;
       } else if (gst_caps_intersect (caps, filter->intcaps)) {
         /* they have to be the same since both are fixed */
         return GST_PAD_LINK_OK;
       } else {
+        GST_DEBUG ("Tried to connect the source of float2int, but with bad caps");
+        gst_caps_debug (filter->intcaps, "float2int internal int source pad caps");
         return GST_PAD_LINK_REFUSED;
       }
     } else {
       /* it's a float pad */
       if (!filter->intcaps) {
-
-        /* FIXME: we really shouldn't have to convert to int and back */
         intcaps = gst_caps_copy (gst_pad_template_get_caps (float2int_src_factory ()));
         gst_caps_get_int (caps, "rate", &rate);
         gst_caps_set (intcaps, "rate", GST_PROPS_INT (rate));
         gst_caps_set (intcaps, "channels", GST_PROPS_INT (filter->numchannels));
-	GST_CAPS_FLAG_SET (intcaps, GST_CAPS_FIXED);
         gst_caps_debug (intcaps, "int source pad caps going into try_set_caps()");
         
         if (gst_pad_try_set_caps (filter->srcpad, intcaps) <= 0) {
           return GST_PAD_LINK_REFUSED;
 	}
 
-	/* FIXME: refcounting? */
-        filter->floatcaps = caps;
-        filter->intcaps = intcaps;
+        if (!gst_caps_get_int (caps, "buffer-frames", &buffer_frames)) {
+          g_warning ("You have an old float plugin that needs to be converted "
+                     "to the new caps format. Please file a bug!");
+          return GST_PAD_LINK_REFUSED;
+        }
+        if (filter->buffer_frames && filter->buffer_frames != buffer_frames) {
+          GST_DEBUG ("Tried to set buffer_frames=%d but it was already set to %d",
+                     buffer_frames, filter->buffer_frames);
+          return GST_PAD_LINK_REFUSED;
+        }
+
+        filter->floatcaps = gst_caps_ref (caps);
+        filter->intcaps = gst_caps_ref (intcaps);
         filter->rate = rate;
+        filter->buffer_frames = buffer_frames;
 
         for (l=filter->channels; l; l=l->next) {
 	  if (GST_FLOAT2INT_CHANNEL (l)->sinkpad == pad) {
@@ -259,8 +259,14 @@ gst_float2int_connect (GstPad *pad, GstCaps *caps)
 	  
           if (gst_pad_try_set_caps (GST_FLOAT2INT_CHANNEL (l)->sinkpad, caps) == FALSE) {
 	    /* We don't have the caps set, so remove them */
-	    filter->floatcaps = NULL;
-	    filter->intcaps = NULL;
+            if (filter->floatcaps) {
+              gst_caps_unref (filter->floatcaps);
+              filter->floatcaps = NULL;
+            }
+            if (filter->intcaps) {
+              gst_caps_unref (filter->intcaps);
+              filter->intcaps = NULL;
+            }
             return GST_PAD_LINK_REFUSED;
 	  }
 	}
@@ -299,15 +305,21 @@ gst_float2int_request_new_pad (GstElement *element, GstPadTemplate *templ, const
   name = g_strdup_printf ("sink%d", plugin->channelcount);
   channel->sinkpad = gst_pad_new_from_template (templ, name);
   gst_element_add_pad (GST_ELEMENT (plugin), channel->sinkpad);
-  gst_pad_set_link_function (channel->sinkpad, gst_float2int_connect);
+  gst_pad_set_link_function (channel->sinkpad, gst_float2int_link);
   gst_pad_set_bufferpool_function (channel->sinkpad, gst_float2int_get_bufferpool);
-  channel->bytestream = gst_bytestream_new (channel->sinkpad);
   
   plugin->channels = g_slist_append (plugin->channels, channel);
   plugin->numchannels++;
   plugin->channelcount++;
   
-  GST_DEBUG (0, "float2int added pad %s\n", name);
+  if (plugin->intcaps) {
+    gst_caps_set (plugin->intcaps, "channels", GST_PROPS_INT (plugin->numchannels));
+    if (gst_pad_try_set_caps (plugin->srcpad, plugin->intcaps) <= 0) {
+      g_warning ("float2int: added a pad, but failed to update src caps");
+    }
+  }
+
+  GST_DEBUG ("float2int added pad %s\n", name);
 
   g_free (name);
   return channel->sinkpad;
@@ -320,7 +332,7 @@ gst_float2int_pad_removed (GstElement *element,
   GstFloat2Int *plugin;
   GSList *p;
   
-  GST_DEBUG(0, "float2int removed pad %s\n", GST_OBJECT_NAME (pad));
+  GST_DEBUG ("float2int removed pad %s\n", GST_OBJECT_NAME (pad));
   
   plugin = GST_FLOAT2INT (element);
 
@@ -338,7 +350,8 @@ gst_float2int_pad_removed (GstElement *element,
       
       g_slist_free (p_copy);
 
-      gst_bytestream_destroy (channel->bytestream);
+      if (channel->buffer)
+        gst_buffer_unref (channel->buffer);
       g_free (channel);
     } else {
       p = p->next;
@@ -346,20 +359,6 @@ gst_float2int_pad_removed (GstElement *element,
   }
 }
 
-static gboolean
-all_channels_eos (GSList *inputs)
-{
-  for (; inputs; inputs = inputs->next) {
-    GstFloat2IntInputChannel *c = inputs->data;
-
-    if (c->eos == FALSE) {
-      return FALSE;
-    }
-  }
-
-  return TRUE;
-}
-
 static void
 gst_float2int_loop (GstElement *element)
 {
@@ -370,9 +369,8 @@ gst_float2int_loop (GstElement *element)
   gint num_frames, i, j;
   GSList *l;
   GstFloat2IntInputChannel *channel;
-  guint32 got_bytes, waiting;
-  GstEvent *event = NULL;
-  guint8 *ptr;
+  gint frames_to_write;
+  gint numchannels = plugin->numchannels;
     
   if (!plugin->channels) {
     gst_element_error (element, "float2int: at least one sink pad needs to be connected");
@@ -381,86 +379,108 @@ gst_float2int_loop (GstElement *element)
 
   if (!plugin->pool)
     if (! (plugin->pool = gst_pad_get_bufferpool (plugin->srcpad)))
-      plugin->pool = gst_buffer_pool_get_default (sizeof(gint16) * 1024 * plugin->numchannels, 5);
+      plugin->pool = gst_buffer_pool_get_default (sizeof(gint16) * 1024 * numchannels, 1);
 
   g_assert (plugin->pool);
 
-  do {
-    /* get new output buffer */
-    buf_out = gst_buffer_new_from_pool (plugin->pool, 0, 0);
-    num_frames = GST_BUFFER_SIZE (buf_out) / plugin->numchannels / sizeof (gint16);
-    
-    data_out = (gint16*)GST_BUFFER_DATA(buf_out);
+  /* get new output buffer */
+  buf_out = gst_buffer_new_from_pool (plugin->pool, 0, 0);
+  num_frames = GST_BUFFER_SIZE (buf_out) / numchannels / sizeof (gint16);
+  g_return_if_fail (GST_BUFFER_SIZE (buf_out) == num_frames * numchannels * sizeof (gint16));
+  
+  data_out = (gint16*)GST_BUFFER_DATA(buf_out);
+  
+  i = 0;
+  frames_to_write = num_frames;
 
-    /* FIXME: assert that buf_out is cleanly divisable by numchannels */
+  while (frames_to_write) {
+    if (!plugin->frames_remaining) {
+      /* the buffer-frames GstPropsEntry ensures us that all buffers coming in
+         will be the same size. if they are not, that pad will send EOS on the
+         next pull. */
+      F2I_DEBUG ("pulling buffers...");
+      plugin->frames_remaining = G_MAXINT;
 
-    l = plugin->channels;
-    i = 0;
-    while (l) {
-      channel = GST_FLOAT2INT_CHANNEL (l);
+      for (l = plugin->channels; l; l=l->next) {
+        channel = GST_FLOAT2INT_CHANNEL (l);
 
-      if (channel->eos) {
-	l = l->next;
-	continue;
-      }
+        if (channel->buffer)
+          gst_buffer_unref (channel->buffer);
+        channel->buffer = NULL;
+      get_buffer:
+        channel->buffer = GST_BUFFER (gst_pad_pull (channel->sinkpad));
+        g_return_if_fail (channel->buffer != NULL);
 
-      got_bytes = gst_bytestream_peek_bytes (channel->bytestream, &ptr,
-          num_frames * sizeof (gfloat));
-      data_in = (gfloat *)ptr;
-
-      /* FIXME we should do something with the data if got_bytes is more than zero */
-      if (got_bytes < num_frames * sizeof (gfloat)) {
-        /* we need to check for an event. */
-        gst_bytestream_get_status (channel->bytestream, &waiting, &event);
-
-        if (event) {
-          switch (GST_EVENT_TYPE(event)) {
-          case GST_EVENT_EOS:
-            /* if we get an EOS event from one of our sink pads, we assume that
-               pad's finished handling data. Let's just send an EOS. */
-            GST_DEBUG (0, "got an EOS event");
-
-	    if (all_channels_eos (plugin->channels)) {
-              gst_element_set_eos (element);
-	    } else {
-	      channel->eos = TRUE;
-	    }
-	    
-            gst_pad_push (plugin->srcpad, GST_BUFFER (event));
-
-            gst_pad_event_default (plugin->srcpad, event);
-            break;
-          default:
-            gst_pad_event_default (plugin->srcpad, event);
-            break;
+        if (GST_IS_EVENT (channel->buffer)) {
+          F2I_DEBUG ("got an event");
+          if (GST_EVENT_TYPE (channel->buffer) == GST_EVENT_EOS) {
+            plugin->frames_remaining = 0;
+            GST_INFO ("float2int got eos from pad %s%s", GST_DEBUG_PAD_NAME (channel->sinkpad));
+          } else {
+            gst_pad_event_default (plugin->srcpad, GST_EVENT (channel->buffer));
+            goto get_buffer; /* goto, hahaha */
           }
+	} else {
+          F2I_DEBUG ("got a buffer %d frames big", GST_BUFFER_SIZE (channel->buffer)/4);
+          plugin->frames_remaining = MIN (plugin->frames_remaining,
+                                          GST_BUFFER_SIZE (channel->buffer) / sizeof (gfloat));
         }
-      } else {
-        if (!plugin->rate) {
-          gst_element_error (element, "caps were never set, bailing...");
-          return;
-        }
-
-        for (j = 0; j < num_frames; j++) {
-          data_out[(j*plugin->numchannels) + i] = (gint16)(gst_cast_float(data_in[j] * 32767.0F));
-        }
-        
-        gst_bytestream_flush (channel->bytestream, got_bytes);
-
-        GST_DEBUG (0, "done copying data");
       }
-      
-      i++;
-      l = g_slist_next (l);
     }
 
-    GST_BUFFER_TIMESTAMP(buf_out) = plugin->offset * GST_SECOND / plugin->rate;
+    if (plugin->frames_remaining) {
+      gint to_process;
 
-    plugin->offset += num_frames;
-    gst_pad_push(plugin->srcpad,buf_out);
+      F2I_DEBUG ("processing %d%s", plugin->frames_remaining,
+                 frames_to_write >= plugin->frames_remaining ? " (last time)" : "");
+      to_process = MIN (frames_to_write, plugin->frames_remaining);
+      for (l=plugin->channels, i=0; l; l=l->next, i++) {
+        F2I_DEBUG ("processing for %d frames on channel %d", to_process, i);
+        channel = GST_FLOAT2INT_CHANNEL (l);
+        g_return_if_fail (channel->buffer != NULL);
+        data_in = (gfloat*) GST_BUFFER_DATA (channel->buffer);
+        data_in += GST_BUFFER_SIZE (channel->buffer) / sizeof (float) - plugin->frames_remaining;
+        
+        for (j = 0; j < to_process; j++)
+          data_out[(j*numchannels) + i] = (gint16)(gst_cast_float(data_in[j] * 32767.0F));
+      }
+      data_out += to_process * numchannels;
+      
+      if (to_process < plugin->frames_remaining) {
+        /* we're finished with this output buffer */
+        plugin->frames_remaining -= to_process;
+        frames_to_write = 0;
+      } else {
+        /* note there's still the possibility that one of the buffers had less
+           data, so we're possibly discarding some data from other float pads;
+           that's ok though, because the pad that didn't give us buffer_frames
+           worth of data will send EOS on the next pull */
+        frames_to_write -= plugin->frames_remaining;
+        plugin->frames_remaining = 0;
+      }
+    } else {
+      /* a pad has run out of data */
+      GST_BUFFER_SIZE (buf_out) = GST_BUFFER_SIZE (buf_out)
+        - frames_to_write * numchannels * sizeof (gint16);
 
-    gst_element_yield (element);
-  } while (TRUE);
+      if (GST_BUFFER_SIZE (buf_out)) {
+        GST_BUFFER_TIMESTAMP (buf_out) = plugin->offset * GST_SECOND / plugin->rate;
+        gst_pad_push (plugin->srcpad, GST_DATA (buf_out));
+      } else {
+        /* we got the eos event on one of the pads */
+        gst_buffer_unref (buf_out);
+        gst_pad_push (plugin->srcpad, GST_DATA ((GstBuffer*)gst_event_new (GST_EVENT_EOS)));
+        gst_element_set_eos (element);
+      }
+      
+      return;
+    }
+  }
+  
+  GST_BUFFER_TIMESTAMP (buf_out) = plugin->offset * GST_SECOND / plugin->rate;
+  
+  plugin->offset += num_frames;
+  gst_pad_push (plugin->srcpad, GST_DATA (buf_out));
 }
 
 static GstElementStateReturn
@@ -483,10 +503,25 @@ gst_float2int_change_state (GstElement *element)
       break;
       
     case GST_STATE_PAUSED_TO_READY:
-      plugin->intcaps = NULL;
-      plugin->floatcaps = NULL;
+      if (plugin->floatcaps) {
+        gst_caps_unref (plugin->floatcaps);
+        plugin->floatcaps = NULL;
+      }
+      if (plugin->intcaps) {
+        gst_caps_unref (plugin->intcaps);
+        plugin->intcaps = NULL;
+      }
+      for (p = plugin->channels; p; p = p->next) {
+	GstFloat2IntInputChannel *c = p->data;
+
+        if (c->buffer)
+          gst_buffer_unref (c->buffer);
+        c->buffer = NULL;
+	c->eos = FALSE;
+      }
+      plugin->frames_remaining = 0;
       break;
-      
+
     default:
       break;
   }
@@ -502,9 +537,6 @@ gst_float2int_factory_init (GstPlugin *plugin)
 {
   GstElementFactory *factory;
   
-  if (! gst_library_load ("gstbytestream"))
-    return FALSE;
-
   factory = gst_element_factory_new ("float2int", GST_TYPE_FLOAT2INT,
                                      &float2int_details);
   g_return_val_if_fail(factory != NULL, FALSE);
