@@ -1,10 +1,13 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "mulaw-encode.h"
 #include "mulaw-decode.h"
 
 /* elementfactory information */
 static GstElementDetails mulawenc_details = {
   "PCM to Mu Law conversion",
-  "Filter/Audio/Conversion",
+  "Codec/Audio/Encoder",
   "LGPL",
   "Convert 16bit PCM to 8bit mu law",
   VERSION,
@@ -15,7 +18,7 @@ static GstElementDetails mulawenc_details = {
 /* elementfactory information */
 static GstElementDetails mulawdec_details = {
   "Mu Law to PCM conversion",
-  "Filter/Audio/Conversion",
+  "Codec/Audio/Decoder",
   "LGPL",
   "Convert 8bit mu law to 16bit PCM",
   VERSION,
@@ -29,13 +32,13 @@ mulaw_factory (void)
   return 
     gst_caps_new (
   	"test_src",
-    	"audio/raw",
+    	"audio/x-mulaw",
 	gst_props_new (
-    	  "format",   GST_PROPS_STRING ("int"),
-    	    "law",    GST_PROPS_INT (1),
-    	    "width",  GST_PROPS_INT(8),
-    	    "depth",  GST_PROPS_INT(8),
-    	    "signed", GST_PROPS_BOOLEAN(FALSE),
+    	    "width",    GST_PROPS_INT(8),
+    	    "depth",    GST_PROPS_INT(8),
+    	    "signed",   GST_PROPS_BOOLEAN(FALSE),
+    	    "rate",     GST_PROPS_INT_RANGE (8000, 192000),
+            "channels", GST_PROPS_INT_RANGE (1, 2),
 	    NULL));
 }
 
@@ -45,14 +48,14 @@ linear_factory (void)
   return 
     gst_caps_new (
   	"test_sink",
-    	"audio/raw",
+    	"audio/x-raw-int",
 	gst_props_new (
-    	  "format",     GST_PROPS_STRING ("int"),
-      	    "law",      GST_PROPS_INT(0),
-      	    "width",    GST_PROPS_INT(16),
-      	    "depth",    GST_PROPS_INT(16),
-      	    "signed",   GST_PROPS_BOOLEAN(TRUE),
-      	    "endianness",    GST_PROPS_INT(G_BYTE_ORDER),
+      	    "width",      GST_PROPS_INT(16),
+      	    "depth",      GST_PROPS_INT(16),
+      	    "signed",     GST_PROPS_BOOLEAN(TRUE),
+      	    "endianness", GST_PROPS_INT(G_BYTE_ORDER),
+    	    "rate",       GST_PROPS_INT_RANGE (8000, 192000),
+            "channels",   GST_PROPS_INT_RANGE (1, 2),
 	    NULL));
 }
 
