@@ -28,7 +28,7 @@ extern "C" {
 #endif /* __cplusplus */
 
 #include <gst/gst.h>
-#include <gst/bytestream/bytestream.h>
+#include <gst/gstbytestream.h>
 	
 #include "modplug_types.h"	
 
@@ -75,9 +75,7 @@ struct _GstModPlug {
   gboolean need_discont;
   gboolean eos;
   gint64 seek_at;
-  guint64 total_samples;
   guint64 song_size;
-  guint64 bytes_read;
 
   CSoundFile *mSoundFile;
   gboolean opened; /* set to TRUE when mSoundFile is created */
