@@ -509,7 +509,7 @@ enum {
 	MIDIOUT_VOLUME,
 	MIDIOUT_PAN,
 	MIDIOUT_BANKSEL,
-	MIDIOUT_PROGRAM,
+	MIDIOUT_PROGRAM
 };
 
 
@@ -940,7 +940,7 @@ int _muldivr(long a, long b, long c);
      ({ unsigned short int __bsx = (x);					      \
         ((((__bsx) >> 8) & 0xff) | (((__bsx) & 0xff) << 8)); }))
 #else
-static __inline unsigned short int
+static inline unsigned short int
 bswap_16 (unsigned short int __bsx)
 {
   return ((((__bsx) >> 8) & 0xff) | (((__bsx) & 0xff) << 8));
@@ -955,7 +955,7 @@ bswap_16 (unsigned short int __bsx)
         ((((__bsx) & 0xff000000) >> 24) | (((__bsx) & 0x00ff0000) >>  8) |    \
 	 (((__bsx) & 0x0000ff00) <<  8) | (((__bsx) & 0x000000ff) << 24)); }))
 #else
-static __inline unsigned int
+static inline unsigned int
 bswap_32 (unsigned int __bsx)
 {
   return ((((__bsx) & 0xff000000) >> 24) | (((__bsx) & 0x00ff0000) >>  8) |
