@@ -18,6 +18,9 @@
  */
 
 /*#define GST_DEBUG_ENABLED */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gstmpegaudioparse.h>
 
 
@@ -32,6 +35,15 @@ static GstElementDetails mp3parse_details = {
   "(C) 1999",
 };
 
+static GstCaps * mp3_type_find (GstByteStream *bs, gpointer data);
+
+static GstTypeDefinition mp3type_definition = {
+  "mp3_audio/mpeg",
+  "audio/mpeg",
+  ".mp3 .mp2 .mp1 .mpga",
+  mp3_type_find,
+};
+
 static GstPadTemplate*
 mp3_src_factory (void)
 {
@@ -42,14 +54,13 @@ mp3_src_factory (void)
   	GST_PAD_ALWAYS,
   	gst_caps_new (
   	  "mp3parse_src",
-    	  "audio/x-mp3",
-	  /*
+    	  "audio/mpeg",
 	  gst_props_new (
+	    "mpegversion", GST_PROPS_INT (1),
     	    "layer",   GST_PROPS_INT_RANGE (1, 3),
-    	    "bitrate", GST_PROPS_INT_RANGE (8, 320),
-    	    "framed",  GST_PROPS_BOOLEAN (TRUE),
-	    */
-	    NULL),
+            "rate",    GST_PROPS_INT_RANGE (8000, 48000),
+            "channels", GST_PROPS_INT_RANGE (1, 2),
+	    NULL)),
 	NULL);
 }
 
@@ -63,7 +74,7 @@ mp3_sink_factory (void)
   	GST_PAD_ALWAYS,
   	gst_caps_new (
   	  "mp3parse_sink",
-    	  "audio/x-mp3",
+    	  "audio/mpeg",
 	  NULL),
 	NULL);
 };
@@ -86,13 +97,14 @@ static GstPadTemplate *sink_temp, *src_temp;
 static void	gst_mp3parse_class_init		(GstMPEGAudioParseClass *klass);
 static void	gst_mp3parse_init		(GstMPEGAudioParse *mp3parse);
 
-static void	gst_mp3parse_loop		(GstElement *element);
-static void	gst_mp3parse_chain		(GstPad *pad,GstBuffer *buf);
+static void	gst_mp3parse_chain		(GstPad *pad,GstData *_data);
 static long	bpf_from_header			(GstMPEGAudioParse *parse, unsigned long header);
 static int	head_check			(unsigned long head);
 
 static void	gst_mp3parse_set_property		(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void	gst_mp3parse_get_property		(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
+static GstElementStateReturn
+		gst_mp3parse_change_state	(GstElement *element);
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_mp3parse_signals[LAST_SIGNAL] = { 0 }; */
@@ -112,11 +124,221 @@ gst_mp3parse_get_type(void) {
       0,
       (GInstanceInitFunc)gst_mp3parse_init,
     };
-    mp3parse_type = g_type_register_static(GST_TYPE_ELEMENT, "GstMPEGAudioParse", &mp3parse_info, 0);
+    mp3parse_type = g_type_register_static (GST_TYPE_ELEMENT,
+					    "GstMPEGAudioParse",
+					    &mp3parse_info, 0);
   }
   return mp3parse_type;
 }
 
+static guint mp3types_bitrates[2][3][16] =
+{ { {0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, },
+    {0, 32, 48, 56,  64,  80,  96, 112, 128, 160, 192, 224, 256, 320, 384, },
+    {0, 32, 40, 48,  56,  64,  80,  96, 112, 128, 160, 192, 224, 256, 320, } },
+  { {0, 32, 48, 56,  64,  80,  96, 112, 128, 144, 160, 176, 192, 224, 256, },
+    {0,  8, 16, 24,  32,  40,  48,  56,  64,  80,  96, 112, 128, 144, 160, },
+    {0,  8, 16, 24,  32,  40,  48,  56,  64,  80,  96, 112, 128, 144, 160, } },
+};
+
+static guint mp3types_freqs[3][3] =
+{ {44100, 48000, 32000},
+  {22050, 24000, 16000}, 
+  {11025, 12000,  8000}};
+
+static inline guint
+mp3_type_frame_length_from_header (guint32 header, guint *put_layer,
+				   guint *put_channels, guint *put_bitrate,
+				   guint *put_samplerate)
+{
+  guint length;
+  gulong mode, samplerate, bitrate, layer, version, channels;
+
+  /* we don't need extension, copyright, original or
+   * emphasis for the frame length */
+  header >>= 6;
+
+  /* mode */
+  mode = header & 0x3;
+  header >>= 3;
+
+  /* padding */
+  length = header & 0x1;
+  header >>= 1;
+
+  /* sampling frequency */
+  samplerate = header & 0x3;
+  if (samplerate == 3)
+    return 0;
+  header >>= 2;
+
+  /* bitrate index */
+  bitrate = header & 0xF;
+  if (bitrate == 15 || bitrate == 0)
+    return 0;
+
+  /* ignore error correction, too */
+  header >>= 5;
+
+  /* layer */
+  layer = 4 - (header & 0x3);
+  if (layer == 4)
+    return 0;
+  header >>= 2;
+
+  /* version */
+  version = header & 0x3;
+  if (version == 1)
+    return 0;
+
+  /* lookup */
+  channels = (mode == 3) ? 1 : 2;
+  bitrate = mp3types_bitrates[version == 3 ? 0 : 1][layer - 1][bitrate];
+  samplerate = mp3types_freqs[version > 0 ? version - 1 : 0][samplerate];
+
+  /* calculating */
+  if (layer == 1) {
+    length = ((12000 * bitrate / samplerate) + length) * 4;
+  } else {
+    length += ((layer == 3 && version == 0) ? 144000 : 72000) * bitrate / samplerate;
+  }
+
+  GST_DEBUG ("Calculated mp3 frame length of %u bytes", length);
+  GST_DEBUG ("samplerate = %lu - bitrate = %lu - layer = %lu - version = %lu"
+	     " - channels = %lu",
+	     samplerate, bitrate, layer, version, channels);
+
+  if (put_layer)
+    *put_layer = layer;
+  if (put_channels)
+    *put_channels = channels;
+  if (put_bitrate)
+    *put_bitrate = bitrate;
+  if (put_samplerate)
+    *put_samplerate = samplerate;
+
+  return length;
+}
+
+/**
+ * The chance that random data is identified as a valid mp3 header is 63 / 2^18
+ * (0.024%) per try. This makes the function for calculating false positives
+ *   1 - (1 - ((63 / 2 ^18) ^ GST_MP3_TYPEFIND_MIN_HEADERS)) ^ buffersize)
+ * This has the following probabilities of false positives:
+ * bufsize	          MIN_HEADERS
+ * (bytes)	1	2	3	4
+ * 4096		62.6%	 0.02%	 0%	 0%
+ * 16384	98%	 0.09%	 0%	 0%
+ * 1 MiB       100%	 5.88%	 0%	 0%
+ * 1 GiB       100%    100%	 1.44%   0%
+ * 1 TiB       100%    100%    100%      0.35%
+ * This means that the current choice (3 headers by most of the time 4096 byte
+ * buffers is pretty safe for now.
+ *
+ * The max. size of each frame is 1440 bytes, which means that for N frames
+ * to be detected, we need 1440 * GST_MP3_TYPEFIND_MIN_HEADERS + 3 of data.
+ * Assuming we step into the stream right after the frame header, this
+ * means we need 1440 * (GST_MP3_TYPEFIND_MIN_HEADERS + 1) - 1 + 3 bytes
+ * of data (5762) to always detect any mp3.
+ */
+
+/* increase this value when this function finds too many false positives */
+#define GST_MP3_TYPEFIND_MIN_HEADERS 3
+#define GST_MP3_TYPEFIND_MIN_DATA (1440 * (GST_MP3_TYPEFIND_MIN_HEADERS + 1) - 1 + 3)
+
+static GstCaps *
+mp3_caps_create (guint layer, guint channels,
+		 guint bitrate, guint samplerate)
+{
+  GstCaps *new;
+
+  g_assert (layer);
+  g_assert (samplerate);
+  g_assert (bitrate);
+  g_assert (channels);
+
+  new = GST_CAPS_NEW ("mp3_type_find",
+		      "audio/mpeg",
+			"mpegversion", GST_PROPS_INT (1),
+			"layer",       GST_PROPS_INT (layer),
+			/*"bitrate",     GST_PROPS_INT (bitrate),*/
+			"rate",        GST_PROPS_INT (samplerate),
+			"channels",    GST_PROPS_INT (channels));
+
+  return new;
+}
+
+static GstCaps *
+mp3_type_find (GstByteStream *bs, gpointer private)
+{
+  GstBuffer *buf = NULL;
+  GstCaps *new = NULL;
+  guint8 *data;
+  guint size;
+  guint32 head;
+  guint layer = 0, bitrate = 0, samplerate = 0, channels = 0;
+
+  /* note that even if we don't get the requested size,
+   * it might still be a (very small) mp3 */
+  gst_bytestream_peek (bs, &buf, GST_MP3_TYPEFIND_MIN_DATA);
+  if (!buf) {
+    goto done;
+  }
+  
+  data = GST_BUFFER_DATA (buf);
+  size = GST_BUFFER_SIZE (buf);
+  
+  while (size >= 4) {
+    head = GUINT32_FROM_BE(*((guint32 *)data));
+    if ((head & 0xffe00000) == 0xffe00000) {
+      guint length;
+      guint prev_layer = 0, prev_bitrate = 0,
+	    prev_channels = 0, prev_samplerate = 0;
+      guint found = 0; /* number of valid headers found */
+      guint pos = 0;
+
+      do {
+        if (!(length = mp3_type_frame_length_from_header (head, &layer,
+							  &channels, &bitrate,
+							  &samplerate))) {
+          break;
+	}
+	if ((prev_layer && prev_layer != layer) || !layer ||
+	    (prev_bitrate && prev_bitrate != bitrate) || !bitrate ||
+	    (prev_samplerate && prev_samplerate != samplerate) || !samplerate ||
+	    (prev_channels && prev_channels != channels) || !channels) {
+          /* this means an invalid property, or a change, which likely
+	   * indicates that this is not a mp3 but just a random bytestream */
+	  break;
+	}
+	prev_layer = layer;
+	prev_bitrate = bitrate;
+	prev_channels = channels;
+	prev_samplerate = samplerate;
+	pos += length;
+	if (++found >= GST_MP3_TYPEFIND_MIN_HEADERS) {
+          /* we're pretty sure that this is mp3 now */
+          new = mp3_caps_create (layer, channels, bitrate, samplerate);
+	  goto done;
+        }
+
+        /* and now, find a new head */
+        head = GUINT32_FROM_BE(*((guint32 *) &(data[pos])));  
+        if ((head & 0xffe00000) != 0xffe00000)
+	  break;
+      } while (TRUE);
+    }
+    data++;
+    size--;
+  }
+
+done:
+  if (buf != NULL) {
+    gst_buffer_unref (buf);
+  }
+
+  return new;
+}
+
 static void
 gst_mp3parse_class_init (GstMPEGAudioParseClass *klass)
 {
@@ -130,13 +352,15 @@ gst_mp3parse_class_init (GstMPEGAudioParseClass *klass)
     g_param_spec_int("skip","skip","skip",
                      G_MININT,G_MAXINT,0,G_PARAM_READWRITE)); /* CHECKME */
   g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_BIT_RATE,
-    g_param_spec_int("bit_rate","bit_rate","bit_rate",
+    g_param_spec_int("bitrate","Bitrate","Bit Rate",
                      G_MININT,G_MAXINT,0,G_PARAM_READABLE)); /* CHECKME */
 
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
 
   gobject_class->set_property = gst_mp3parse_set_property;
   gobject_class->get_property = gst_mp3parse_get_property;
+
+  gstelement_class->change_state = gst_mp3parse_change_state;
 }
 
 static void
@@ -145,11 +369,8 @@ gst_mp3parse_init (GstMPEGAudioParse *mp3parse)
   mp3parse->sinkpad = gst_pad_new_from_template(sink_temp, "sink");
   gst_element_add_pad(GST_ELEMENT(mp3parse),mp3parse->sinkpad);
 
-  gst_element_set_loop_function (GST_ELEMENT(mp3parse),gst_mp3parse_loop);
-#if 1	/* set this to one to use the old chaining code */
   gst_pad_set_chain_function(mp3parse->sinkpad,gst_mp3parse_chain);
   gst_element_set_loop_function (GST_ELEMENT(mp3parse),NULL);
-#endif
 
   mp3parse->srcpad = gst_pad_new_from_template(src_temp, "src");
   gst_element_add_pad(GST_ELEMENT(mp3parse),mp3parse->srcpad);
@@ -158,76 +379,14 @@ gst_mp3parse_init (GstMPEGAudioParse *mp3parse)
   mp3parse->partialbuf = NULL;
   mp3parse->skip = 0;
   mp3parse->in_flush = FALSE;
-}
 
-static guint32
-gst_mp3parse_next_header (guchar *buf,guint32 len,guint32 start)
-{
-  guint32 offset = start;
-  int f = 0;
-
-  while (offset < (len - 4)) {
-    fprintf(stderr,"%02x ",buf[offset]);
-    if (buf[offset] == 0xff)
-      f = 1;
-    else if (f && ((buf[offset] >> 4) == 0x0f))
-      return offset - 1;
-    else
-      f = 0;
-    offset++;
-  }
-  return -1;
+  mp3parse->rate = mp3parse->channels = mp3parse->layer = -1;
 }
 
 static void
-gst_mp3parse_loop (GstElement *element)
-{
-  GstMPEGAudioParse *parse = GST_MP3PARSE(element);
-  GstBuffer *inbuf, *outbuf;
-  guint32 size, offset;
-  guchar *data;
-  guint32 start;
-  guint32 header;
-  gint bpf;
-
-  while (1) {
-    /* get a new buffer */
-    inbuf = gst_pad_pull (parse->sinkpad);
-    size = GST_BUFFER_SIZE (inbuf);
-    data = GST_BUFFER_DATA (inbuf);
-    offset = 0;
-fprintf(stderr, "have buffer of %d bytes\n",size);
-
-    /* loop through it and find all the frames */
-    while (offset < (size - 4)) {
-      start = gst_mp3parse_next_header (data,size,offset);
-fprintf(stderr, "skipped %d bytes searching for the next header\n",start-offset);
-      header = GUINT32_FROM_BE(*((guint32 *)(data+start)));
-fprintf(stderr, "header is 0x%08x\n",header);
-
-      /* figure out how big the frame is supposed to be */
-      bpf = bpf_from_header (parse, header);
-
-      /* see if there are enough bytes in this buffer for the whole frame */
-      if ((start + bpf) <= size) {
-        outbuf = gst_buffer_create_sub (inbuf,start,bpf);
-fprintf(stderr, "sending buffer of %d bytes\n",bpf);
-        gst_pad_push (parse->srcpad, outbuf);
-        offset = start + bpf;
-
-      /* if not, we have to deal with it somehow */
-      } else {
-fprintf(stderr,"don't have enough data for this frame\n");
-        
-        break;
-      }
-    }
-  }
-}
-
-static void
-gst_mp3parse_chain (GstPad *pad, GstBuffer *buf)
+gst_mp3parse_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstMPEGAudioParse *mp3parse;
   guchar *data;
   glong size,offset = 0;
@@ -243,7 +402,7 @@ gst_mp3parse_chain (GstPad *pad, GstBuffer *buf)
 
   mp3parse = GST_MP3PARSE (gst_pad_get_parent (pad));
 
-  GST_DEBUG (0,"mp3parse: received buffer of %d bytes",GST_BUFFER_SIZE(buf));
+  GST_DEBUG ("mp3parse: received buffer of %d bytes",GST_BUFFER_SIZE(buf));
 
   last_ts = GST_BUFFER_TIMESTAMP(buf);
 
@@ -274,12 +433,12 @@ gst_mp3parse_chain (GstPad *pad, GstBuffer *buf)
   while (offset < size-4) {
     int skipped = 0;
 
-    GST_DEBUG (0,"mp3parse: offset %ld, size %ld ",offset, size);
+    GST_DEBUG ("mp3parse: offset %ld, size %ld ",offset, size);
 
     /* search for a possible start byte */
     for (;((data[offset] != 0xff) && (offset < size));offset++) skipped++;
     if (skipped && !mp3parse->in_flush) {
-      GST_DEBUG (0,"mp3parse: **** now at %ld skipped %d bytes",offset,skipped);
+      GST_DEBUG ("mp3parse: **** now at %ld skipped %d bytes",offset,skipped);
     }
     /* construct the header word */
     header = GUINT32_FROM_BE(*((guint32 *)(data+offset)));
@@ -306,12 +465,15 @@ gst_mp3parse_chain (GstPad *pad, GstBuffer *buf)
         if ((size-offset)<(bpf+4)) { if (mp3parse->in_flush) break; } /* wait until we have the the entire current frame as well as the next frame header */
 
         header2 = GUINT32_FROM_BE(*((guint32 *)(data+offset+bpf)));
-        GST_DEBUG(0,"mp3parse: header=%08X, header2=%08X, bpf=%d", (unsigned int)header, (unsigned int)header2, bpf );
+        GST_DEBUG ("mp3parse: header=%08X, header2=%08X, bpf=%d", (unsigned int)header, (unsigned int)header2, bpf );
 
-        #define HDRMASK ~( (0xF<<12)/*bitrate*/ | (1<<9)/*padding*/ | (3<<4)/*mode extension*/ ) /* mask the bits which are allowed to differ between frames */
+/* mask the bits which are allowed to differ between frames */
+#define HDRMASK ~((0xF << 12)  /* bitrate */ | \
+		  (0x1 <<  9)  /* padding */ | \
+		  (0x3 <<  4)) /*mode extension*/
 
         if ( (header2&HDRMASK) != (header&HDRMASK) ) { /* require 2 matching headers in a row */
-           GST_DEBUG(0,"mp3parse: next header doesn't match (header=%08X, header2=%08X, bpf=%d)", (unsigned int)header, (unsigned int)header2, bpf );
+           GST_DEBUG ("mp3parse: next header doesn't match (header=%08X, header2=%08X, bpf=%d)", (unsigned int)header, (unsigned int)header2, bpf );
            offset++; /* This frame is invalid.  Start looking for a valid frame at the next position in the stream */
            continue;
         }
@@ -320,7 +482,7 @@ gst_mp3parse_chain (GstPad *pad, GstBuffer *buf)
 
       /* if we don't have the whole frame... */
       if ((size - offset) < bpf) {
-        GST_DEBUG (0,"mp3parse: partial buffer needed %ld < %d ",(size-offset), bpf);
+        GST_DEBUG ("mp3parse: partial buffer needed %ld < %d ",(size-offset), bpf);
 	break;
       } else {
 
@@ -328,30 +490,37 @@ gst_mp3parse_chain (GstPad *pad, GstBuffer *buf)
 
         offset += bpf;
 	if (mp3parse->skip == 0) {
-          GST_DEBUG (0,"mp3parse: pushing buffer of %d bytes",GST_BUFFER_SIZE(outbuf));
+          GST_DEBUG ("mp3parse: pushing buffer of %d bytes",GST_BUFFER_SIZE(outbuf));
 	  if (mp3parse->in_flush) {
 	    /* FIXME do some sort of flush event */
 	    mp3parse->in_flush = FALSE;
 	  }
 	  GST_BUFFER_TIMESTAMP(outbuf) = last_ts;
-          gst_pad_push(mp3parse->srcpad,outbuf);
+	  GST_BUFFER_DURATION(outbuf) = 8 * (GST_SECOND/1000) * GST_BUFFER_SIZE(outbuf) / mp3parse->bit_rate;
+
+          if (GST_PAD_CAPS (mp3parse->srcpad) != NULL) {
+            gst_pad_push(mp3parse->srcpad,GST_DATA (outbuf));
+          } else {
+            GST_DEBUG ("No capsnego yet, delaying buffer push");
+            gst_buffer_unref (outbuf);
+          }
 	}
 	else {
-          GST_DEBUG (0,"mp3parse: skipping buffer of %d bytes",GST_BUFFER_SIZE(outbuf));
+          GST_DEBUG ("mp3parse: skipping buffer of %d bytes",GST_BUFFER_SIZE(outbuf));
           gst_buffer_unref(outbuf);
 	  mp3parse->skip--;
 	}
       }
     } else {
       offset++;
-      if (!mp3parse->in_flush) GST_DEBUG (0,"mp3parse: *** wrong header, skipping byte (FIXME?)");
+      if (!mp3parse->in_flush) GST_DEBUG ("mp3parse: *** wrong header, skipping byte (FIXME?)");
     }
   }
   /* if we have processed this block and there are still */
   /* bytes left not in a partial block, copy them over. */
   if (size-offset > 0) {
     glong remainder = (size - offset);
-    GST_DEBUG (0,"mp3parse: partial buffer needed %ld for trailing bytes",remainder);
+    GST_DEBUG ("mp3parse: partial buffer needed %ld for trailing bytes",remainder);
 
     outbuf = gst_buffer_create_sub(mp3parse->partialbuf,offset,remainder);
     gst_buffer_unref(mp3parse->partialbuf);
@@ -363,74 +532,62 @@ gst_mp3parse_chain (GstPad *pad, GstBuffer *buf)
   }
 }
 
-static int mp3parse_tabsel[2][3][16] =
-{ { {0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, },
-    {0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, },
-    {0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, } },
-  { {0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, },
-    {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, },
-    {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, } },
-};
-
-static long mp3parse_freqs[9] =
-{44100, 48000, 32000, 22050, 24000, 16000, 11025, 12000, 8000};
-
-
 static long
 bpf_from_header (GstMPEGAudioParse *parse, unsigned long header)
 {
-  int layer_index,layer,lsf,samplerate_index,padding;
-  long bpf;
+  guint bitrate, layer, rate, channels, length;
 
-  /*mpegver = (header >> 19) & 0x3; // don't need this for bpf */
-  layer_index = (header >> 17) & 0x3;
-  layer = 4 - layer_index;
-  lsf = (header & (1 << 20)) ? ((header & (1 << 19)) ? 0 : 1) : 1;
-  parse->bit_rate = mp3parse_tabsel[lsf][layer - 1][((header >> 12) & 0xf)];
-  samplerate_index = (header >> 10) & 0x3;
-  padding = (header >> 9) & 0x1;
-
-  if (layer == 1) {
-    bpf = parse->bit_rate * 12000;
-    bpf /= mp3parse_freqs[samplerate_index];
-    bpf = ((bpf + padding) << 2);
-  } else {
-    bpf = parse->bit_rate * 144000;
-    bpf /= mp3parse_freqs[samplerate_index];
-    bpf += padding;
+  if (!(length = mp3_type_frame_length_from_header (header, &layer,
+						    &channels,
+						    &bitrate, &rate))) {
+    return 0;
   }
 
-  /*g_print("%08x: layer %d lsf %d bitrate %d samplerate_index %d padding %d - bpf %d\n", */
-/*header,layer,lsf,bitrate,samplerate_index,padding,bpf); */
+  if (channels != parse->channels ||
+      rate     != parse->rate     ||
+      layer    != parse->layer    ||
+      bitrate  != parse->bit_rate) {
+    GstCaps *caps = mp3_caps_create (layer, channels, bitrate, rate);
 
-  return bpf;
+    if (gst_pad_try_set_caps(parse->srcpad, caps) <= 0) {
+      gst_element_error (GST_ELEMENT (parse),
+                         "mp3parse: failed to negotiate format with next element");
+    }
+
+    parse->channels = channels;
+    parse->layer    = layer;
+    parse->rate     = rate;
+    parse->bit_rate = bitrate;
+  }
+
+  return length;
 }
 
 static gboolean
 head_check (unsigned long head)
 {
-  GST_DEBUG (0,"checking mp3 header 0x%08lx",head);
+  GST_DEBUG ("checking mp3 header 0x%08lx",head);
   /* if it's not a valid sync */
   if ((head & 0xffe00000) != 0xffe00000) {
-    GST_DEBUG (0,"invalid sync");return FALSE; }
+    GST_DEBUG ("invalid sync");return FALSE; }
   /* if it's an invalid MPEG version */
   if (((head >> 19) & 3) == 0x1) {
-    GST_DEBUG (0,"invalid MPEG version");return FALSE; }
+    GST_DEBUG ("invalid MPEG version");return FALSE; }
   /* if it's an invalid layer */
   if (!((head >> 17) & 3)) {
-    GST_DEBUG (0,"invalid layer");return FALSE; }
+    GST_DEBUG ("invalid layer");return FALSE; }
   /* if it's an invalid bitrate */
   if (((head >> 12) & 0xf) == 0x0) {
-    GST_DEBUG (0,"invalid bitrate");return FALSE; }
+    GST_DEBUG ("invalid bitrate");return FALSE; }
   if (((head >> 12) & 0xf) == 0xf) {
-    GST_DEBUG (0,"invalid bitrate");return FALSE; }
+    GST_DEBUG ("invalid bitrate");return FALSE; }
   /* if it's an invalid samplerate */
   if (((head >> 10) & 0x3) == 0x3) {
-    GST_DEBUG (0,"invalid samplerate");return FALSE; }
+    GST_DEBUG ("invalid samplerate");return FALSE; }
   if ((head & 0xffff0000) == 0xfffe0000) { 
-    GST_DEBUG (0,"invalid sync");return FALSE; }
+    GST_DEBUG ("invalid sync");return FALSE; }
   if (head & 0x00000002) {
-        GST_DEBUG (0,"invalid emphasis");return FALSE; }
+        GST_DEBUG ("invalid emphasis");return FALSE; }
 
   return TRUE;
 }
@@ -475,10 +632,33 @@ gst_mp3parse_get_property (GObject *object, guint prop_id, GValue *value, GParam
   }
 }
 
+static GstElementStateReturn 
+gst_mp3parse_change_state (GstElement *element) 
+{
+  GstMPEGAudioParse *src;
+
+  g_return_val_if_fail(GST_IS_MP3PARSE(element), GST_STATE_FAILURE);
+  src = GST_MP3PARSE(element);
+
+  switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_PAUSED_TO_READY:
+      src->channels = -1; src->rate = -1; src->layer = -1;
+      break;
+    default:
+      break;
+  }
+
+  if (GST_ELEMENT_CLASS(parent_class)->change_state)
+    return GST_ELEMENT_CLASS(parent_class)->change_state(element);
+
+  return GST_STATE_SUCCESS;
+}
+
 static gboolean
 plugin_init (GModule *module, GstPlugin *plugin)
 {
   GstElementFactory *factory;
+  GstTypeFactory *type;
 
   /* create an elementfactory for the mp3parse element */
   factory = gst_element_factory_new ("mp3parse",
@@ -494,6 +674,10 @@ plugin_init (GModule *module, GstPlugin *plugin)
 
   gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
 
+  /* type finding */
+  type = gst_type_factory_new (&mp3type_definition);
+  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
+
   return TRUE;
 }
 
