@@ -54,25 +54,26 @@ gst_mpeg_packetize_destroy (GstMPEGPacketize *packetize)
 }
 
 static GstData*
-parse_packhead (GstMPEGPacketize * packetize)
+parse_packhead (GstMPEGPacketize *packetize)
 {
   gint length = 8 + 4;
   guint8 *buf;
   GstBuffer *outbuf;
   guint32 got_bytes;
 
-  GST_DEBUG (0, "packetize: in parse_packhead");
+  GST_DEBUG ("packetize: in parse_packhead");
 
   got_bytes = gst_bytestream_peek_bytes (packetize->bs, &buf, length);
   if (got_bytes < length)
     return NULL;
+
   buf += 4;
 
-  GST_DEBUG (0, "code %02x", *buf);
+  GST_DEBUG ("code %02x", *buf);
 
   /* start parsing the stream */
   if ((*buf & 0xc0) == 0x40) {
-    GST_DEBUG (0, "packetize::parse_packhead setting mpeg2");
+    GST_DEBUG ("packetize::parse_packhead setting mpeg2");
     packetize->MPEG2 = TRUE;
     length += 2;
     got_bytes = gst_bytestream_peek_bytes (packetize->bs, &buf, length);
@@ -80,7 +81,7 @@ parse_packhead (GstMPEGPacketize * packetize)
       return NULL;
   }
   else {
-    GST_DEBUG (0, "packetize::parse_packhead setting mpeg1");
+    GST_DEBUG ("packetize::parse_packhead setting mpeg1");
     packetize->MPEG2 = FALSE;
   }
 
@@ -91,6 +92,19 @@ parse_packhead (GstMPEGPacketize * packetize)
   return GST_DATA (outbuf);
 }
 
+static GstData*
+parse_end (GstMPEGPacketize *packetize)
+{
+  guint32 got_bytes;
+  GstBuffer *outbuf;
+
+  got_bytes = gst_bytestream_read (packetize->bs, &outbuf, 4);
+  if (got_bytes < 4)
+    return NULL;
+
+  return GST_DATA (outbuf);
+}
+
 static inline GstData*
 parse_generic (GstMPEGPacketize *packetize)
 {
@@ -100,15 +114,16 @@ parse_generic (GstMPEGPacketize *packetize)
   guint32 got_bytes;
   gint16 length = 6;
 
-  GST_DEBUG (0, "packetize: in parse_generic");
+  GST_DEBUG ("packetize: in parse_generic");
 
   got_bytes = gst_bytestream_peek_bytes (bs, (guint8**)&buf, length);
   if (got_bytes < 6) 
     return NULL;
+
   buf += 4;
 
   length += GUINT16_FROM_BE (*(guint16 *) buf);
-  GST_DEBUG (0, "packetize: header_length %d", length);
+  GST_DEBUG ("packetize: header_length %d", length);
 
   got_bytes = gst_bytestream_read (packetize->bs, &outbuf, length);
   if (got_bytes < length) 
@@ -129,17 +144,18 @@ parse_chunk (GstMPEGPacketize *packetize)
 
   chunksize = gst_bytestream_peek_bytes (bs, (guint8**)&buf, 4096);
   if (chunksize == 0)
-    return FALSE;
+    return NULL;
+
   offset = 4;
 
   code = GUINT32_FROM_BE (*((guint32 *)(buf+offset)));
 
-  GST_DEBUG (0, "code = %08x", code);
+  GST_DEBUG ("code = %08x", code);
 
   while ((code & 0xffffff00) != 0x100L) {
     code = (code << 8) | buf[offset++];
     
-    GST_DEBUG (0, "  code = %08x", code);
+    GST_DEBUG ("  code = %08x", code);
 
     if (offset == chunksize) {
       chunksize = gst_bytestream_peek_bytes (bs, (guint8**)&buf, offset + 4096);
@@ -168,18 +184,19 @@ find_start_code (GstMPEGPacketize *packetize)
   gint chunksize;
 
   chunksize = gst_bytestream_peek_bytes (bs, (guint8**)&buf, 4096);
-  if (chunksize == 0)
+  if (chunksize < 5)
     return FALSE;
+
   offset = 4;
 
   code = GUINT32_FROM_BE (*((guint32 *)(buf)));
 
-  GST_DEBUG (0, "code = %08x", code);
+  GST_DEBUG ("code = %08x %p %08x", code, buf, chunksize);
 
   while ((code & 0xffffff00) != 0x100L) {
     code = (code << 8) | buf[offset++];
     
-    GST_DEBUG (0, "  code = %08x", code);
+    GST_DEBUG ("  code = %08x %p %08x", code, buf, chunksize);
 
     if (offset == chunksize) {
       gst_bytestream_flush_fast (bs, offset);
@@ -204,15 +221,13 @@ gst_mpeg_packetize_read (GstMPEGPacketize *packetize)
   gboolean got_event = FALSE;
   GstData *outbuf = NULL;
 
-  if (packetize == NULL) {
-    g_warning ("gst_mpeg_packetize_read: packetize == NULL !");
-    return NULL;
-  }
+  g_return_val_if_fail (packetize != NULL, NULL);
+
   while (outbuf == NULL) {
     if (!find_start_code (packetize))
       got_event = TRUE;
     else {
-      GST_DEBUG (0, "packetize: have chunk 0x%02X", packetize->id);
+      GST_DEBUG ("packetize: have chunk 0x%02X", packetize->id);
       if (packetize->type == GST_MPEG_PACKETIZE_SYSTEM) {
 	if (packetize->resync) {
           if (packetize->id != PACK_START_CODE) {
@@ -233,8 +248,14 @@ gst_mpeg_packetize_read (GstMPEGPacketize *packetize)
 	    if (!outbuf)
 	      got_event = TRUE;
 	    break;
+	  case ISO11172_END_START_CODE:
+	    outbuf = parse_end (packetize);
+	    if (!outbuf)
+	      got_event = TRUE;
+	    break;
           default:
 	    if (packetize->MPEG2 && ((packetize->id < 0xBD) || (packetize->id > 0xFE))) {
+              gst_bytestream_flush (packetize->bs, 4);
 	      g_warning ("packetize: ******** unknown id 0x%02X",packetize->id);
 	    }
 	    else {
@@ -262,7 +283,7 @@ gst_mpeg_packetize_read (GstMPEGPacketize *packetize)
 
       switch (etype) {
         case GST_EVENT_DISCONTINUOUS:
-          GST_DEBUG (GST_CAT_EVENT, "packetize: discont\n");
+          GST_DEBUG ("packetize: discont\n");
 	  gst_bytestream_flush_fast (packetize->bs, remaining);
 	  break;
       }
