@@ -83,10 +83,8 @@ oneton_sink_factory (void)
 		GST_PAD_ALWAYS,
 		gst_caps_new (
 			"int_n_channel_sink",
-			"audio/raw",
+			"audio/x-raw-int",
 			gst_props_new (
-				"format", GST_PROPS_STRING ("int"),
-				"law", GST_PROPS_INT (0),
 				"endianness", GST_PROPS_INT (G_BYTE_ORDER),
 				"signed", GST_PROPS_BOOLEAN (TRUE),
 				"width", GST_PROPS_INT (16),
@@ -106,10 +104,8 @@ oneton_src_factory (void)
 		GST_PAD_SOMETIMES,
 		gst_caps_new (
 			"int_n_channel_src",
-			"audio/raw",
+			"audio/x-raw-int",
 			gst_props_new (
-				"format", GST_PROPS_STRING ("int"),
-				"law", GST_PROPS_INT (0),
 				"endianness", GST_PROPS_INT (G_BYTE_ORDER),
 				"signed", GST_PROPS_BOOLEAN (TRUE),
 				"width", GST_PROPS_INT (16),
@@ -125,7 +121,7 @@ GType gst_oneton_get_type (void);
 static void gst_oneton_class_init (GstOneToNClass *klass);
 static void gst_oneton_init (GstOneToN *oneton);
 static void gst_oneton_chain (GstPad *pad,
-			      GstBuffer *buf);
+			      GstData *_data);
 static void inline gst_oneton_fast_16bit_chain (gint16 *data,
 						int channels,
 						gint16 **mono_data,
@@ -167,6 +163,11 @@ gst_oneton_connect (GstPad *pad,
 		}
 
 		srccaps = gst_caps_copy (caps);
+
+		/* make sure we onw this copy */
+		gst_caps_ref (srccaps);
+		gst_caps_sink (srccaps);
+
 		gst_caps_set (srccaps, "channels", GST_PROPS_INT (1));
 		gst_caps_get_int (caps, "width", &oneton->width);
 
@@ -195,13 +196,18 @@ gst_oneton_connect (GstPad *pad,
 			pad = gst_pad_new_from_template (srctemplate, pad_name);
 			g_free (pad_name);
 
-			if (!gst_pad_try_set_caps (pad, srccaps)) {
+			/* FIXME handle DELAYED too */
+			if (gst_pad_try_set_caps (pad, srccaps) < 0) {
 				return GST_PAD_LINK_REFUSED;
 			}
 			gst_element_add_pad (GST_ELEMENT (oneton), pad);
 
 			oneton->srcpads = g_list_append (oneton->srcpads, pad);
 		}
+
+		/* we don't need the copy anymore, caps will be owned by the pads
+		 * when negotiation was successful */
+		gst_caps_unref (srccaps);
 	}
 
 	return GST_PAD_LINK_OK;
@@ -253,8 +259,9 @@ gst_oneton_init (GstOneToN *oneton)
 
 static void
 gst_oneton_chain (GstPad *pad,
-		  GstBuffer *buf)
+		  GstData *_data)
 {
+	GstBuffer *buf = GST_BUFFER (_data);
 	GstOneToN *oneton;
 	gint16 *data;
 	gint16 **mono_data;
@@ -282,7 +289,7 @@ gst_oneton_chain (GstPad *pad,
 	}
 
 	if (oneton->channels == 1) {
-		gst_pad_push (GST_PAD (oneton->srcpads->data), buf);
+		gst_pad_push (GST_PAD (oneton->srcpads->data), GST_DATA (buf));
 		return;
 	}
 	
@@ -325,7 +332,7 @@ gst_oneton_chain (GstPad *pad,
 	for (i = 0, p = oneton->srcpads; p; p = p->next, i++) {
 		GstPad *pad = p->data;
 
-		gst_pad_push (pad, out_bufs[i]);
+		gst_pad_push (pad, GST_DATA (out_bufs[i]));
 	}
 
 	g_free (out_bufs);
