@@ -19,6 +19,9 @@
 
 
 /*#define DEBUG_ENABLED */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gstvideoscale.h>
 #include <videoscale.h>
 
@@ -43,8 +46,6 @@ enum {
 
 enum {
   ARG_0,
-  ARG_WIDTH,
-  ARG_HEIGHT,
   ARG_METHOD,
   /* FILL ME */
 };
@@ -73,7 +74,7 @@ static void	gst_videoscale_init		(GstVideoscale *videoscale);
 static void	gst_videoscale_set_property		(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void	gst_videoscale_get_property		(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
-static void	gst_videoscale_chain		(GstPad *pad, GstBuffer *buf);
+static void	gst_videoscale_chain		(GstPad *pad, GstData *_data);
 static GstCaps * gst_videoscale_get_capslist(void);
 
 static GstElementClass *parent_class = NULL;
@@ -109,12 +110,6 @@ gst_videoscale_class_init (GstVideoscaleClass *klass)
   gobject_class = (GObjectClass*)klass;
   gstelement_class = (GstElementClass*)klass;
 
-  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_WIDTH,
-    g_param_spec_int("width","width","width",
-                     G_MININT,G_MAXINT,0,G_PARAM_READWRITE)); /* CHECKME */
-  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_HEIGHT,
-    g_param_spec_int("height","height","height",
-                     G_MININT,G_MAXINT,0,G_PARAM_READWRITE)); /* CHECKME */
   g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_METHOD,
     g_param_spec_enum("method","method","method",
                       GST_TYPE_VIDEOSCALE_METHOD,0,G_PARAM_READWRITE)); /* CHECKME! */
@@ -132,11 +127,23 @@ gst_videoscale_src_template_factory(void)
   static GstPadTemplate *templ = NULL;
 
   if(!templ){
-    GstCaps *caps = GST_CAPS_NEW("src","video/raw",
+    GstCaps *caps;
+    GstCaps *caps1 = GST_CAPS_NEW("src","video/x-raw-yuv",
 		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT));
+		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
+                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+    GstCaps *caps2 = GST_CAPS_NEW("src","video/x-raw-rgb",
+		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
+		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
+                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
 
-    caps = gst_caps_intersect(caps, gst_videoscale_get_capslist ());
+    caps = gst_caps_intersect(caps1, gst_videoscale_get_capslist ());
+    gst_caps_unref (caps1);
+    caps1 = caps;
+    caps = gst_caps_intersect(caps2, gst_videoscale_get_capslist ());
+    gst_caps_unref (caps2);
+    caps2 = caps;
+    caps = gst_caps_append(caps1, caps2);
 
     templ = GST_PAD_TEMPLATE_NEW("src", GST_PAD_SRC, GST_PAD_ALWAYS, caps);
   }
@@ -149,11 +156,23 @@ gst_videoscale_sink_template_factory(void)
   static GstPadTemplate *templ = NULL;
 
   if(!templ){
-    GstCaps *caps = GST_CAPS_NEW("sink","video/raw",
+    GstCaps *caps;
+    GstCaps *caps1 = GST_CAPS_NEW("src","video/x-raw-yuv",
 		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT));
+		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
+                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+    GstCaps *caps2 = GST_CAPS_NEW("src","video/x-raw-rgb",
+		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
+		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
+                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
 
-    caps = gst_caps_intersect(caps, gst_videoscale_get_capslist ());
+    caps = gst_caps_intersect(caps1, gst_videoscale_get_capslist ());
+    gst_caps_unref (caps1);
+    caps1 = caps;
+    caps = gst_caps_intersect(caps2, gst_videoscale_get_capslist ());
+    gst_caps_unref (caps2);
+    caps2 = caps;
+    caps = gst_caps_append(caps1, caps2);
 
     templ = GST_PAD_TEMPLATE_NEW("src", GST_PAD_SINK, GST_PAD_ALWAYS, caps);
   }
@@ -187,10 +206,10 @@ gst_videoscale_getcaps (GstPad *pad, GstCaps *caps)
   GstVideoscale *videoscale;
   GstCaps *capslist = NULL;
   GstCaps *peercaps;
-  GstCaps *sizecaps;
+  GstCaps *sizecaps1, *sizecaps2;
   int i;
 
-  GST_DEBUG(0,"gst_videoscale_src_link");
+  GST_DEBUG ("gst_videoscale_src_link");
   videoscale = GST_VIDEOSCALE (gst_pad_get_parent (pad));
   
   /* get list of peer's caps */
@@ -213,12 +232,22 @@ gst_videoscale_getcaps (GstPad *pad, GstCaps *caps)
   }
   gst_caps_unref (peercaps);
 
-  sizecaps = GST_CAPS_NEW("videoscale_size","video/raw",
+  sizecaps1 = GST_CAPS_NEW("src","video/x-raw-yuv",
 		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT));
+		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
+                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+  sizecaps2 = GST_CAPS_NEW("src","video/x-raw-rgb",
+		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
+		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
+                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
 
-  caps = gst_caps_intersect(caps, gst_videoscale_get_capslist ());
-  gst_caps_unref (sizecaps);
+  caps = gst_caps_intersect(sizecaps1, gst_videoscale_get_capslist ());
+  gst_caps_unref (sizecaps1);
+  sizecaps1 = caps;
+  caps = gst_caps_intersect(sizecaps2, gst_videoscale_get_capslist ());
+  gst_caps_unref (sizecaps2);
+  sizecaps2 = caps;
+  caps = gst_caps_append(sizecaps1, sizecaps2);
 
   return caps;
 }
@@ -231,26 +260,30 @@ gst_videoscale_src_link (GstPad *pad, GstCaps *caps)
   GstPadLinkReturn ret;
   GstCaps *peercaps;
 
-  GST_DEBUG(0,"gst_videoscale_src_link");
+  GST_DEBUG ("gst_videoscale_src_link");
   videoscale = GST_VIDEOSCALE (gst_pad_get_parent (pad));
 
   if (!GST_CAPS_IS_FIXED (caps)) {
     return GST_PAD_LINK_DELAYED;
   }
 
+  gst_caps_debug(caps,"ack");
+
   videoscale->format = videoscale_find_by_caps (caps);
   g_return_val_if_fail(videoscale->format, GST_PAD_LINK_REFUSED);
 
   gst_caps_get_int (caps, "width", &videoscale->to_width);
   gst_caps_get_int (caps, "height", &videoscale->to_height);
 
-  GST_DEBUG(0,"width %d height %d",videoscale->to_width,videoscale->to_height);
+  GST_DEBUG ("width %d height %d",videoscale->to_width,videoscale->to_height);
 
   peercaps = gst_caps_copy(caps);
 
   gst_caps_set(peercaps, "width", GST_PROPS_INT_RANGE (0, G_MAXINT));
   gst_caps_set(peercaps, "height", GST_PROPS_INT_RANGE (0, G_MAXINT));
 
+  g_print("setting caps to %s\n", gst_caps_to_string(peercaps));
+
   ret = gst_pad_try_set_caps (videoscale->srcpad, peercaps);
 
   gst_caps_unref(peercaps);
@@ -272,8 +305,10 @@ gst_videoscale_sink_link (GstPad *pad, GstCaps *caps)
   GstVideoscale *videoscale;
   GstPadLinkReturn ret;
   GstCaps *peercaps;
+  GstCaps *srccaps;
+  GstCaps *caps1;
 
-  GST_DEBUG(0,"gst_videoscale_src_link");
+  GST_DEBUG ("gst_videoscale_sink_link");
   videoscale = GST_VIDEOSCALE (gst_pad_get_parent (pad));
 
   if (!GST_CAPS_IS_FIXED (caps)) {
@@ -283,30 +318,44 @@ gst_videoscale_sink_link (GstPad *pad, GstCaps *caps)
   videoscale->format = videoscale_find_by_caps (caps);
   g_return_val_if_fail(videoscale->format, GST_PAD_LINK_REFUSED);
 
+  ret = gst_pad_try_set_caps (videoscale->srcpad, gst_caps_copy(caps));
+  if (ret == GST_PAD_LINK_OK || ret == GST_PAD_LINK_DONE) {
+    videoscale->passthru = TRUE;
+    return ret;
+  }
+
+  videoscale->passthru = FALSE;
+
+  srccaps = gst_caps_copy(caps);
+  gst_caps_set(srccaps, "width", GST_PROPS_INT_RANGE (0, G_MAXINT));
+  gst_caps_set(srccaps, "height", GST_PROPS_INT_RANGE (0, G_MAXINT));
+
+  peercaps = gst_pad_get_allowed_caps(videoscale->srcpad);
+
+  caps1 = gst_caps_intersect(peercaps, srccaps);
+
+  if (!GST_CAPS_IS_FIXED (caps1)) {
+    /* FIXME */
+    return GST_PAD_LINK_DELAYED;
+  }
+
+  g_print("setting caps to %s\n", gst_caps_to_string(caps1));
+
+  ret = gst_pad_try_set_caps (videoscale->srcpad, caps1);
+  if (ret != GST_PAD_LINK_OK && ret != GST_PAD_LINK_DONE) {
+    return ret;
+  }
+
+  gst_caps_get_int (caps1, "width", &videoscale->to_width);
+  gst_caps_get_int (caps1, "height", &videoscale->to_height);
+
   gst_caps_get_int (caps, "width", &videoscale->from_width);
   gst_caps_get_int (caps, "height", &videoscale->from_height);
+  gst_caps_get_float (caps, "framerate", &videoscale->framerate);
 
-  peercaps = gst_caps_copy(caps);
+  caps = gst_pad_get_caps (videoscale->srcpad);
 
-  if(videoscale->force_size){
-    gst_caps_set(peercaps, "width", GST_PROPS_INT (videoscale->forced_width));
-    gst_caps_set(peercaps, "height", GST_PROPS_INT (videoscale->forced_height));
-  }else{
-    gst_caps_set(peercaps, "width", GST_PROPS_INT_RANGE (0, G_MAXINT));
-    gst_caps_set(peercaps, "height", GST_PROPS_INT_RANGE (0, G_MAXINT));
-  }
-
-  ret = gst_pad_try_set_caps (videoscale->srcpad, peercaps);
-
-  gst_caps_unref(peercaps);
-
-  if(ret==GST_PAD_LINK_OK){
-    caps = gst_pad_get_caps (videoscale->srcpad);
-
-    gst_caps_get_int (caps, "width", &videoscale->to_width);
-    gst_caps_get_int (caps, "height", &videoscale->to_height);
-    gst_videoscale_setup(videoscale);
-  }
+  gst_videoscale_setup(videoscale);
 
   return ret;
 }
@@ -314,7 +363,7 @@ gst_videoscale_sink_link (GstPad *pad, GstCaps *caps)
 static void
 gst_videoscale_init (GstVideoscale *videoscale)
 {
-  GST_DEBUG(0,"gst_videoscale_init");
+  GST_DEBUG ("gst_videoscale_init");
   videoscale->sinkpad = gst_pad_new_from_template (
 		  GST_PAD_TEMPLATE_GET (gst_videoscale_sink_template_factory),
 		  "sink");
@@ -331,7 +380,6 @@ gst_videoscale_init (GstVideoscale *videoscale)
   gst_pad_set_getcaps_function(videoscale->srcpad,gst_videoscale_getcaps);
 
   videoscale->inited = FALSE;
-  videoscale->force_size = FALSE;
 
   videoscale->method = GST_VIDEOSCALE_NEAREST;
   /*videoscale->method = GST_VIDEOSCALE_BILINEAR; */
@@ -340,14 +388,15 @@ gst_videoscale_init (GstVideoscale *videoscale)
 
 
 static void
-gst_videoscale_chain (GstPad *pad, GstBuffer *buf)
+gst_videoscale_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstVideoscale *videoscale;
   guchar *data;
   gulong size;
   GstBuffer *outbuf;
 
-  GST_DEBUG (0,"gst_videoscale_chain");
+  GST_DEBUG ("gst_videoscale_chain");
 
   g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_PAD (pad));
@@ -360,14 +409,14 @@ gst_videoscale_chain (GstPad *pad, GstBuffer *buf)
   size = GST_BUFFER_SIZE(buf);
 
   if(videoscale->passthru){
-    gst_pad_push(videoscale->srcpad, buf);
+    gst_pad_push(videoscale->srcpad, GST_DATA (buf));
     return;
   }
 
-  GST_DEBUG (0,"gst_videoscale_chain: got buffer of %ld bytes in '%s'",size,
+  GST_DEBUG ("gst_videoscale_chain: got buffer of %ld bytes in '%s'",size,
 		              GST_OBJECT_NAME (videoscale));
  
-  GST_DEBUG(0,"size=%ld from=%dx%d to=%dx%d fromsize=%ld (should be %d) tosize=%d",
+  GST_DEBUG ("size=%ld from=%dx%d to=%dx%d fromsize=%ld (should be %d) tosize=%d",
 	size,
 	videoscale->from_width, videoscale->from_height,
 	videoscale->to_width, videoscale->to_height,
@@ -383,15 +432,15 @@ gst_videoscale_chain (GstPad *pad, GstBuffer *buf)
   GST_BUFFER_TIMESTAMP(outbuf) = GST_BUFFER_TIMESTAMP(buf);
 
   g_return_if_fail(videoscale->format);
-  GST_DEBUG (0,"format %s",videoscale->format->fourcc);
+  GST_DEBUG ("format %s",videoscale->format->fourcc);
   g_return_if_fail(videoscale->format->scale);
 
   videoscale->format->scale(videoscale, GST_BUFFER_DATA(outbuf), data);
 
-  GST_DEBUG (0,"gst_videoscale_chain: pushing buffer of %d bytes in '%s'",GST_BUFFER_SIZE(outbuf),
+  GST_DEBUG ("gst_videoscale_chain: pushing buffer of %d bytes in '%s'",GST_BUFFER_SIZE(outbuf),
 	              GST_OBJECT_NAME (videoscale));
 
-  gst_pad_push(videoscale->srcpad, outbuf);
+  gst_pad_push(videoscale->srcpad, GST_DATA (outbuf));
 
   gst_buffer_unref(buf);
 }
@@ -405,16 +454,8 @@ gst_videoscale_set_property (GObject *object, guint prop_id, const GValue *value
   g_return_if_fail(GST_IS_VIDEOSCALE(object));
   src = GST_VIDEOSCALE(object);
 
-  GST_DEBUG(0,"gst_videoscale_set_property");
+  GST_DEBUG ("gst_videoscale_set_property");
   switch (prop_id) {
-    case ARG_WIDTH:
-      src->forced_width = g_value_get_int (value);
-      src->force_size = TRUE;
-      break;
-    case ARG_HEIGHT:
-      src->forced_height = g_value_get_int (value);
-      src->force_size = TRUE;
-      break;
     case ARG_METHOD:
       src->method = g_value_get_enum (value);
       break;
@@ -433,12 +474,6 @@ gst_videoscale_get_property (GObject *object, guint prop_id, GValue *value, GPar
   src = GST_VIDEOSCALE(object);
 
   switch (prop_id) {
-    case ARG_WIDTH:
-      g_value_set_int (value, src->forced_width);
-      break;
-    case ARG_HEIGHT:
-      g_value_set_int (value, src->forced_height);
-      break;
     case ARG_METHOD:
       g_value_set_enum (value, src->method);
       break;
