@@ -17,7 +17,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gst/gst.h>
+#include <gst/audio/audio.h>
 #include "gstvolenv.h"
 
 #define MSG_L_IDX 17		/* 0x00020000 PLUGIN_LOADING */
@@ -47,47 +51,27 @@ enum {
   ARG_CONTROLPOINT
 };
 
-static GstPadTemplate*
-volenv_src_template_factory (void)
-{
-  static GstPadTemplate *template = NULL;
+GST_PAD_TEMPLATE_FACTORY (src_factory,
+  "src",
+  GST_PAD_SRC,
+  GST_PAD_ALWAYS,
+  gst_caps_new (
+    "volenv_src_int",
+    "audio/x-raw-int",
+      GST_AUDIO_INT_PAD_TEMPLATE_PROPS
+  )
+);
 
-  if (!template) {
-    template = gst_pad_template_new (
-      "src",
-      GST_PAD_SRC,
-      GST_PAD_ALWAYS,
-      gst_caps_new (
-       "test_src",
-       "audio/raw",
-       gst_props_new (
-         "channels", GST_PROPS_INT_RANGE (1, 2),
-	 NULL)),
-      NULL);
-  }
-  return template;
-}
-
-static GstPadTemplate*
-volenv_sink_template_factory (void)
-{
-  static GstPadTemplate *template = NULL;
-
-  if (!template) {
-    template = gst_pad_template_new (
-      "sink",
-      GST_PAD_SINK,
-      GST_PAD_ALWAYS,
-      gst_caps_new (
-        "test_sink",
-        "audio/raw",
-	gst_props_new (
-          "channels",    GST_PROPS_INT_RANGE (1, 2),
-	  NULL)),
-      NULL);
-  }
-  return template;
-}
+GST_PAD_TEMPLATE_FACTORY (sink_factory,
+  "sink",
+  GST_PAD_SINK,
+  GST_PAD_ALWAYS,
+  gst_caps_new (
+    "volenv_src_int",
+    "audio/x-raw-int",
+      GST_AUDIO_INT_PAD_TEMPLATE_PROPS
+  )
+);
 
 static void		gst_volenv_class_init		(GstVolEnvClass *klass);
 static void		gst_volenv_init			(GstVolEnv *filter);
@@ -97,10 +81,10 @@ static void		gst_volenv_set_property		(GObject *object, guint prop_id,
 static void		gst_volenv_get_property		(GObject *object, guint prop_id, 
 							 GValue *value, GParamSpec *pspec);
 
-static void		gst_volenv_chain		(GstPad *pad, GstBuffer *buf);
-static void inline 	gst_volenv_fast_16bit_chain 	(gint16* data, gint16** out_data, 
+static void		gst_volenv_chain		(GstPad *pad, GstData *_data);
+static void inline 	gst_volenv_fast_16bit_chain 	(gint16* data, gint16* out_data, 
 			         			 guint numsamples, GstVolEnv* filter);
-static void inline 	gst_volenv_fast_8bit_chain	(gint8* data, gint8** out_data,
+static void inline 	gst_volenv_fast_8bit_chain	(gint8* data, gint8* out_data,
                                 			 guint numsamples, GstVolEnv* filter);
 
 void print_volume_envelope (GstVolEnv* filter);
@@ -170,8 +154,10 @@ gst_volenv_class_init (GstVolEnvClass *klass)
 static void
 gst_volenv_init (GstVolEnv *filter)
 {
-  filter->sinkpad = gst_pad_new_from_template(volenv_sink_template_factory(),"sink");
-  filter->srcpad = gst_pad_new_from_template(volenv_src_template_factory(),"src");
+  filter->sinkpad = gst_pad_new_from_template(
+	GST_PAD_TEMPLATE_GET (sink_factory), "sink");
+  filter->srcpad = gst_pad_new_from_template(
+	GST_PAD_TEMPLATE_GET (src_factory), "src");
 
   filter->run_time = 0.0;
   filter->level = 1.0;
@@ -189,8 +175,9 @@ gst_volenv_init (GstVolEnv *filter)
 }
 
 static void
-gst_volenv_chain (GstPad *pad, GstBuffer *buf)
+gst_volenv_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstVolEnv *filter;
   gint16 *in_data;
   gint16 *out_data;
@@ -211,7 +198,6 @@ gst_volenv_chain (GstPad *pad, GstBuffer *buf)
 
 /*  printf ("DEBUG : time %f, level %f, increase %e\n", filter->run_time, filter->level, filter->increase); */
 
-  caps = NULL;
   caps = GST_PAD_CAPS(pad);
   if (caps == NULL) {
     gst_element_error (GST_ELEMENT (filter), "capsnego was never performed, bailing...\n");
@@ -233,31 +219,27 @@ gst_volenv_chain (GstPad *pad, GstBuffer *buf)
 
   switch (width) {
     case 16:
-	gst_volenv_fast_16bit_chain(in_data, &out_data, 
+	gst_volenv_fast_16bit_chain(in_data, out_data, 
             num_samples, filter);
 	break;
     case 8:
-	{
-	  gint8 *ptr;
-	  num_samples *= 2;
-	  gst_volenv_fast_8bit_chain((gint8*)in_data,&ptr,
-              num_samples, filter);
-	  out_data = (guint16 *)ptr;
-	}
+	num_samples *= 2;
+	gst_volenv_fast_8bit_chain((gint8*)in_data,(gint8*)out_data,
+            num_samples, filter);
 	break;
   }
   filter->run_time += (double) num_samples / (2 * sample_rate);
   gst_buffer_unref(buf);
-  gst_pad_push(filter->srcpad,outbuf);
+  gst_pad_push(filter->srcpad,GST_DATA (outbuf));
 }
 
 static void inline
-gst_volenv_fast_16bit_chain(gint16* in_data, gint16** out_data, 
+gst_volenv_fast_16bit_chain(gint16* in_data, gint16* out_data, 
 			         guint num_samples, GstVolEnv *filter)
 #include "filter.func"
 
 static void inline
-gst_volenv_fast_8bit_chain(gint8* in_data, gint8** out_data,
+gst_volenv_fast_8bit_chain(gint8* in_data, gint8* out_data,
                                 guint num_samples, GstVolEnv *filter)
 #include "filter.func"
 
@@ -294,7 +276,7 @@ gst_volenv_set_property (GObject *object, guint prop_id, const GValue *value, GP
         }
     filter->arg_rise = g_value_get_double (value);
     filter->increase = filter->arg_rise / (double) rate;
-    GST_DEBUG (MSG_E_IDX, "filter increase set to %e",
+    GST_DEBUG ("filter increase set to %e",
                  filter->increase);
 
     break;
@@ -314,7 +296,7 @@ gst_volenv_set_property (GObject *object, guint prop_id, const GValue *value, GP
 	  filter->control_point = g_value_get_string (value);
 
 	  sscanf (filter->control_point, "%lf:%lf", &cp_time, &cp_level);
-          GST_DEBUG (MSG_L_IDX, "volenv : added cp : level %f at time %f",
+          GST_DEBUG ("volenv : added cp : level %f at time %f",
 		cp_level, cp_time);
 	  *pcp_time = cp_time;
 	  *pcp_level = cp_level;
@@ -355,11 +337,11 @@ gst_volenv_set_property (GObject *object, guint prop_id, const GValue *value, GP
 	    	filter->rise = (cp_level - *psl) / (cp_time - *pst);
 	    }
 
-            GST_DEBUG (MSG_E_IDX, "second cp registered at %f level %f",
+            GST_DEBUG ("second cp registered at %f level %f",
 		cp_time, cp_level);
-            GST_DEBUG (MSG_E_IDX, "first cp registered at %f level %f",
+            GST_DEBUG ("first cp registered at %f level %f",
 		*pst, *psl);
-            GST_DEBUG (MSG_E_IDX, "second cp registered with rise %e",
+            GST_DEBUG ("second cp registered with rise %e",
 		filter->rise);
 	    /* new caps callback takes care of converting rise to increase */
 	  }
@@ -404,8 +386,10 @@ plugin_init (GModule *module, GstPlugin *plugin)
                                    &volenv_details);
   g_return_val_if_fail(factory != NULL, FALSE);
   
-  gst_element_factory_add_pad_template (factory, volenv_src_template_factory ());
-  gst_element_factory_add_pad_template (factory, volenv_sink_template_factory ());
+  gst_element_factory_add_pad_template (factory,
+	GST_PAD_TEMPLATE_GET (src_factory));
+  gst_element_factory_add_pad_template (factory,
+	GST_PAD_TEMPLATE_GET (sink_factory));
 
   gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
 
