@@ -18,6 +18,9 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 
 #include <gstwavparse.h>
@@ -25,7 +28,11 @@
 static void		gst_wavparse_class_init		(GstWavParseClass *klass);
 static void		gst_wavparse_init		(GstWavParse *wavparse);
 
-static GstCaps*		wav_type_find			(GstBuffer *buf, gpointer private);
+static GstElementStateReturn
+			gst_wavparse_change_state 	(GstElement *element);
+
+static GstCaps*		wav_type_find			(GstByteStream *bs,
+							 gpointer private);
 
 static const GstFormat*	gst_wavparse_get_formats	(GstPad *pad);
 static const GstQueryType *
@@ -39,7 +46,7 @@ static gboolean		gst_wavparse_pad_convert 	(GstPad *pad,
 							 gint64 src_value,
 							 GstFormat *dest_format,
 							 gint64 *dest_value);
-static void		gst_wavparse_chain		(GstPad *pad, GstBuffer *buf);
+static void		gst_wavparse_chain		(GstPad *pad, GstData *_data);
 
 static const GstEventMask*
 			gst_wavparse_get_event_masks 	(GstPad *pad);
@@ -73,10 +80,8 @@ GST_PAD_TEMPLATE_FACTORY (src_template_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "wavparse_raw",
-    "audio/raw",
-      "format",            GST_PROPS_STRING ("int"),
-       "law",              GST_PROPS_INT (0),
-       "endianness",       GST_PROPS_INT (G_BYTE_ORDER),
+    "audio/x-raw-int",
+       "endianness",       GST_PROPS_INT (G_LITTLE_ENDIAN),
        "signed",           GST_PROPS_LIST (
 				GST_PROPS_BOOLEAN (FALSE),
 				GST_PROPS_BOOLEAN (TRUE)
@@ -93,18 +98,21 @@ GST_PAD_TEMPLATE_FACTORY (src_template_factory,
        "channels",         GST_PROPS_INT_RANGE (1, 2)
   ),
   GST_CAPS_NEW (
-    "wavparse_mp3",
-    "audio/x-mp3",
-    NULL
+    "wavparse_mpeg",
+    "audio/mpeg",
+      "rate",              GST_PROPS_INT_RANGE (8000, 48000),
+      "channels",          GST_PROPS_INT_RANGE (1, 2),
+      "layer",             GST_PROPS_INT_RANGE (1, 3)
   ),
   GST_CAPS_NEW (
     "parsewav_law",
-    "audio/raw",
-      "format",            GST_PROPS_STRING ("int"),
-      "law",               GST_PROPS_INT_RANGE (1, 2),
-      "endianness",        GST_PROPS_INT (G_LITTLE_ENDIAN),
-      "width",             GST_PROPS_INT (8),
-      "depth",             GST_PROPS_INT (8),
+    "audio/x-alaw",
+      "rate",              GST_PROPS_INT_RANGE (8000, 48000),
+      "channels",          GST_PROPS_INT_RANGE (1, 2)
+  ),
+  GST_CAPS_NEW (
+    "parsewav_law",
+    "audio/x-mulaw",
       "rate",              GST_PROPS_INT_RANGE (8000, 48000),
       "channels",          GST_PROPS_INT_RANGE (1, 2)
   )
@@ -164,6 +172,8 @@ gst_wavparse_class_init (GstWavParseClass *klass)
   gstelement_class = (GstElementClass*) klass;
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+
+  gstelement_class->change_state = gst_wavparse_change_state;
 }
 
 static void 
@@ -194,7 +204,6 @@ gst_wavparse_init (GstWavParse *wavparse)
 
 
   wavparse->riff = NULL;
-
   wavparse->state = GST_WAVPARSE_UNKNOWN;
   wavparse->riff = NULL;
   wavparse->riff_nextlikely = 0;
@@ -205,75 +214,119 @@ gst_wavparse_init (GstWavParse *wavparse)
 }
 
 static GstCaps*
-wav_type_find (GstBuffer *buf, gpointer private)
+wav_type_find (GstByteStream *bs, gpointer private)
 {
-  gchar *data = GST_BUFFER_DATA (buf);
+  GstCaps *new = NULL;
+  GstBuffer *buf = NULL;
 
-  if (GST_BUFFER_SIZE (buf) < 12) return NULL;
-  if (strncmp (&data[0], "RIFF", 4)) return NULL;
-  if (strncmp (&data[8], "WAVE", 4)) return NULL;
+  if (gst_bytestream_peek (bs, &buf, 12) == 12) {
+    gchar *data = GST_BUFFER_DATA (buf);
 
-  return gst_caps_new ("wav_type_find", "audio/x-wav", NULL);
-}
-
-static inline void
-gst_wavparse_push (GstWavParse *wavparse,
-                   GstBuffer   *buf)
-{
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-  /* PCM Wav can be multi-byte, but the byte order is
-   * little endian. We prefer native byte order since
-   * that's what other elements expect. */
-  if (wavparse->format == GST_RIFF_WAVE_FORMAT_PCM) {
-    switch (wavparse->width) {
-      case 8:
-        /* nothing needed */
-        break;
-
-      case 16: {
-        GstBuffer *bsbuf;
-        gint16 *bsdata, *data;
-        guint i;
-
-        if (gst_buffer_needs_copy_on_write(buf)) {
-          bsbuf = gst_buffer_new_and_alloc(GST_BUFFER_SIZE(buf));
-        } else {
-          bsbuf = buf;
-        }
-
-        bsdata = (gint16 *) GST_BUFFER_DATA(bsbuf);
-        data = (gint16 *) GST_BUFFER_DATA(buf);
-
-        /* use GUINT16_FROM_LE for conversion */
-        for (i = 0; i < (GST_BUFFER_SIZE(bsbuf)/2); i++) {
-          bsdata[i] = GINT16_FROM_LE(data[i]);
-        }
-
-        /* if we've created a new buffer, we no longer need the old
-         * one here - so unref() it */
-        if (buf != bsbuf) {
-          GST_BUFFER_TIMESTAMP(bsbuf) = GST_BUFFER_TIMESTAMP(buf);
-          gst_buffer_unref(buf);
-        }
-
-        buf = bsbuf;
-        break;
-      }
-
-      default:
-        /* we only support 8/16 bit audio */
-        g_assert_not_reached();
-        break;
+    if (!strncmp (&data[0], "RIFF", 4) &&
+        !strncmp (&data[8], "WAVE", 4)) {
+      new = GST_CAPS_NEW ("wav_type_find",
+			  "audio/x-wav",
+			    NULL);
     }
   }
-#endif
 
-  gst_pad_push (wavparse->srcpad, buf);
+  if (buf != NULL) {
+    gst_buffer_unref (buf);
+  }
+
+  return new;
+}
+
+static void wav_new_chunk_callback(GstRiffChunk *chunk, gpointer data)
+{
+  GstWavParse *wavparse;
+
+  wavparse = GST_WAVPARSE (data);
+
+  GST_DEBUG("new tag " GST_FOURCC_FORMAT "\n", GST_FOURCC_ARGS(chunk->id));
+
+  if(chunk->id == GST_RIFF_TAG_fmt){
+    GstWavParseFormat *format;
+    GstCaps *caps = NULL;
+
+    /* we can gather format information now */
+    format = (GstWavParseFormat *)((guchar *) GST_BUFFER_DATA (wavparse->buf) + chunk->offset);
+
+    wavparse->bps      = GUINT16_FROM_LE(format->wBlockAlign);
+    wavparse->rate     = GUINT32_FROM_LE(format->dwSamplesPerSec);
+    wavparse->channels = GUINT16_FROM_LE(format->wChannels);
+    wavparse->width    = GUINT16_FROM_LE(format->wBitsPerSample);
+    wavparse->format	 = GINT16_FROM_LE(format->wFormatTag);
+
+    /* set the caps on the src pad */
+    /* FIXME: handle all of the other formats as well */
+    switch (wavparse->format)
+    {
+      case GST_RIFF_WAVE_FORMAT_ALAW:
+      case GST_RIFF_WAVE_FORMAT_MULAW: {
+	gchar *mime = (wavparse->format == GST_RIFF_WAVE_FORMAT_ALAW) ?
+			      "audio/x-alaw" : "audio/x-mulaw";
+	if (!(wavparse->width == 8)) {
+	  g_warning("Ignoring invalid width %d",
+		    wavparse->width);
+	  return;
+	}
+	caps = GST_CAPS_NEW (
+		      "parsewav_src",
+		      mime,
+			"rate",	GST_PROPS_INT (wavparse->rate),
+			"channels",	GST_PROPS_INT (wavparse->channels)
+	      );
+      }
+	break;
+      case GST_RIFF_WAVE_FORMAT_PCM:
+	caps = GST_CAPS_NEW (
+		      "parsewav_src",
+		      "audio/x-raw-int",
+			"endianness",	GST_PROPS_INT (G_LITTLE_ENDIAN),
+			"signed",     GST_PROPS_BOOLEAN ((wavparse->width > 8) ? TRUE : FALSE),
+			"width",	GST_PROPS_INT (wavparse->width),
+			"depth",	GST_PROPS_INT (wavparse->width),
+			"rate",	GST_PROPS_INT (wavparse->rate),
+			"channels",	GST_PROPS_INT (wavparse->channels)
+	      );
+	break;
+      case GST_RIFF_WAVE_FORMAT_MPEGL12:
+      case GST_RIFF_WAVE_FORMAT_MPEGL3: {
+	gint layer = (wavparse->format == GST_RIFF_WAVE_FORMAT_MPEGL12) ? 2 : 3;
+	caps = GST_CAPS_NEW (
+		      "parsewav_src",
+		      "audio/mpeg",
+			"layer",	GST_PROPS_INT (layer),
+			"rate",	GST_PROPS_INT (wavparse->rate),
+			"channels",	GST_PROPS_INT (wavparse->channels)
+	      );
+      }
+	break;
+
+      default:
+	gst_element_error (GST_ELEMENT (wavparse), "wavparse: format %d not handled", wavparse->format);
+	return;
+    }
+
+    if (gst_pad_try_set_caps (wavparse->srcpad, caps) <= 0) {
+      gst_element_error (GST_ELEMENT (wavparse), "Could not set caps");
+      return;
+    }
+
+    GST_DEBUG ("frequency %d, channels %d",
+	       wavparse->rate, wavparse->channels);
+
+    /* we're now looking for the data chunk */
+    wavparse->state = GST_WAVPARSE_CHUNK_DATA;
+  }
+
 }
 
 static void
-gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
+gst_wavparse_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstWavParse *wavparse;
   gboolean buffer_riffed = FALSE;	/* so we don't parse twice */
   gulong size;
@@ -284,11 +337,13 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
   g_return_if_fail (GST_BUFFER_DATA (buf) != NULL);
 
   wavparse = GST_WAVPARSE (gst_pad_get_parent (pad));
-  GST_DEBUG (0, "gst_wavparse_chain: got buffer in '%s'",
+  GST_DEBUG ("gst_wavparse_chain: got buffer in '%s'",
           gst_object_get_name (GST_OBJECT (wavparse)));
 
   size = GST_BUFFER_SIZE (buf);
 
+  wavparse->buf = buf;
+
   /* walk through the states in priority order */
   /* we're in the data region */
   if (wavparse->state == GST_WAVPARSE_DATA) {
@@ -298,8 +353,10 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
     /* we can't go beyond the max length */
     maxsize = wavparse->riff_nextlikely - GST_BUFFER_OFFSET (buf);
 
-    /* if we're expected to see a new chunk in this buffer */
-    if (maxsize < size) {
+    if (maxsize == 0) {
+      return;
+    } else if (maxsize < size) {
+      /* if we're expected to see a new chunk in this buffer */
       GstBuffer *newbuf;
 
       newbuf = gst_buffer_create_sub (buf, 0, maxsize);
@@ -322,16 +379,24 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
 		       &GST_BUFFER_TIMESTAMP (buf));
 
       if (wavparse->need_discont) {
-        gst_pad_push (wavparse->srcpad, 
-	              GST_BUFFER (gst_event_new_discontinuous (FALSE,
+        if (buf && GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
+          gst_pad_push (wavparse->srcpad, 
+	                GST_DATA (gst_event_new_discontinuous (FALSE,
+      			      GST_FORMAT_BYTES, wavparse->offset,
 			      GST_FORMAT_TIME, GST_BUFFER_TIMESTAMP (buf), 
 			      NULL)));
+	} else {
+          gst_pad_push (wavparse->srcpad, 
+	                GST_DATA (gst_event_new_discontinuous (FALSE,
+      			      GST_FORMAT_BYTES, wavparse->offset,
+			      NULL)));
+	}
         wavparse->need_discont = FALSE;
       }
-      gst_wavparse_push (wavparse, buf);
-    }
-    else
+      gst_pad_push (wavparse->srcpad, GST_DATA (buf));
+    } else {
       gst_buffer_unref (buf);
+    }
 
     wavparse->offset += size;
 
@@ -339,7 +404,8 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
   }
 
   if (wavparse->state == GST_WAVPARSE_OTHER) {
-    GST_DEBUG (0, "we're in unknown territory here, not passing on");
+    GST_DEBUG ("we're in unknown territory here, not passing on");
+    gst_buffer_unref(buf);
     return;
   }
 
@@ -352,143 +418,67 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
   if (wavparse->state == GST_WAVPARSE_UNKNOWN) {
     gint retval;
 
-    GST_DEBUG (0, "GstWavParse: checking for RIFF format");
+    GST_DEBUG ("GstWavParse: checking for RIFF format");
 
     /* create a new RIFF parser */
-    wavparse->riff = gst_riff_new ();
+    wavparse->riff = gst_riff_parser_new (wav_new_chunk_callback, wavparse);
 
     /* give it the current buffer to start parsing */
-    retval = gst_riff_next_buffer (wavparse->riff, buf, 0);
+    retval = gst_riff_parser_next_buffer (wavparse->riff, buf, 0);
     buffer_riffed = TRUE;
     if (retval < 0) {
-      GST_DEBUG (0, "sorry, isn't RIFF");
+      GST_DEBUG ("sorry, isn't RIFF");
+      gst_buffer_unref(buf);
       return;
     }
 
     /* this has to be a file of form WAVE for us to deal with it */
     if (wavparse->riff->form != gst_riff_fourcc_to_id ("WAVE")) {
-      GST_DEBUG (0, "sorry, isn't WAVE");
+      GST_DEBUG ("sorry, isn't WAVE");
+      gst_buffer_unref(buf);
       return;
     }
 
     /* at this point we're waiting for the 'fmt ' chunk */
-    wavparse->state = GST_WAVPARSE_CHUNK_FMT;
+    /* careful, the state may have changed in the callback */
+    if (wavparse->state == GST_WAVPARSE_UNKNOWN){
+      wavparse->state = GST_WAVPARSE_CHUNK_FMT;
+    }
   }
 
   /* we're now looking for the 'fmt ' chunk to get the audio info */
   if (wavparse->state == GST_WAVPARSE_CHUNK_FMT) {
-    GstRiffChunk *fmt;
-    GstWavParseFormat *format;
 
-    GST_DEBUG (0, "GstWavParse: looking for fmt chunk");
+    GST_DEBUG ("GstWavParse: looking for fmt chunk");
 
     /* there's a good possibility we may not have parsed this buffer */
     if (buffer_riffed == FALSE) {
-      gst_riff_next_buffer (wavparse->riff, buf, GST_BUFFER_OFFSET (buf));
+      gst_riff_parser_next_buffer (wavparse->riff, buf, GST_BUFFER_OFFSET (buf));
       buffer_riffed = TRUE;
     }
 
-    /* see if the fmt chunk is available yet */
-    fmt = gst_riff_get_chunk (wavparse->riff, "fmt ");
-
-    /* if we've got something, deal with it */
-    if (fmt != NULL) {
-      GstCaps *caps = NULL;
-
-      /* we can gather format information now */
-      format = (GstWavParseFormat *)((guchar *) GST_BUFFER_DATA (buf) + fmt->offset);
-
-      wavparse->bps      = GUINT16_FROM_LE(format->wBlockAlign);
-      wavparse->rate     = GUINT32_FROM_LE(format->dwSamplesPerSec);
-      wavparse->channels = GUINT16_FROM_LE(format->wChannels);
-      wavparse->width    = GUINT16_FROM_LE(format->wBitsPerSample);
-      wavparse->format	 = GINT16_FROM_LE(format->wFormatTag);
-
-      /* set the caps on the src pad */
-      /* FIXME: handle all of the other formats as well */
-      switch (wavparse->format)
-      {
-        case GST_RIFF_WAVE_FORMAT_ALAW:
-        case GST_RIFF_WAVE_FORMAT_MULAW:
-	  if (!(wavparse->width == 8)) {
-	    gst_element_error (GST_ELEMENT (wavparse), "wavparse: invalid wave file");
-	    return;
-	  }
-          caps = GST_CAPS_NEW (
-			"parsewav_src",
-			"audio/raw",
-			"format",	GST_PROPS_STRING ("int"),
-			  "law",	GST_PROPS_INT (wavparse->format == GST_RIFF_WAVE_FORMAT_ALAW ? 2 : 1),
-			  "endianness",	GST_PROPS_INT (G_BYTE_ORDER),
-			  "width",	GST_PROPS_INT (8),
-			  "depth",	GST_PROPS_INT (8),
-			  "rate",	GST_PROPS_INT (wavparse->rate),
-			  "channels",	GST_PROPS_INT (wavparse->channels)
-		);
-	  break;
-        case GST_RIFF_WAVE_FORMAT_PCM:
-          caps = GST_CAPS_NEW (
-			"parsewav_src",
-			"audio/raw",
-			"format",	GST_PROPS_STRING ("int"),
-			  "law",	GST_PROPS_INT (0),
-			  "endianness",	GST_PROPS_INT (G_BYTE_ORDER),
-			  "signed",     GST_PROPS_BOOLEAN ((wavparse->width > 8) ? TRUE : FALSE),
-			  "width",	GST_PROPS_INT (wavparse->width),
-			  "depth",	GST_PROPS_INT (wavparse->width),
-			  "rate",	GST_PROPS_INT (wavparse->rate),
-			  "channels",	GST_PROPS_INT (wavparse->channels)
-		);
-	  break;
-        case GST_RIFF_WAVE_FORMAT_MPEGL12:
-        case GST_RIFF_WAVE_FORMAT_MPEGL3:
-	  caps = GST_CAPS_NEW (
-			"parsewav_src",
-			"audio/x-mp3",
-			NULL
-		);
-	  break;
-	default:
-          gst_element_error (GST_ELEMENT (wavparse), "wavparse: format %d not handled", wavparse->format);
-          return;
-      }
-
-      if (gst_pad_try_set_caps (wavparse->srcpad, caps) <= 0) {
-        gst_element_error (GST_ELEMENT (wavparse), "Could not set caps");
-        return;
-      }
-
-      GST_DEBUG (0, "frequency %d, channels %d",
-		 wavparse->rate, wavparse->channels);
-
-      /* we're now looking for the data chunk */
-      wavparse->state = GST_WAVPARSE_CHUNK_DATA;
-    } else {
-      /* otherwise we just sort of give up for this buffer */
-      gst_buffer_unref (buf);
-      return;
-    }
+    return;
   }
 
   /* now we look for the data chunk */
   if (wavparse->state == GST_WAVPARSE_CHUNK_DATA) {
-    GstBuffer *newbuf;
     GstRiffChunk *datachunk;
 
-    GST_DEBUG (0, "GstWavParse: looking for data chunk");
+    GST_DEBUG ("GstWavParse: looking for data chunk");
 
     /* again, we might need to parse the buffer */
     if (buffer_riffed == FALSE) {
-      gst_riff_next_buffer (wavparse->riff, buf, GST_BUFFER_OFFSET (buf));
+      gst_riff_parser_next_buffer (wavparse->riff, buf, GST_BUFFER_OFFSET (buf));
       buffer_riffed = TRUE;
     }
 
-    datachunk = gst_riff_get_chunk (wavparse->riff, "data");
+    datachunk = gst_riff_parser_get_chunk (wavparse->riff, GST_RIFF_TAG_data);
 
     if (datachunk != NULL) {
       gulong subsize;
+      GstBuffer *newbuf;
 
-      GST_DEBUG (0, "data begins at %ld", datachunk->offset);
+      GST_DEBUG ("data begins at %ld", datachunk->offset);
 
       wavparse->datastart = datachunk->offset;
 
@@ -497,7 +487,7 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
 
       /* now we construct a new buffer for the remainder */
       subsize = size - datachunk->offset;
-      GST_DEBUG (0, "sending last %ld bytes along as audio", subsize);
+      GST_DEBUG ("sending last %ld bytes along as audio", subsize);
 
       newbuf = gst_buffer_create_sub (buf, datachunk->offset, subsize);
       gst_buffer_unref (buf);
@@ -505,7 +495,7 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
       GST_BUFFER_TIMESTAMP (newbuf) = 0;
 
       if (GST_PAD_IS_USABLE (wavparse->srcpad))
-        gst_wavparse_push (wavparse, newbuf);
+        gst_pad_push (wavparse->srcpad, GST_DATA (newbuf));
       else
 	gst_buffer_unref (newbuf);
 
@@ -515,13 +505,13 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
       wavparse->state = GST_WAVPARSE_DATA;
 
       /* however, we may be expecting another chunk at some point */
-      wavparse->riff_nextlikely = gst_riff_get_nextlikely (wavparse->riff);
-    } else {
-      /* otherwise we just sort of give up for this buffer */
-      gst_buffer_unref (buf);
+      wavparse->riff_nextlikely = gst_riff_parser_get_nextlikely (wavparse->riff);
+
       return;
     }
   }
+
+  gst_buffer_unref (buf);
 }
 
 /* convert and query stuff */
@@ -531,7 +521,7 @@ gst_wavparse_get_formats (GstPad *pad)
   static GstFormat formats[] = {
     GST_FORMAT_TIME,
     GST_FORMAT_BYTES,
-    GST_FORMAT_UNITS,	/* a "frame", ie a set of samples per Hz */
+    GST_FORMAT_DEFAULT,	/* a "frame", ie a set of samples per Hz */
     0,
     0
   };
@@ -548,15 +538,11 @@ gst_wavparse_pad_convert (GstPad *pad,
   GstWavParse *wavparse;
 
   wavparse = GST_WAVPARSE (gst_pad_get_parent (pad));
-  /* FIXME default should be samples in this case IMO */
-  if (*dest_format == GST_FORMAT_DEFAULT)
-    *dest_format = GST_FORMAT_TIME;
   
   bytes_per_sample = wavparse->channels * wavparse->width / 8;
   if (bytes_per_sample == 0) {
-    g_warning ("bytes_per_sample is 0, internal error\n");
-    g_warning ("channels %d,  width %d\n",
-	     wavparse->channels, wavparse->width);
+    GST_DEBUG ("bytes_per_sample is 0, probably an mp3 - channels %d,  width %d\n",
+	          wavparse->channels, wavparse->width);
     return FALSE;
   }
   byterate = (glong) (bytes_per_sample * wavparse->rate);
@@ -564,18 +550,18 @@ gst_wavparse_pad_convert (GstPad *pad,
     g_warning ("byterate is 0, internal error\n");
     return FALSE;
   }
-  GST_DEBUG (0, "bytes per sample: %d\n", bytes_per_sample);
+  GST_DEBUG ("bytes per sample: %d\n", bytes_per_sample);
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
-      if (*dest_format == GST_FORMAT_UNITS)
+      if (*dest_format == GST_FORMAT_DEFAULT)
         *dest_value = src_value / bytes_per_sample;
       else if (*dest_format == GST_FORMAT_TIME)
         *dest_value = src_value * GST_SECOND / byterate;
       else
         return FALSE;
       break;
-    case GST_FORMAT_UNITS:
+    case GST_FORMAT_DEFAULT:
       if (*dest_format == GST_FORMAT_BYTES)
         *dest_value = src_value * bytes_per_sample;
       else if (*dest_format == GST_FORMAT_TIME)
@@ -586,7 +572,7 @@ gst_wavparse_pad_convert (GstPad *pad,
     case GST_FORMAT_TIME:
       if (*dest_format == GST_FORMAT_BYTES)
 	*dest_value = src_value * byterate / GST_SECOND;
-      else if (*dest_format == GST_FORMAT_UNITS)
+      else if (*dest_format == GST_FORMAT_DEFAULT)
 	*dest_value = src_value * wavparse->rate / GST_SECOND;
       else
         return FALSE;
@@ -634,7 +620,7 @@ gst_wavparse_pad_query (GstPad *pad, GstQueryType type,
     g_warning ("Could not query sink pad's peer\n");
     return FALSE;
   }
-  GST_DEBUG (0, "pad_query done, value %" G_GINT64_FORMAT "\n", *value);
+  GST_DEBUG ("pad_query done, value %" G_GINT64_FORMAT "\n", *value);
   return TRUE;
 }
 
@@ -655,7 +641,7 @@ gst_wavparse_srcpad_event (GstPad *pad, GstEvent *event)
   GstWavParse *wavparse = GST_WAVPARSE (GST_PAD_PARENT (pad));
   gboolean res = FALSE;
 
-  GST_DEBUG(0, "event %d", GST_EVENT_TYPE (event));
+  GST_DEBUG ("event %d", GST_EVENT_TYPE (event));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
@@ -685,7 +671,7 @@ gst_wavparse_srcpad_event (GstPad *pad, GstEvent *event)
 	 seek = gst_event_new_seek (
 			 GST_FORMAT_BYTES |
 			 (GST_EVENT_SEEK_TYPE (event) & ~GST_SEEK_FORMAT_MASK), 
-			 byteoffset + wavparse->datastart);
+			 byteoffset + (GST_EVENT_SEEK_METHOD (event) == GST_SEEK_METHOD_END ? 0 : wavparse->datastart));
 
 	 res = gst_pad_send_event (GST_PAD_PEER (wavparse->sinkpad), seek);
 
@@ -705,12 +691,49 @@ gst_wavparse_srcpad_event (GstPad *pad, GstEvent *event)
   return res;
 }
 
+static GstElementStateReturn
+gst_wavparse_change_state (GstElement *element)
+{
+  GstWavParse *wavparse = GST_WAVPARSE (element);
+
+  switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_NULL_TO_READY:
+      break;
+    case GST_STATE_READY_TO_PAUSED:
+      break;
+    case GST_STATE_PAUSED_TO_PLAYING:
+      break;
+    case GST_STATE_PLAYING_TO_PAUSED:
+      break;
+    case GST_STATE_PAUSED_TO_READY:
+      wavparse->riff = NULL;
+      wavparse->state = GST_WAVPARSE_UNKNOWN;
+      wavparse->riff_nextlikely = 0;
+      wavparse->size = 0;
+      wavparse->bps = 0;
+      wavparse->offset = 0;
+      wavparse->need_discont = FALSE;
+      break;
+    case GST_STATE_READY_TO_NULL:
+      break;
+  }
+
+  if (GST_ELEMENT_CLASS (parent_class)->change_state)
+    return GST_ELEMENT_CLASS (parent_class)->change_state (element);
+
+  return GST_STATE_SUCCESS;
+}
+
 static gboolean
 plugin_init (GModule *module, GstPlugin *plugin)
 {
   GstElementFactory *factory;
   GstTypeFactory *type;
 
+  if(!gst_library_load("gstriff")){
+    return FALSE;
+  }
+
   /* create an elementfactory for the wavparse element */
   factory = gst_element_factory_new ("wavparse", GST_TYPE_WAVPARSE,
                                     &gst_wavparse_details);
