@@ -17,7 +17,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
+#include "gstosselement.h"
 #include "gstosssink.h"
 #include "gstosssrc.h"
 #include "gstossgst.h"
@@ -25,16 +29,16 @@
 static gboolean
 plugin_init (GModule *module, GstPlugin *plugin)
 {
-  gboolean ret;
+  if (!gst_library_load ("gstaudio"))
+    return FALSE;
 
-  ret = gst_osssink_factory_init (plugin);
-  g_return_val_if_fail (ret == TRUE, FALSE);
-
-  ret = gst_osssrc_factory_init (plugin);
-  g_return_val_if_fail (ret == TRUE, FALSE);
-
-  ret = gst_ossgst_factory_init (plugin);
-  g_return_val_if_fail (ret == TRUE, FALSE);
+  if (!gst_osselement_factory_init (plugin) ||
+      !gst_osssrc_factory_init (plugin) ||
+      !gst_osssink_factory_init (plugin) ||
+      !gst_ossgst_factory_init (plugin)) {
+    g_warning ("Failed to register OSS elements!");
+    return FALSE;
+  }
 
   return TRUE;
 }
