@@ -20,10 +20,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <sys/ioctl.h>
 #include <sys/soundcard.h>
+#include <errno.h>
+#include <unistd.h>
 
-#include <gstosssink.h>
+#include "gstosssink.h"
 
 /* elementfactory information */
 static GstElementDetails gst_osssink_details = {  
@@ -39,7 +44,7 @@ static GstElementDetails gst_osssink_details = {
 
 static void 			gst_osssink_class_init		(GstOssSinkClass *klass);
 static void 			gst_osssink_init		(GstOssSink *osssink);
-static void 			gst_osssink_finalize		(GObject *object);
+static void 			gst_osssink_dispose		(GObject *object);
 
 static GstElementStateReturn 	gst_osssink_change_state	(GstElement *element);
 static void		 	gst_osssink_set_clock		(GstElement *element, GstClock *clock);
@@ -55,14 +60,14 @@ static gboolean 		gst_osssink_query 		(GstElement *element, GstQueryType type,
 static gboolean 		gst_osssink_sink_query 		(GstPad *pad, GstQueryType type,
 								 GstFormat *format, gint64 *value);
 
-static GstPadLinkReturn	gst_osssink_sinkconnect		(GstPad *pad, GstCaps *caps);
+static GstPadLinkReturn		gst_osssink_sinkconnect		(GstPad *pad, GstCaps *caps);
 
 static void 			gst_osssink_set_property	(GObject *object, guint prop_id, const GValue *value, 
 		  						 GParamSpec *pspec);
 static void 			gst_osssink_get_property	(GObject *object, guint prop_id, GValue *value, 
 								 GParamSpec *pspec);
 
-static void 			gst_osssink_chain		(GstPad *pad,GstBuffer *buf);
+static void 			gst_osssink_chain		(GstPad *pad,GstData *_data);
 
 /* OssSink signals and args */
 enum {
@@ -72,11 +77,11 @@ enum {
 
 enum {
   ARG_0,
-  ARG_DEVICE,
   ARG_MUTE,
   ARG_FRAGMENT,
   ARG_BUFFER_SIZE,
-  ARG_SYNC
+  ARG_SYNC,
+  ARG_CHUNK_SIZE,
   /* FILL ME */
 };
 
@@ -86,9 +91,7 @@ GST_PAD_TEMPLATE_FACTORY (osssink_sink_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "osssink_sink",
-    "audio/raw",
-      "format",     GST_PROPS_STRING ("int"),   /* hack */
-      "law",        GST_PROPS_INT (0),
+    "audio/x-raw-int",
       "endianness", GST_PROPS_INT (G_BYTE_ORDER),
       "signed",     GST_PROPS_LIST (
          	      GST_PROPS_BOOLEAN (FALSE),
@@ -127,7 +130,7 @@ gst_osssink_get_type (void)
       0,
       (GInstanceInitFunc)gst_osssink_init,
     };
-    osssink_type = g_type_register_static (GST_TYPE_ELEMENT, "GstOssSink", &osssink_info, 0);
+    osssink_type = g_type_register_static (GST_TYPE_OSSELEMENT, "GstOssSink", &osssink_info, 0);
   }
 
   return osssink_type;
@@ -148,13 +151,13 @@ gst_osssink_get_bufferpool (GstPad *pad)
 }
 
 static void
-gst_osssink_finalize (GObject *object)
+gst_osssink_dispose (GObject *object)
 {
   GstOssSink *osssink = (GstOssSink *) object;
 
-  g_free (osssink->common.device);
+  gst_object_unparent (GST_OBJECT (osssink->provided_clock));
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -166,14 +169,11 @@ gst_osssink_class_init (GstOssSinkClass *klass)
   gobject_class = (GObjectClass*)klass;
   gstelement_class = (GstElementClass*)klass;
 
-  parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
+  parent_class = g_type_class_ref(GST_TYPE_OSSELEMENT);
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DEVICE,
-    g_param_spec_string ("device", "Device", "The device to use for output",
-                         "/dev/dsp", G_PARAM_READWRITE)); /* CHECKME! */
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MUTE,
     g_param_spec_boolean ("mute", "Mute", "Mute the audio",
-                          TRUE, G_PARAM_READWRITE)); 
+                          FALSE, G_PARAM_READWRITE)); 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SYNC,
     g_param_spec_boolean ("sync", "Sync", "If syncing on timestamps should be enabled",
                           TRUE, G_PARAM_READWRITE)); 
@@ -182,8 +182,11 @@ gst_osssink_class_init (GstOssSinkClass *klass)
 	    	      "The fragment as 0xMMMMSSSS (MMMM = total fragments, 2^SSSS = fragment size)",
                       0, G_MAXINT, 6, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BUFFER_SIZE,
-    g_param_spec_int ("buffer_size", "Buffer size", "The buffer size",
-                      0, G_MAXINT, 4096, G_PARAM_READWRITE));
+    g_param_spec_uint ("buffer_size", "Buffer size", "Size of buffers in osssink's bufferpool (bytes)",
+                       0, G_MAXINT, 4096, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_CHUNK_SIZE,
+    g_param_spec_uint ("chunk_size", "Chunk size", "Write data in chunk sized buffers",
+                       0, G_MAXUINT, 4096, G_PARAM_READWRITE));
 
   gst_osssink_signals[SIGNAL_HANDOFF] =
     g_signal_new ("handoff", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
@@ -192,7 +195,7 @@ gst_osssink_class_init (GstOssSinkClass *klass)
   
   gobject_class->set_property = gst_osssink_set_property;
   gobject_class->get_property = gst_osssink_get_property;
-  gobject_class->finalize     = gst_osssink_finalize;
+  gobject_class->dispose      = gst_osssink_dispose;
   
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_osssink_change_state);
   gstelement_class->query 	 = GST_DEBUG_FUNCPTR (gst_osssink_query);
@@ -216,13 +219,14 @@ gst_osssink_init (GstOssSink *osssink)
 
   gst_pad_set_chain_function (osssink->sinkpad, gst_osssink_chain);
 
-  gst_osscommon_init (&osssink->common);
-
   osssink->bufsize = 4096;
+  osssink->chunk_size = 4096;
   osssink->resync = FALSE;
+  osssink->mute = FALSE;
   osssink->sync = TRUE;
   osssink->sinkpool = NULL;
-  osssink->provided_clock = GST_CLOCK (gst_oss_clock_new ("ossclock", gst_osssink_get_time, osssink));
+  osssink->provided_clock = gst_audio_clock_new ("ossclock", gst_osssink_get_time, osssink);
+  gst_object_set_parent (GST_OBJECT (osssink->provided_clock), GST_OBJECT (osssink));
   osssink->handled = 0;
 
   GST_FLAG_SET (osssink, GST_ELEMENT_THREAD_SUGGESTED);
@@ -238,10 +242,10 @@ gst_osssink_sinkconnect (GstPad *pad, GstCaps *caps)
   if (!GST_CAPS_IS_FIXED (caps))
     return GST_PAD_LINK_DELAYED;
 
-  if (!gst_osscommon_parse_caps (&osssink->common, caps))
+  if (!gst_osselement_parse_caps (GST_OSSELEMENT (osssink), caps))
     return GST_PAD_LINK_REFUSED;
 
-  if (!gst_osscommon_sync_parms (&osssink->common)) {
+  if (!gst_osselement_sync_parms (GST_OSSELEMENT (osssink))) {
     return GST_PAD_LINK_REFUSED;
   }
 
@@ -253,12 +257,12 @@ gst_osssink_get_delay (GstOssSink *osssink)
 {
   gint delay = 0;
 
-  if (osssink->common.fd == -1)
+  if (GST_OSSELEMENT (osssink)->fd == -1)
     return 0;
 
-  if (ioctl (osssink->common.fd, SNDCTL_DSP_GETODELAY, &delay) < 0) {
+  if (ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_GETODELAY, &delay) < 0) {
     audio_buf_info info;
-    if (ioctl (osssink->common.fd, SNDCTL_DSP_GETOSPACE, &info) < 0) {
+    if (ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_GETOSPACE, &info) < 0) {
       delay = 0;
     }
     else {
@@ -275,7 +279,7 @@ gst_osssink_get_time (GstClock *clock, gpointer data)
   gint delay;
   GstClockTime res;
 
-  if (!osssink->common.bps)
+  if (!GST_OSSELEMENT (osssink)->bps)
     return 0;
 
   delay = gst_osssink_get_delay (osssink);
@@ -286,7 +290,7 @@ gst_osssink_get_time (GstClock *clock, gpointer data)
   if (((guint64)delay) > osssink->handled) {
     delay = osssink->handled;
   }
-  res =  (osssink->handled - delay) * GST_SECOND / osssink->common.bps;
+  res =  (osssink->handled - delay) * GST_SECOND / GST_OSSELEMENT (osssink)->bps;
 
   return res;
 }
@@ -312,8 +316,9 @@ gst_osssink_set_clock (GstElement *element, GstClock *clock)
 }
 
 static void 
-gst_osssink_chain (GstPad *pad, GstBuffer *buf) 
+gst_osssink_chain (GstPad *pad, GstData *_data) 
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstOssSink *osssink;
   GstClockTime buftime;
 
@@ -325,21 +330,18 @@ gst_osssink_chain (GstPad *pad, GstBuffer *buf)
 
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_EOS:
-        ioctl (osssink->common.fd, SNDCTL_DSP_SYNC);
-	gst_oss_clock_set_active (osssink->provided_clock, FALSE);
+        ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_SYNC);
+	gst_audio_clock_set_active (GST_AUDIO_CLOCK (osssink->provided_clock), FALSE);
 	gst_pad_event_default (pad, event);
         return;
-      case GST_EVENT_NEW_MEDIA:
-	g_print ("new media\n");
-        break;
       case GST_EVENT_DISCONTINUOUS:
       {
 	gint64 value;
 
-        ioctl (osssink->common.fd, SNDCTL_DSP_RESET);
+        ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_RESET);
 	if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &value)) {
           if (!gst_clock_handle_discont (osssink->clock, value))
-	    gst_oss_clock_set_active (osssink->provided_clock, FALSE);
+	    gst_audio_clock_set_active (GST_AUDIO_CLOCK (osssink->provided_clock), FALSE);
 	  osssink->handled = 0;
 	}
 	osssink->resync = TRUE;
@@ -353,7 +355,7 @@ gst_osssink_chain (GstPad *pad, GstBuffer *buf)
     return;
   }
 
-  if (!osssink->common.bps) {
+  if (!GST_OSSELEMENT (osssink)->bps) {
     gst_buffer_unref (buf);
     gst_element_error (GST_ELEMENT (osssink), "capsnego was never performed, unknown data type");
     return;
@@ -361,10 +363,11 @@ gst_osssink_chain (GstPad *pad, GstBuffer *buf)
 
   buftime = GST_BUFFER_TIMESTAMP (buf);
 
-  if (osssink->common.fd >= 0) {
+  if (GST_OSSELEMENT (osssink)->fd >= 0) {
     if (!osssink->mute) {
       guchar *data = GST_BUFFER_DATA (buf);
       gint size = GST_BUFFER_SIZE (buf);
+      gint to_write = 0;
 
       if (osssink->clock) {
         gint delay = 0;
@@ -372,7 +375,7 @@ gst_osssink_chain (GstPad *pad, GstBuffer *buf)
 	GstClockTimeDiff jitter;
     
 	delay = gst_osssink_get_delay (osssink);
-	queued = delay * GST_SECOND / osssink->common.bps;
+	queued = delay * GST_SECOND / GST_OSSELEMENT (osssink)->bps;
 
 	if  (osssink->resync && osssink->sync) {
           GstClockID id = gst_clock_new_single_shot_id (osssink->clock, buftime - queued);
@@ -382,29 +385,45 @@ gst_osssink_chain (GstPad *pad, GstBuffer *buf)
 
 	  if (jitter >= 0) {
             gst_clock_handle_discont (osssink->clock, buftime - queued + jitter);
-	    write (osssink->common.fd, data, size);
-	    gst_oss_clock_set_active (osssink->provided_clock, TRUE);
+	    to_write = size;
+	    gst_audio_clock_set_active ((GstAudioClock*)osssink->provided_clock, TRUE);
 	    osssink->resync = FALSE;
-	    osssink->handled += size;
 	  }
 	}
 	else {
-	  write (osssink->common.fd, data, size);
-	  osssink->handled += size;
+	  to_write = size;
         }
       }
       /* no clock, try to be as fast as possible */
       else {
         audio_buf_info ospace;
 
-        ioctl (osssink->common.fd, SNDCTL_DSP_GETOSPACE, &ospace);
+        ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_GETOSPACE, &ospace);
 
         if (ospace.bytes >= size) {
-          write (osssink->common.fd, data, size);
+	  to_write = size;
+	}
+      }
+
+      while (to_write > 0) {
+        gint done = write (GST_OSSELEMENT (osssink)->fd, data, 
+                           MIN (to_write, osssink->chunk_size));
+
+        if (done == -1) {
+          if (errno != EINTR)
+	    break;
+	}
+	else {
+	  to_write -= done;
+	  data += done;
+	  osssink->handled += done;
 	}
       }
     }
   }
+
+  gst_audio_clock_update_time ((GstAudioClock*)osssink->provided_clock, buftime);
+
   gst_buffer_unref (buf);
 }
 
@@ -413,7 +432,7 @@ gst_osssink_get_formats (GstPad *pad)
 {
   static const GstFormat formats[] = {
     GST_FORMAT_TIME,
-    GST_FORMAT_UNITS,
+    GST_FORMAT_DEFAULT,
     GST_FORMAT_BYTES,
     0
   };
@@ -428,8 +447,9 @@ gst_osssink_convert (GstPad *pad, GstFormat src_format, gint64 src_value,
 
   osssink = GST_OSSSINK (gst_pad_get_parent (pad));
   
-  return gst_osscommon_convert (&osssink->common, src_format, src_value,
-		                dest_format, dest_value);
+  return gst_osselement_convert (GST_OSSELEMENT (osssink),
+				 src_format, src_value,
+		                 dest_format, dest_value);
 }
 
 static const GstQueryType*
@@ -492,27 +512,16 @@ gst_osssink_set_property (GObject *object, guint prop_id, const GValue *value, G
   osssink = GST_OSSSINK (object);
 
   switch (prop_id) {
-    case ARG_DEVICE:
-      /* disallow changing the device while it is opened
-         get_property("device") should return the right one */
-      if (!GST_FLAG_IS_SET (osssink, GST_OSSSINK_OPEN))
-      {
-	g_free (osssink->common.device);
-        osssink->common.device = g_strdup (g_value_get_string (value));
-	g_object_notify (object, "device");
-      }
-      break;
     case ARG_MUTE:
       osssink->mute = g_value_get_boolean (value);
       g_object_notify (G_OBJECT (osssink), "mute");
       break;
     case ARG_FRAGMENT:
-      osssink->common.fragment = g_value_get_int (value);
-      gst_osscommon_sync_parms (&osssink->common);
+      GST_OSSELEMENT (osssink)->fragment = g_value_get_int (value);
+      gst_osselement_sync_parms (GST_OSSELEMENT (osssink));
       break;
     case ARG_BUFFER_SIZE:
-      if (osssink->bufsize == g_value_get_int (value)) break;
-      osssink->bufsize = g_value_get_int (value);
+      osssink->bufsize = g_value_get_uint (value);
       osssink->sinkpool = gst_buffer_pool_get_default (osssink->bufsize, 6);
       g_object_notify (object, "buffer_size");
       break;
@@ -520,6 +529,9 @@ gst_osssink_set_property (GObject *object, guint prop_id, const GValue *value, G
       osssink->sync = g_value_get_boolean (value);
       g_object_notify (G_OBJECT (osssink), "sync");
       break;
+    case ARG_CHUNK_SIZE:
+      osssink->chunk_size = g_value_get_uint (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -534,21 +546,21 @@ gst_osssink_get_property (GObject *object, guint prop_id, GValue *value, GParamS
   osssink = GST_OSSSINK (object);
 
   switch (prop_id) {
-    case ARG_DEVICE:
-      g_value_set_string (value, osssink->common.device);
-      break;
     case ARG_MUTE:
       g_value_set_boolean (value, osssink->mute);
       break;
     case ARG_FRAGMENT:
-      g_value_set_int (value, osssink->common.fragment);
+      g_value_set_int (value, GST_OSSELEMENT (osssink)->fragment);
       break;
     case ARG_BUFFER_SIZE:
-      g_value_set_int (value, osssink->bufsize);
+      g_value_set_uint (value, osssink->bufsize);
       break;
     case ARG_SYNC:
       g_value_set_boolean (value, osssink->sync);
       break;
+    case ARG_CHUNK_SIZE:
+      g_value_set_uint (value, osssink->chunk_size);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -563,43 +575,23 @@ gst_osssink_change_state (GstElement *element)
   osssink = GST_OSSSINK (element);
 
   switch (GST_STATE_TRANSITION (element)) {
-    case GST_STATE_NULL_TO_READY:
-      if (!GST_FLAG_IS_SET (element, GST_OSSSINK_OPEN)) {
-	gchar *error;
-
-        if (!gst_osscommon_open_audio (&osssink->common, GST_OSSCOMMON_WRITE, &error)) {
-	  gst_element_error (GST_ELEMENT (osssink), error);
-	  g_free (error);
-          return GST_STATE_FAILURE;
-        }
-	GST_FLAG_SET (element, GST_OSSSINK_OPEN);
-      }
-      break;
     case GST_STATE_READY_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
       osssink->resync = TRUE;
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
-    {
       if (GST_FLAG_IS_SET (element, GST_OSSSINK_OPEN)) 
-        ioctl (osssink->common.fd, SNDCTL_DSP_RESET, 0);
-      gst_oss_clock_set_active (osssink->provided_clock, FALSE);
+        ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_RESET, 0);
+      gst_audio_clock_set_active (GST_AUDIO_CLOCK (osssink->provided_clock), FALSE);
       osssink->resync = TRUE;
       break;
-    }
     case GST_STATE_PAUSED_TO_READY:
       if (GST_FLAG_IS_SET (element, GST_OSSSINK_OPEN))
-        ioctl (osssink->common.fd, SNDCTL_DSP_RESET, 0);
-      gst_osscommon_reset (&osssink->common);
+        ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_RESET, 0);
+      gst_osselement_reset (GST_OSSELEMENT (osssink));
       break;
-    case GST_STATE_READY_TO_NULL:
-      if (GST_FLAG_IS_SET (element, GST_OSSSINK_OPEN)) {
-        gst_osscommon_close_audio (&osssink->common);
-        GST_FLAG_UNSET (osssink, GST_OSSSINK_OPEN);
-
-        GST_INFO (GST_CAT_PLUGIN_INFO, "osssink: closed sound device");
-      }
+    default:
       break;
   }
       
@@ -623,4 +615,3 @@ gst_osssink_factory_init (GstPlugin *plugin)
 
   return TRUE;
 }
-
