@@ -1,5 +1,6 @@
-/* GDK - The GIMP Drawing Kit
- * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+/* GStreamer
+ * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
+ * Copyright (C) <2003> David Schleef <ds@schleef.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,10 +18,17 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/* 
+ * Portions derived from:
+ *
+ * GDK - The GIMP Drawing Kit
+ * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+ */
+
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
-#include <errno.h>
-#include <sys/shm.h> /* for SHM symbols */
 #include <gst/gst.h>
 /* gcc -ansi -pedantic on GNU/Linux causes warnings and errors
  * unless this is defined:
@@ -58,32 +66,25 @@
 #include <X11/extensions/XShm.h>
 #endif /* USE_SHM */
 
-#ifdef HAVE_XVIDEO
 # include <X11/extensions/Xv.h>
 # include <X11/extensions/Xvlib.h>
-#endif
 
 #include "gstxvimage.h"
 
-#ifdef HAVE_XVIDEO
 static GStaticMutex omg = G_STATIC_MUTEX_INIT; /* omg, static vars */
-static int ver, rel, req, ev, err;
-static int formats;
-static int adaptors; 
-static XvAdaptorInfo        *ai;
-static XvImageFormatValues  *fo;
-static int im_adaptor, im_port, im_format;
-#endif
-static GstCaps *capslist = NULL;
+
+static int _gst_xvimage_error_caught;
 
 
 static void 	_gst_xvimage_destroy 		(GstXvImage *image);
 static void 	_gst_xvimage_put		(GstXWindow *window, GstXvImage *image);
+static gboolean _gst_xvimage_try_put            (GstXWindow *window, GstXvImage *image);
 
 gboolean
 _gst_xvimage_check_xvideo(Display *display)
 {
-#ifdef HAVE_XVIDEO
+  int ver, rel, req, ev, err;
+
   g_static_mutex_lock (&omg);
   display = XOpenDisplay(NULL);
 
@@ -96,131 +97,25 @@ _gst_xvimage_check_xvideo(Display *display)
     g_static_mutex_unlock (&omg);
     return TRUE;
   }
-#endif /* HAVE_XVIDEO */
   return FALSE;
 }
 
-GstCaps*
-_gst_xvimage_get_capslist (void)
+static int
+_gst_xvimage_error_handler(Display *d, XErrorEvent *event)
 {
-  return gst_caps_ref (capslist);
-}
-
-void
-_gst_xvimage_init(Display *display)
-{
-#ifdef HAVE_XVIDEO
-  int i;
-    
-  if (display == NULL)
-    return;
-
-  if (!_gst_xvimage_check_xvideo(display)) {
-    g_warning("Xv: Server has no Xvideo extention support\n");
-    return;
-  }
-  g_static_mutex_lock (&omg);
-  im_adaptor = im_port = im_format = -1;
-  if (Success != XvQueryAdaptors(display,DefaultRootWindow(display),&adaptors,&ai)) {
-    g_warning ("Xv: XvQueryAdaptors failed");
-    g_static_mutex_unlock (&omg);
-    return;
-  }
-  GST_INFO(GST_CAT_PLUGIN_INFO, "Xv: %d adaptors available.",adaptors);
-
-  capslist = NULL;
-  for (i = 0; i < adaptors; i++) {
-    GST_INFO(GST_CAT_PLUGIN_INFO, "Xv: %s:%s%s%s%s%s, ports %ld-%ld",
-                     ai[i].name,
-	            (ai[i].type & XvInputMask)  ? " input"  : "",
-		    (ai[i].type & XvOutputMask) ? " output" : "",
-		    (ai[i].type & XvVideoMask)  ? " video"  : "",
-		    (ai[i].type & XvStillMask)  ? " still"  : "",
-		    (ai[i].type & XvImageMask)  ? " image"  : "",
-		    ai[i].base_id,
-		    ai[i].base_id+ai[i].num_ports-1);
-
-    if ((ai[i].type & XvInputMask) &&
-        (ai[i].type & XvImageMask)) 
-    {
-      if (im_port == -1) {
-        GstCaps *caps = NULL;
-	gint j;
-	      
-        im_port = ai[i].base_id;
-        im_adaptor = i;
-
-	{
-	  int count;
-	  const XvAttribute * const attr = XvQueryPortAttributes(display, im_port, &count);
-	  static const char autopaint[] = "XV_AUTOPAINT_COLORKEY";
-
-	  for (j = 0; j < count; ++j)
-	    if (!strcmp(attr[j].name, autopaint))
-	      {
-		const Atom atom = XInternAtom(display, autopaint, False);
-		XvSetPortAttribute(display, im_port, atom, 1);
-		break;
-	      }
-	}
-
-        /* *** image scaler port *** */
-        fo = XvListImageFormats(display, im_port, &formats);
-
-        GST_INFO(GST_CAT_PLUGIN_INFO,"  image format list for port %d",im_port);
-        for(j = 0; j < formats; j++) {
-          gulong fourcc;
-
-          fourcc = GULONG_FROM_LE(fo[j].id);
-
-          GST_INFO(GST_CAT_PLUGIN_INFO, "    0x%x (%4.4s) %s %.32s (%d:%d;%d,%d:%d:%d,%d:%d:%d)",
-                       fo[j].id,
-                       (char*)&fourcc,
-                       (fo[j].format == XvPacked) ? "packed" : "planar",
-		       fo[j].component_order,
-		       fo[j].y_sample_bits,
-		       fo[j].u_sample_bits,
-		       fo[j].v_sample_bits,
-		       fo[j].horz_y_period,
-		       fo[j].horz_u_period,
-		       fo[j].horz_v_period,
-		       fo[j].vert_y_period,
-		       fo[j].vert_u_period,
-		       fo[j].vert_v_period
-			 );
-          caps = gst_caps_new (
-			  "xvideosink_caps",
-		          "video/raw", 
-          		  gst_props_new (
-			    "format",  GST_PROPS_FOURCC (fo[j].id),
-			      "width",   GST_PROPS_INT_RANGE (0, G_MAXINT),
-			      "height",   GST_PROPS_INT_RANGE (0, G_MAXINT),
-			     NULL
-			  ));
-
-  	  capslist = gst_caps_append (capslist, caps);
-        }
-      }
-    }
-  }
-  g_static_mutex_unlock (&omg);
-#else
-  g_warning ("compiled without Xvideo extension support\n");
-#endif
+  _gst_xvimage_error_caught++;
+  return 0;
 }
 
 GstXvImage*
-_gst_xvimage_new (GstXvImageType  type,
+_gst_xvimage_new (GstImageFormat *format,
 		  GstXWindow *window,
 	          gint          width,
 	          gint          height)
 {
-#ifdef HAVE_XVIDEO
 #ifdef USE_SHM
   GstXvImage *image = NULL;
   XShmSegmentInfo *x_shm_info;
-  gint i;
-  gboolean have_xv_scale = FALSE;
 
   g_static_mutex_lock (&omg);
 
@@ -231,35 +126,14 @@ _gst_xvimage_new (GstXvImageType  type,
   GST_IMAGE_PUTFUNC (image) 	= (GstImagePutFunc) _gst_xvimage_put;
   GST_IMAGE_WINDOW (image)      = window;
 
-  image->type = type;
+  image->type = format->im_format;
   image->width = width;
   image->height = height;
 
-  /* *** image scaler port *** */
-  if (im_port == -1) {
-    GST_INFO(GST_CAT_PLUGIN_INFO, "Xv: no usable hw scaler port found");
-    g_static_mutex_unlock (&omg);
-    return NULL;
-  } else {
-    fo = XvListImageFormats(window->disp, im_port, &formats);
-    for(i = 0; i < formats; i++) {
-       if (type == fo[i].id) {
-	 have_xv_scale = 1;
-	 im_format = fo[i].id;
-       }
-    }
-    if (!have_xv_scale) {
-      GST_INFO(GST_CAT_PLUGIN_INFO,"Xv: no usable image format found (port %d)",
-                   im_port);
-      g_static_mutex_unlock (&omg);
-      return NULL;
-    }
-  }
-
   image->x_shm_info = g_new (XShmSegmentInfo, 1);
-  image->im_port = im_port;
+  image->im_port = format->im_port;
   x_shm_info = image->x_shm_info;
-  image->im_format = im_format;
+  image->im_format = format->im_format;
 
   image->ximage = XvShmCreateImage(window->disp, image->im_port, 
      image->im_format, 0, width, height, x_shm_info);
@@ -279,7 +153,7 @@ _gst_xvimage_new (GstXvImageType  type,
 
   if (x_shm_info->shmid == -1)
   {
-    g_warning ("gstxvimage.c: shmget failed: %s (%d)", strerror (errno), errno);
+    g_warning ("shmget failed!");
 
     XFree (image->ximage);
     g_free (image->x_shm_info);
@@ -339,6 +213,14 @@ _gst_xvimage_new (GstXvImageType  type,
     GST_IMAGE_SIZE (image) = image->ximage->data_size;
   }
 
+  if(!_gst_xvimage_try_put(window, image)){
+    g_warning ("XvShmPutImage failed");
+
+    g_free (image);
+    g_static_mutex_unlock (&omg);
+    return NULL;
+  }
+
   g_static_mutex_unlock (&omg);
   _gst_xwindow_ref (window);
   return image;
@@ -346,9 +228,6 @@ _gst_xvimage_new (GstXvImageType  type,
   g_warning ("compiled without shared memory support: Xv image creation not supported");
   return NULL;
 #endif /* USE_SHM */
-#else
-  g_warning ("compiled without Xvideo extention support\n");
-#endif
   return NULL; /* well tell me if it's wrong to do this */
 }
 
@@ -379,11 +258,43 @@ _gst_xvimage_destroy (GstXvImage *image)
   g_free (image);
 }
 
+static gboolean
+_gst_xvimage_try_put (GstXWindow *window,
+		 GstXvImage *image)
+{
+#ifdef USE_SHM
+  XWindowAttributes attr; 
+  void *old_handler;
+
+  g_return_val_if_fail (window != NULL, FALSE);
+  g_return_val_if_fail (image != NULL, FALSE);
+
+  XGetWindowAttributes(window->disp, window->win, &attr); 
+
+  XSync(window->disp,0);
+  _gst_xvimage_error_caught = 0;
+  old_handler = XSetErrorHandler(_gst_xvimage_error_handler);
+  XvShmPutImage (window->disp, image->im_port, window->win,
+		window->gc, image->ximage,
+		0, 0, image->width, image->height, 
+		0, 0, attr.width, attr.height, False);
+  XSync(window->disp, False);
+  XSetErrorHandler(old_handler);
+  if(_gst_xvimage_error_caught>0){
+    return FALSE;
+  }
+
+  return TRUE;
+#else /* USE_SHM */
+  g_error ("trying to draw shared memory image when gst was compiled without shared memory support");
+  return FALSE;
+#endif /* USE_SHM */
+}
+
 static void
 _gst_xvimage_put (GstXWindow *window,
 		 GstXvImage *image)
 {
-#ifdef HAVE_XVIDEO
 #ifdef USE_SHM
   XWindowAttributes attr; 
 
@@ -401,6 +312,132 @@ _gst_xvimage_put (GstXWindow *window,
 #else /* USE_SHM */
   g_error ("trying to draw shared memory image when gst was compiled without shared memory support");
 #endif /* USE_SHM */
-#endif /* HAVE_XVIDEO */
+}
+
+void
+_gst_xvimage_add_formats(GstXWindow *window)
+{
+  int i;
+  int j;
+  int im_adaptor;
+  int im_port;
+  int adaptors;
+  XvAdaptorInfo *ai;
+  XvImageFormatValues *fo;
+  int formats;
+  
+  g_return_if_fail(window);
+
+  if (!_gst_xvimage_check_xvideo(window->disp)) {
+    GST_DEBUG ("Xv: Server has no Xvideo extention support");
+    return;
+  }
+
+  g_static_mutex_lock (&omg);
+  im_adaptor = im_port = -1;
+  if (Success != XvQueryAdaptors(window->disp,DefaultRootWindow(window->disp),
+	&adaptors,&ai)) {
+    g_warning ("Xv: XvQueryAdaptors failed");
+    g_static_mutex_unlock (&omg);
+    return;
+  }
+  GST_DEBUG ( "Xv: %d adaptors available.",adaptors);
+
+  for (i = 0; i < adaptors; i++) {
+    GST_DEBUG ( "Xv: %s:%s%s%s%s%s, ports %ld-%ld",
+                     ai[i].name,
+	            (ai[i].type & XvInputMask)  ? " input"  : "",
+		    (ai[i].type & XvOutputMask) ? " output" : "",
+		    (ai[i].type & XvVideoMask)  ? " video"  : "",
+		    (ai[i].type & XvStillMask)  ? " still"  : "",
+		    (ai[i].type & XvImageMask)  ? " image"  : "",
+		    ai[i].base_id,
+		    ai[i].base_id+ai[i].num_ports-1);
+
+    if (!((ai[i].type & XvInputMask) && (ai[i].type & XvImageMask)))
+      continue;
+
+    if (im_port != -1)
+      continue;
+
+    im_port = ai[i].base_id;
+    im_adaptor = i;
+
+    {
+      int count;
+      const XvAttribute * const attr = XvQueryPortAttributes(window->disp,
+	  im_port, &count);
+
+      for (j = 0; j < count; ++j) {
+	if (strcmp(attr[j].name, "XV_AUTOPAINT_COLORKEY") == 0) {
+	  const Atom atom = XInternAtom(window->disp, "XV_AUTOPAINT_COLORKEY",
+	      False);
+	  XvSetPortAttribute(window->disp, im_port, atom, 1);
+	  break;
+	}
+      }
+    }
+
+    /* *** image scaler port *** */
+    fo = XvListImageFormats(window->disp, im_port, &formats);
+
+    GST_DEBUG ("  image format list for port %d",im_port);
+    for(j = 0; j < formats; j++) {
+      GstImageFormat *image_format;
+
+      image_format = g_new0(GstImageFormat, 1);
+
+      image_format->type = GST_TYPE_XVIMAGE;
+      image_format->im_adaptor = im_adaptor;
+      image_format->im_port = im_port;
+      image_format->im_format = fo[j].id;
+
+      if(fo[j].type == XvRGB){
+      	GST_DEBUG ( "    RGB 0x%x (" GST_FOURCC_FORMAT ") %s (%08x,%08x,%08x)",
+	    fo[j].id,
+	    GST_FOURCC_ARGS(fo[j].id),
+	    (fo[j].format == XvPacked) ? "packed" : "planar",
+	    fo[j].red_mask,
+	    fo[j].green_mask,
+	    fo[j].blue_mask);
+        image_format->caps = GST_CAPS_NEW ( "xvideosink_caps",
+	    "video/x-raw-rgb", 
+	    "endianness", GST_PROPS_INT (G_BYTE_ORDER),
+	    "depth",     GST_PROPS_INT (fo[j].depth),
+	    "bpp",       GST_PROPS_INT (fo[j].bits_per_pixel),
+	    "blue_mask", GST_PROPS_INT (fo[j].red_mask),
+	    "green_mask", GST_PROPS_INT (fo[j].green_mask),
+	    "red_mask",  GST_PROPS_INT (fo[j].blue_mask),
+            "width",     GST_PROPS_INT_RANGE (0, G_MAXINT),
+	    "height",    GST_PROPS_INT_RANGE (0, G_MAXINT),
+	    "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+      }else{
+      	GST_DEBUG ( "    YUV 0x%x (" GST_FOURCC_FORMAT ") %s %.32s (%d:%d;%d,%d:%d:%d,%d:%d:%d)",
+	    fo[j].id,
+	    GST_FOURCC_ARGS(fo[j].id),
+	    (fo[j].format == XvPacked) ? "packed" : "planar",
+	    fo[j].component_order,
+	    fo[j].y_sample_bits,
+	    fo[j].u_sample_bits,
+	    fo[j].v_sample_bits,
+	    fo[j].horz_y_period,
+	    fo[j].horz_u_period,
+	    fo[j].horz_v_period,
+	    fo[j].vert_y_period,
+	    fo[j].vert_u_period,
+	    fo[j].vert_v_period);
+
+        image_format->caps = GST_CAPS_NEW ( "xvideosink_caps",
+	    "video/x-raw-yuv", 
+	    "format",      GST_PROPS_FOURCC (fo[j].id),
+            "width",     GST_PROPS_INT_RANGE (0, G_MAXINT),
+	    "height",    GST_PROPS_INT_RANGE (0, G_MAXINT),
+	    "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+      }
+
+      g_ptr_array_add(window->image_formats, image_format);
+    }
+  }
+  g_static_mutex_unlock (&omg);
 }
 
