@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
+ * Copyright (C) <2003> David Schleef <ds@schleef.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,7 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 
 #include <sys/time.h>
@@ -38,33 +41,21 @@ static GstElementDetails gst_xvideosink_details = {
 };
 
 /* xvideosink signals and args */
-enum {
-  SIGNAL_FRAME_DISPLAYED,
-  SIGNAL_HAVE_SIZE,
-  SIGNAL_HAVE_XID,
-  LAST_SIGNAL
-};
-
 
 enum {
   ARG_0,
-  ARG_XID,
   ARG_SCALE,
-  ARG_FRAMES_DISPLAYED,
-  ARG_FRAME_TIME,
   ARG_DISABLE_XV,
   ARG_TOPLEVEL,
   ARG_AUTOSIZE,
   ARG_NEED_NEW_WINDOW,
 };
-#define FORMAT_IS_RGB(format) ((format) == GST_MAKE_FOURCC ('R','G','B',' '))
 
 static void		gst_xvideosink_class_init	(GstXVideoSinkClass *klass);
 static void		gst_xvideosink_init		(GstXVideoSink *xvideosink);
 static void 		gst_xvideosink_dispose 		(GObject *object);
 
-static void		gst_xvideosink_chain		(GstPad *pad, GstBuffer *buf);
-static void		gst_xvideosink_set_clock	(GstElement *element, GstClock *clock);
+static void		gst_xvideosink_chain		(GstPad *pad, GstData *_data);
 static GstElementStateReturn
 			gst_xvideosink_change_state 	(GstElement *element);
 
@@ -82,24 +73,23 @@ static void		gst_xvideosink_xwindow_free	(GstXVideoSink *xvideosink);
 static gboolean		gst_xvideosink_image_new	(GstXVideoSink *xvideosink);
 static void		gst_xvideosink_image_free	(GstXVideoSink *xvideosink);
 
-static void	 	gst_xvideosink_get_pad_template_caps (GstXVideoSink *xvideosink);
-static void	 	gst_xvideosink_free_pad_template_caps (GstXVideoSink *xvideosink);
-
 /* default template - initiated with class struct to allow gst-register to work
    with X running */
 GST_PAD_TEMPLATE_FACTORY (gst_xvideosink_sink_template_factory,
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "videosink_sink",
-    "video/raw",
-    NULL
-  )
+  GST_CAPS_NEW ( "videosink_rgbsink", "video/x-raw-rgb",
+    "framerate", GST_PROPS_FLOAT_RANGE(0, G_MAXFLOAT),
+    "width", GST_PROPS_INT_RANGE(0, G_MAXINT),
+    "height", GST_PROPS_INT_RANGE(0, G_MAXINT)),
+  GST_CAPS_NEW ( "videosink_yuvsink", "video/x-raw-yuv",
+    "framerate", GST_PROPS_FLOAT_RANGE(0, G_MAXFLOAT),
+    "width", GST_PROPS_INT_RANGE(0, G_MAXINT),
+    "height", GST_PROPS_INT_RANGE(0, G_MAXINT))
 )
 
 static GstElementClass *parent_class = NULL;
-static guint gst_xvideosink_signals[LAST_SIGNAL] = { 0 };
 
 GType
 gst_xvideosink_get_type (void)
@@ -108,16 +98,19 @@ gst_xvideosink_get_type (void)
 
   if (!xvideosink_type) {
     static const GTypeInfo xvideosink_info = {
-      sizeof(GstXVideoSinkClass),      NULL,
-      NULL,
-      (GClassInitFunc)gst_xvideosink_class_init,
+      sizeof (GstXVideoSinkClass),
       NULL,
       NULL,
-      sizeof(GstXVideoSink),
+      (GClassInitFunc) gst_xvideosink_class_init,
+      NULL,
+      NULL,
+      sizeof (GstXVideoSink),
       0,
-      (GInstanceInitFunc)gst_xvideosink_init,
+      (GInstanceInitFunc) gst_xvideosink_init,
     };
-    xvideosink_type = g_type_register_static(GST_TYPE_ELEMENT, "GstXVideoSink", &xvideosink_info, 0);
+    xvideosink_type = g_type_register_static(GST_TYPE_VIDEOSINK,
+                                             "GstXVideoSink",
+                                             &xvideosink_info, 0);
   }
   return xvideosink_type;
 }
@@ -127,21 +120,14 @@ gst_xvideosink_class_init (GstXVideoSinkClass *klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
+  GstVideoSinkClass *gstvs_class;
 
-  gobject_class = (GObjectClass*)klass;
-  gstelement_class = (GstElementClass*)klass;
+  gobject_class = (GObjectClass*) klass;
+  gstelement_class = (GstElementClass*) klass;
+  gstvs_class = (GstVideoSinkClass*) klass;
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_XID,
-    g_param_spec_int ("xid", "Xid", "The Xid of the window",
-                      G_MININT, G_MAXINT, 0, G_PARAM_READABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FRAMES_DISPLAYED,
-    g_param_spec_int ("frames_displayed", "Frames Displayed", "The number of frames displayed so far",
-                      0,G_MAXINT, 0, G_PARAM_READABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FRAME_TIME,
-    g_param_spec_int ("frame_time", "Frame time", "The interval between frames",
-                      G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DISABLE_XV,
     g_param_spec_boolean ("disable_xv", "Disable XV", "Disable Xv images",
                           TRUE, G_PARAM_READWRITE));
@@ -160,26 +146,14 @@ gst_xvideosink_class_init (GstXVideoSinkClass *klass)
   gobject_class->set_property = gst_xvideosink_set_property;
   gobject_class->get_property = gst_xvideosink_get_property;
 
-  gst_xvideosink_signals[SIGNAL_FRAME_DISPLAYED] =
-    g_signal_new ("frame_displayed", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
-                   G_STRUCT_OFFSET (GstXVideoSinkClass, frame_displayed), NULL, NULL,
-                   g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-  gst_xvideosink_signals[SIGNAL_HAVE_SIZE] =
-    g_signal_new ("have_size", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
-                   G_STRUCT_OFFSET (GstXVideoSinkClass, have_size), NULL, NULL,
-                   gst_marshal_VOID__INT_INT, G_TYPE_NONE, 2,
-                   G_TYPE_INT, G_TYPE_INT);
-  gst_xvideosink_signals[SIGNAL_HAVE_XID] =
-    g_signal_new ("have_xid", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
-                   G_STRUCT_OFFSET (GstXVideoSinkClass, have_xid), NULL, NULL,
-                   gst_marshal_VOID__INT, G_TYPE_NONE, 1,
-                   G_TYPE_INT);
-
   gobject_class->dispose = gst_xvideosink_dispose;
   
   gstelement_class->change_state = gst_xvideosink_change_state;
-  gstelement_class->set_clock = gst_xvideosink_set_clock;
   gstelement_class->release_locks = gst_xvideosink_release_locks;
+  
+  /*gstvs_class->set_video_out = gst_xvideosink_set_video_out;
+  gstvs_class->push_ui_event = gst_xvideosink_push_ui_event;
+  gstvs_class->set_geometry = gst_xvideosink_set_geometry;*/
 }
 
 static void
@@ -222,18 +196,26 @@ gst_xvideosink_buffer_new (GstBufferPool *pool,
 
     /* we have to lock the X connection */
     g_mutex_lock (xvideosink->lock);
-    if (FORMAT_IS_RGB (xvideosink->format)) {
+    if (xvideosink->image_format->type == GST_TYPE_XIMAGE) {
+      GST_DEBUG ("creating ximage");
       image = GST_IMAGE (_gst_ximage_new ( 
 		         xvideosink->window, 
-		         xvideosink->width, 
-			 xvideosink->height));
+		         GST_VIDEOSINK_WIDTH (xvideosink),
+                         GST_VIDEOSINK_HEIGHT (xvideosink)));
     }
+#ifdef HAVE_XVIDEO
+    else if (xvideosink->image_format->type == GST_TYPE_XVIMAGE) {
+      GST_DEBUG ("creating xvimage %08x", xvideosink->image_format->im_format);
+      image = GST_IMAGE (_gst_xvimage_new (
+		         xvideosink->image_format,
+		         xvideosink->window,
+		         GST_VIDEOSINK_WIDTH (xvideosink),
+                         GST_VIDEOSINK_HEIGHT (xvideosink)));
+    }
+#endif
     else {
-      image = GST_IMAGE (_gst_xvimage_new ( 
-		         xvideosink->format,
-		         xvideosink->window, 
-		         xvideosink->width, 
-			 xvideosink->height));
+      g_assert_not_reached();
+      return NULL;
     }
     g_mutex_unlock (xvideosink->lock);
   }
@@ -310,12 +292,7 @@ gst_xvideosink_get_bufferpool (GstPad *pad)
   xvideosink = GST_XVIDEOSINK (gst_pad_get_parent (pad));
 
   if (!xvideosink->bufferpool) {
-    if (FORMAT_IS_RGB (xvideosink->format)) {
-      GST_DEBUG (0, "xvideosink: creating RGB XImage bufferpool");
-    }
-    else {
-      GST_DEBUG (0, "xvideosink: creating YUV XvImage bufferpool");
-    }
+    GST_DEBUG ("xvideosink: creating bufferpool");
 
     xvideosink->bufferpool = gst_buffer_pool_new (
 		    NULL,		/* free */
@@ -339,17 +316,16 @@ gst_xvideosink_get_real_size (GstXVideoSink *xvideosink,
 {
   gint pwidth, pheight;
 
-  *real_x = xvideosink->width;
-  *real_y = xvideosink->height;
+  gst_video_sink_get_geometry (GST_VIDEOSINK (xvideosink), real_x, real_y);
   pwidth  = xvideosink->pixel_width;
   pheight = xvideosink->pixel_height;
 
   if (pwidth && pheight) {
     if (pwidth > pheight) {
-      *real_x = (xvideosink->width * pwidth) / pheight;
+      *real_x = (*real_x * pwidth) / pheight;
     }
     else if (pwidth < pheight) {
-      *real_y = (xvideosink->height * pheight) / pwidth;
+      *real_y = (*real_y * pheight) / pwidth;
     }
   }
 }
@@ -358,71 +334,92 @@ static GstPadLinkReturn
 gst_xvideosink_sinkconnect (GstPad *pad, GstCaps *caps)
 {
   GstXVideoSink *xvideosink;
-  gulong print_format;
+  GstCaps *icaps;
+  int i;
+  GstImageFormat *image_format;
 
   xvideosink = GST_XVIDEOSINK (gst_pad_get_parent (pad));
 
   /* we are not going to act on variable caps */
-  if (!GST_CAPS_IS_FIXED_BP (caps))
+  if (!GST_CAPS_IS_FIXED (caps))
     return GST_PAD_LINK_DELAYED;
 
-  for ( ; caps != NULL; caps = caps->next) {
-    gst_caps_get (caps, 
-		  "format", &xvideosink->format,
-		  "width",  &xvideosink->width,
-		  "height", &xvideosink->height,
-		  NULL);
+  if (GST_CAPS_IS_CHAINED (caps))
+    return GST_PAD_LINK_DELAYED;
 
-    if (gst_caps_has_fixed_property (caps, "pixel_width")) 
-      gst_caps_get_int (caps, "pixel_width", &xvideosink->pixel_width);
-    else 
-      xvideosink->pixel_width = 1;
+  GST_DEBUG ("sinkconnect %s", gst_caps_to_string(caps));
 
-    if (gst_caps_has_fixed_property (caps, "pixel_height")) 
-      gst_caps_get_int (caps, "pixel_height", &xvideosink->pixel_height);
-    else 
-      xvideosink->pixel_height = 1;
-
-    print_format = GULONG_FROM_LE (xvideosink->format);
-
-    GST_DEBUG (GST_CAT_PLUGIN_INFO,
-		  "xvideosink: setting %08x (%4.4s) %dx%d\n", 
-		  xvideosink->format, (gchar*)&print_format, 
-		  xvideosink->width, xvideosink->height);
-
-    g_mutex_lock (xvideosink->lock);
-    if (!gst_xvideosink_image_new (xvideosink)) {
-      g_mutex_unlock (xvideosink->lock);
-      xvideosink->format = 0;
+  image_format = NULL;
+  icaps = NULL;
+  for(i=0;i<xvideosink->window->image_formats->len;i++){
+    image_format = g_ptr_array_index(xvideosink->window->image_formats, i);
+    if(image_format->type == GST_TYPE_XVIMAGE && xvideosink->disable_xv){
       continue;
     }
 
-    if (xvideosink->image == NULL) {
-      /* FIXME: need better error handling? */
-      gst_element_error (GST_ELEMENT (xvideosink), "image creation failed");
-      g_mutex_unlock (xvideosink->lock);
-      xvideosink->format = 0;
-      continue;
+    icaps = gst_caps_intersect (caps, image_format->caps);
+
+    if(icaps != NULL){
+      break;
     }
-
-    g_mutex_unlock (xvideosink->lock);
-
-    do {
-      gint real_x, real_y;
-
-      gst_xvideosink_get_real_size (xvideosink, &real_x, &real_y);
-    
-      if (xvideosink->auto_size) {
-        _gst_xwindow_resize (xvideosink->window, real_x, real_y);
-      }
-      g_signal_emit (G_OBJECT (xvideosink), gst_xvideosink_signals[SIGNAL_HAVE_SIZE], 0,
-		     real_x, real_y);
-    } while (0);
-
-    return GST_PAD_LINK_OK;
   }
 
-  return GST_PAD_LINK_REFUSED;
+  if(icaps == NULL){
+    GST_DEBUG ("no format found");
+    return GST_PAD_LINK_REFUSED;
+  }
+
+  gst_caps_unref(icaps);
+
+  GST_DEBUG ("using format %d",i);
+  xvideosink->image_format = image_format;
+
+  gst_caps_get_int (caps, "width",
+                    &(GST_VIDEOSINK_WIDTH (xvideosink)));
+  gst_caps_get_int (caps, "height",
+                    &(GST_VIDEOSINK_HEIGHT (xvideosink)));
+
+  if (gst_caps_has_fixed_property (caps, "pixel_width")) {
+    gst_caps_get_int (caps, "pixel_width", &xvideosink->pixel_width);
+  } else {
+    xvideosink->pixel_width = 1;
+  }
+
+  if (gst_caps_has_fixed_property (caps, "pixel_height")) {
+    gst_caps_get_int (caps, "pixel_height", &xvideosink->pixel_height);
+  } else {
+    xvideosink->pixel_height = 1;
+  }
+
+  g_mutex_lock (xvideosink->lock);
+  if (!gst_xvideosink_image_new (xvideosink)) {
+    g_mutex_unlock (xvideosink->lock);
+    xvideosink->image_format = NULL;
+    return GST_PAD_LINK_REFUSED;
+  }
+
+  if (xvideosink->image == NULL) {
+    /* FIXME: need better error handling? */
+    g_mutex_unlock (xvideosink->lock);
+    gst_element_error (GST_ELEMENT (xvideosink), "image creation failed");
+    xvideosink->image_format = NULL;
+    return GST_PAD_LINK_REFUSED;
+  }
+
+  g_mutex_unlock (xvideosink->lock);
+
+  {
+    gint real_x, real_y;
+
+    gst_xvideosink_get_real_size (xvideosink, &real_x, &real_y);
+  
+    if (xvideosink->auto_size) {
+      _gst_xwindow_resize (xvideosink->window, real_x, real_y);
+    }
+    gst_video_sink_got_video_size (GST_VIDEOSINK (xvideosink), real_x, real_y);
+  }
+
+  return GST_PAD_LINK_OK;
 }
 
 static GstCaps *
@@ -432,45 +429,64 @@ gst_xvideosink_getcaps (GstPad *pad, GstCaps *caps)
 
   xvideosink = GST_XVIDEOSINK (gst_pad_get_parent (pad));
 
-  if (!xvideosink->formats) {
-    gst_xvideosink_get_pad_template_caps (xvideosink);
+  if (xvideosink->window) {
+    int i;
+    GPtrArray *image_formats;
+    GstImageFormat *image_format;
+
+    caps = NULL;
+    image_formats = xvideosink->window->image_formats;
+    for(i=0;i<image_formats->len;i++){
+      image_format = g_ptr_array_index(image_formats, i);
+      caps = gst_caps_append(caps, gst_caps_copy(image_format->caps));
+    }
+  }else{
+    caps = gst_caps_append(
+	GST_CAPS_NEW ( "videosink_rgbsink", "video/x-raw-rgb",
+	  "framerate", GST_PROPS_FLOAT_RANGE(0, G_MAXFLOAT),
+	  "width", GST_PROPS_INT_RANGE(0, G_MAXINT),
+	  "height", GST_PROPS_INT_RANGE(0, G_MAXINT)),
+	GST_CAPS_NEW ( "videosink_yuvsink", "video/x-raw-yuv",
+	  "framerate", GST_PROPS_FLOAT_RANGE(0, G_MAXFLOAT),
+	  "width", GST_PROPS_INT_RANGE(0, G_MAXINT),
+	  "height", GST_PROPS_INT_RANGE(0, G_MAXINT)));
   }
 
-  return gst_caps_ref (xvideosink->formats);
+  GST_DEBUG ("getcaps returning %s",gst_caps_to_string(caps));
+
+  return caps;
 }
 
 static void
 gst_xvideosink_init (GstXVideoSink *xvideosink)
 {
-  xvideosink->sinkpad = gst_pad_new_from_template (
-                             GST_PAD_TEMPLATE_GET (gst_xvideosink_sink_template_factory), 
-                             "sink");
-  gst_element_add_pad (GST_ELEMENT (xvideosink), xvideosink->sinkpad);
-  gst_pad_set_chain_function (xvideosink->sinkpad, gst_xvideosink_chain);
-  gst_pad_set_link_function (xvideosink->sinkpad, gst_xvideosink_sinkconnect);
-  gst_pad_set_getcaps_function (xvideosink->sinkpad, gst_xvideosink_getcaps);
-  gst_pad_set_bufferpool_function (xvideosink->sinkpad, gst_xvideosink_get_bufferpool);
+  GST_VIDEOSINK_PAD (xvideosink) = gst_pad_new_from_template (
+                    GST_PAD_TEMPLATE_GET (gst_xvideosink_sink_template_factory), 
+                    "sink");
+  gst_element_add_pad (GST_ELEMENT (xvideosink),
+                       GST_VIDEOSINK_PAD (xvideosink));
+  gst_pad_set_chain_function (GST_VIDEOSINK_PAD (xvideosink),
+                              gst_xvideosink_chain);
+  gst_pad_set_link_function (GST_VIDEOSINK_PAD (xvideosink),
+                             gst_xvideosink_sinkconnect);
+  gst_pad_set_getcaps_function (GST_VIDEOSINK_PAD (xvideosink),
+                                gst_xvideosink_getcaps);
+  gst_pad_set_bufferpool_function (GST_VIDEOSINK_PAD (xvideosink),
+                                   gst_xvideosink_get_bufferpool);
 
   /* one-time initialization */
   xvideosink->window = NULL;
   xvideosink->toplevel = TRUE;
-  xvideosink->format = 0;
-  xvideosink->formats = NULL;
+  xvideosink->image_format = NULL;
   xvideosink->lock = g_mutex_new ();
 
   xvideosink->image_pool = NULL;
   xvideosink->pool_lock = g_mutex_new ();
-  xvideosink->clock = NULL;
   xvideosink->send_xid = FALSE;
   xvideosink->need_new_window = FALSE;
 
   xvideosink->image = NULL;
 
-  xvideosink->width = 100;
-  xvideosink->height = 100;
-  xvideosink->disable_xv = FALSE;
-  xvideosink->auto_size = TRUE;
-
   gst_xvideosink_reset (xvideosink);
   
   GST_FLAG_SET(xvideosink, GST_ELEMENT_THREAD_SUGGESTED);
@@ -480,18 +496,11 @@ gst_xvideosink_init (GstXVideoSink *xvideosink)
 static void
 gst_xvideosink_reset (GstXVideoSink *xvideosink)
 {
+  GST_VIDEOSINK_WIDTH (xvideosink) = 100;
+  GST_VIDEOSINK_HEIGHT (xvideosink) = 100;
+  xvideosink->disable_xv = FALSE;
+  xvideosink->auto_size = TRUE;
   xvideosink->correction = 0;
-  xvideosink->frames_displayed = 0;
-}
-
-static void
-gst_xvideosink_set_clock (GstElement *element, GstClock *clock)
-{
-  GstXVideoSink *xvideosink;
-
-  xvideosink = GST_XVIDEOSINK (element);
-  
-  xvideosink->clock = clock;
 }
 
 static void
@@ -515,8 +524,9 @@ gst_xvideosink_dispose (GObject *object)
 }
 
 static void
-gst_xvideosink_chain (GstPad *pad, GstBuffer *buf)
+gst_xvideosink_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstXVideoSink *xvideosink;
   GstClockTime time = GST_BUFFER_TIMESTAMP (buf);
   gint64 jitter;
@@ -534,8 +544,9 @@ gst_xvideosink_chain (GstPad *pad, GstBuffer *buf)
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_DISCONTINUOUS:
 	offset = GST_EVENT_DISCONT_OFFSET (event, 0).value;
-	g_print ("xvideo discont %" G_GINT64_FORMAT "\n", offset);
-	gst_clock_handle_discont (xvideosink->clock, (guint64) offset);
+	GST_DEBUG ("xvideo discont %" G_GINT64_FORMAT, offset);
+	gst_clock_handle_discont (GST_VIDEOSINK_CLOCK (xvideosink),
+                                  (guint64) offset);
 	break;
       default:
 	gst_pad_event_default (pad, event);
@@ -545,20 +556,22 @@ gst_xvideosink_chain (GstPad *pad, GstBuffer *buf)
     return;
   }
 
-  if (xvideosink->clock && time != -1) {
+  if (GST_VIDEOSINK_CLOCK (xvideosink) && time != -1) {
     GstClockReturn ret;
 
-    xvideosink->id = gst_clock_new_single_shot_id (xvideosink->clock, time);
+    xvideosink->id = gst_clock_new_single_shot_id (
+                       GST_VIDEOSINK_CLOCK (xvideosink), time);
 
-    GST_DEBUG (0,"videosink: clock %s wait: %" G_GUINT64_FORMAT " %u\n", 
-		  GST_OBJECT_NAME (xvideosink->clock), time, GST_BUFFER_SIZE (buf));
+    GST_DEBUG ("videosink: clock %s wait: %" G_GUINT64_FORMAT " %u", 
+               GST_OBJECT_NAME (GST_VIDEOSINK_CLOCK (xvideosink)),
+               time, GST_BUFFER_SIZE (buf));
 
     ret = gst_clock_id_wait (xvideosink->id, &jitter);
     gst_clock_id_free (xvideosink->id);
     xvideosink->id = NULL;
   }
-  if (xvideosink->clock)
-    time = gst_clock_get_time (xvideosink->clock);
+  if (GST_VIDEOSINK_CLOCK (xvideosink))
+    time = gst_clock_get_time (GST_VIDEOSINK_CLOCK (xvideosink));
 
   g_mutex_lock (xvideosink->lock);
   /* if we have a pool and the image is from this pool, simply _put it */
@@ -581,23 +594,23 @@ gst_xvideosink_chain (GstPad *pad, GstBuffer *buf)
       return;
     }
   }
-  xvideosink->frames_displayed++;
+
+  gst_video_sink_frame_displayed (GST_VIDEOSINK (xvideosink));
   g_mutex_unlock (xvideosink->lock);
 
-  if (xvideosink->clock) {
-    jitter = gst_clock_get_time (xvideosink->clock) - time;
+  if (GST_VIDEOSINK_CLOCK (xvideosink)) {
+    jitter = gst_clock_get_time (GST_VIDEOSINK_CLOCK (xvideosink)) - time;
 
     xvideosink->correction = (xvideosink->correction + jitter) >> 1;
     xvideosink->correction = 0;
   }
 
   if (xvideosink->send_xid){
-    g_signal_emit (G_OBJECT (xvideosink), gst_xvideosink_signals[SIGNAL_HAVE_XID], 
-		    0, GST_XWINDOW_XID (xvideosink->window));
+    gst_video_sink_got_video_out (
+      GST_VIDEOSINK (xvideosink),
+      (gpointer) GST_XWINDOW_XID (xvideosink->window));
     xvideosink->send_xid = FALSE;
   }
-  g_signal_emit (G_OBJECT (xvideosink), gst_xvideosink_signals[SIGNAL_FRAME_DISPLAYED], 0);
-
 
   gst_buffer_unref (buf);
 }
@@ -615,18 +628,8 @@ gst_xvideosink_set_property (GObject *object, guint prop_id,
   xvideosink = GST_XVIDEOSINK (object);
 
   switch (prop_id) {
-    case ARG_FRAMES_DISPLAYED:
-      xvideosink->frames_displayed = g_value_get_int (value);
-      break;
-    case ARG_FRAME_TIME:
-      xvideosink->frame_time = g_value_get_int (value);
-      break;
     case ARG_DISABLE_XV:
       xvideosink->disable_xv = g_value_get_boolean (value);
-      if (xvideosink->formats) /* get rid of old (possible now invalid) list */
-        gst_xvideosink_free_pad_template_caps (xvideosink);
-      if (gst_element_get_state (GST_ELEMENT (xvideosink)) != GST_STATE_NULL)
-        gst_xvideosink_get_pad_template_caps (xvideosink);
       break;
     case ARG_TOPLEVEL:
       xvideosink->toplevel = g_value_get_boolean (value);
@@ -639,11 +642,7 @@ gst_xvideosink_set_property (GObject *object, guint prop_id,
         if (xvideosink->window) {
           g_mutex_lock (xvideosink->lock);
           gst_xvideosink_xwindow_free (xvideosink);
-          if (gst_xvideosink_xwindow_new (xvideosink)) {
-            g_signal_emit (G_OBJECT (xvideosink), 
-		           gst_xvideosink_signals[SIGNAL_HAVE_SIZE], 0,
-		           xvideosink->width, xvideosink->height);
-          }
+          gst_xvideosink_xwindow_new (xvideosink);
           g_mutex_unlock (xvideosink->lock);
 	} else {
 	  xvideosink->need_new_window = TRUE;
@@ -664,21 +663,6 @@ gst_xvideosink_get_property (GObject *object, guint prop_id, GValue *value, GPar
   xvideosink = GST_XVIDEOSINK(object);
 
   switch (prop_id) {
-    case ARG_XID: {
-      if (xvideosink->window)
-        g_value_set_int (value, GST_XWINDOW_XID (xvideosink->window));
-      else
-        g_value_set_int (value, 0);
-      break;
-    }
-    case ARG_FRAMES_DISPLAYED: {
-      g_value_set_int (value, xvideosink->frames_displayed);
-      break;
-    }
-    case ARG_FRAME_TIME: {
-      g_value_set_int (value, xvideosink->frame_time / GST_SECOND);
-      break;
-    }
     case ARG_DISABLE_XV:
       g_value_set_boolean (value, xvideosink->disable_xv);
       break;
@@ -695,58 +679,6 @@ gst_xvideosink_get_property (GObject *object, guint prop_id, GValue *value, GPar
   }
 }
 
-static void
-gst_xvideosink_free_pad_template_caps (GstXVideoSink *xvideosink)
-{
-  g_assert (xvideosink->formats != NULL);
-  
-  gst_caps_unref (xvideosink->formats);
-  xvideosink->formats = NULL;
-}
-static void
-gst_xvideosink_get_pad_template_caps (GstXVideoSink *xvideosink)
-{
-  GstXWindow *window;
-  GstXImage *ximage;
-  GstCaps *caps = NULL;
-
-  g_assert (xvideosink->formats == NULL);
-
-  g_return_if_fail ((window = _gst_xwindow_new (100, 100, FALSE)));
-  
-  ximage = _gst_ximage_new (window, 100, 100);
-  if (ximage) {
-    caps = GST_CAPS_NEW (
-	     "xvideosink_caps",
-	     "video/raw",
-	      "format",       GST_PROPS_FOURCC (GST_MAKE_FOURCC ('R', 'G', 'B', ' ')),
-	        "bpp",        GST_PROPS_INT (GST_XIMAGE_BPP (ximage)),
-	        "depth",      GST_PROPS_INT (GST_XIMAGE_DEPTH (ximage)),
-	        "endianness", GST_PROPS_INT (GST_XIMAGE_ENDIANNESS (ximage)),
-	        "red_mask",   GST_PROPS_INT (GST_XIMAGE_RED_MASK (ximage)),
-	        "green_mask", GST_PROPS_INT (GST_XIMAGE_GREEN_MASK (ximage)),
-	        "blue_mask",  GST_PROPS_INT (GST_XIMAGE_BLUE_MASK (ximage)),
-	        "width",      GST_PROPS_INT_RANGE (0, G_MAXINT),
-	        "height",     GST_PROPS_INT_RANGE (0, G_MAXINT)
-	    );
-
-    _gst_image_destroy (GST_IMAGE (ximage));
-  }
-
-  if (!xvideosink->disable_xv && _gst_xvimage_check_xvideo (window->disp)) {
-    _gst_xvimage_init(window->disp);
-
-    caps = gst_caps_prepend (caps, _gst_xvimage_get_capslist ());
-  }
-
-  _gst_xwindow_destroy (window);
-
-  gst_caps_ref (caps);
-  gst_caps_sink (caps);
-
-  xvideosink->formats = caps;
-}
-
 static GstElementStateReturn
 gst_xvideosink_change_state (GstElement *element)
 {
@@ -756,8 +688,6 @@ gst_xvideosink_change_state (GstElement *element)
 
   switch (GST_STATE_TRANSITION (element)) {
     case GST_STATE_NULL_TO_READY:
-      if (!xvideosink->formats) /* _getcaps() might already have filled this */
-        gst_xvideosink_get_pad_template_caps (xvideosink);
       if (!gst_xvideosink_xwindow_new (xvideosink))
 	return GST_STATE_FAILURE;
       break;
@@ -769,11 +699,10 @@ gst_xvideosink_change_state (GstElement *element)
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
-      gst_xvideosink_reset (xvideosink);
       break;
     case GST_STATE_READY_TO_NULL:
       gst_xvideosink_xwindow_free (xvideosink);
-      gst_xvideosink_free_pad_template_caps (xvideosink);
+      gst_xvideosink_reset (xvideosink);
       break;
   }
 
@@ -799,22 +728,28 @@ gst_xvideosink_release_locks (GstElement *element)
 static gboolean
 gst_xvideosink_xwindow_new (GstXVideoSink *xvideosink)
 {
-  xvideosink->window = _gst_xwindow_new (xvideosink->width, 
-	                                 xvideosink->height, 
+  xvideosink->window = _gst_xwindow_new (GST_VIDEOSINK_WIDTH (xvideosink), 
+	                                 GST_VIDEOSINK_HEIGHT (xvideosink), 
 					 xvideosink->toplevel);
   if (!xvideosink->window) {
     gst_element_error (GST_ELEMENT (xvideosink), "could not create X window");
     return FALSE;
   }
+  
+  gst_video_sink_got_video_size (GST_VIDEOSINK (xvideosink),
+                                 GST_VIDEOSINK_WIDTH (xvideosink),
+                                 GST_VIDEOSINK_HEIGHT (xvideosink));
+  
   xvideosink->send_xid = TRUE;
   xvideosink->need_new_window = FALSE;
 
-  if (!xvideosink->format) {
+  if (!xvideosink->image_format) {
     return TRUE;
   } else {
     return gst_xvideosink_image_new (xvideosink);
   }
 }
+
 static void
 gst_xvideosink_xwindow_free (GstXVideoSink *xvideosink)
 {
@@ -829,30 +764,36 @@ gst_xvideosink_xwindow_free (GstXVideoSink *xvideosink)
     xvideosink->window = NULL;
   }
 }
+
 static gboolean
 gst_xvideosink_image_new (GstXVideoSink *xvideosink)
 {
-  if (!xvideosink->format)
+  if (!xvideosink->image_format)
     return FALSE;
   
   if (xvideosink->image)
     gst_xvideosink_image_free (xvideosink);
 
   /* create a new image to draw onto */
-  if (FORMAT_IS_RGB (xvideosink->format)) {
+  if (xvideosink->image_format->type == GST_TYPE_XIMAGE) {
     xvideosink->image = GST_IMAGE (_gst_ximage_new ( 
 		      xvideosink->window, 
-		      xvideosink->width, xvideosink->height));
-  } else if (xvideosink->disable_xv) {
-    return FALSE;
-  } else {
-    xvideosink->image = GST_IMAGE (_gst_xvimage_new ( 
-		      xvideosink->format,
-		      xvideosink->window, 
-		      xvideosink->width, xvideosink->height));
+		      GST_VIDEOSINK_WIDTH (xvideosink),
+                      GST_VIDEOSINK_HEIGHT (xvideosink)));
+    return TRUE;
   }
-
-  return TRUE;
+#ifdef HAVE_XVIDEO
+  if (xvideosink->image_format->type == GST_TYPE_XVIMAGE && 
+      !xvideosink->disable_xv) {
+    xvideosink->image = GST_IMAGE (_gst_xvimage_new ( 
+		      xvideosink->image_format,
+		      xvideosink->window, 
+		      GST_VIDEOSINK_WIDTH (xvideosink),
+                      GST_VIDEOSINK_HEIGHT (xvideosink)));
+    return TRUE;
+  }
+#endif
+  return FALSE;
 }
 static void
 gst_xvideosink_image_free (GstXVideoSink *xvideosink)
@@ -870,13 +811,18 @@ plugin_init (GModule *module, GstPlugin *plugin)
 {
   GstElementFactory *factory;
 
+  /* Loading the library containing GstVideoSink, our parent object */
+  if (!gst_library_load ("gstvideo"))
+    return FALSE;
+  
   /* create an elementfactory for the xvideosink element */
-  factory = gst_element_factory_new("xvideosink",GST_TYPE_XVIDEOSINK,
-                                   &gst_xvideosink_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
+  factory = gst_element_factory_new ("xvideosink", GST_TYPE_XVIDEOSINK,
+                                     &gst_xvideosink_details);
+  g_return_val_if_fail (factory != NULL, FALSE);
 
-  gst_element_factory_add_pad_template (factory, 
-                                        GST_PAD_TEMPLATE_GET (gst_xvideosink_sink_template_factory));
+  gst_element_factory_add_pad_template (
+    factory, 
+    GST_PAD_TEMPLATE_GET (gst_xvideosink_sink_template_factory));
 
   gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
 
